/*---------------------------------------------------------------------------

	CmCoEquatorial.h Header for equatorial coordinate computig object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include antirepeat
#ifndef CMCoEquatorialH
#define CMCoEquatorialH



// project include
#include "CMHours.h"
#include "CMDegree.h"



class CMCoHorizontal;
class CMCoEcliptic;
class CMCoLocal;

class CMCoEquatorial
{
   public:
      // object constructor
                                       CMCoEquatorial();
      // copy object constructor
                                       CMCoEquatorial( const CMCoEquatorial *Copy );
      // destroyer
                                       ~CMCoEquatorial();
      // get object righ asension
      CMHours* const                   GetRA( void ) const;
      // get object declination
      CMDegree* const                  GetDec( void )const ;
      // set equinoz epoch
      void                             SetDataEqu( double gg );
      // get ecliptic coordinate
      CMCoEcliptic*                    GetCoEcl( double gg, bool Apparent ) ;
      // get horizontal coordinate
      CMCoHorizontal*                  GetCoOri( double gg, CMCoLocal *Site, bool Apparent ) const;
      // get topocentric equatorial coordinate
      CMCoEquatorial*                  GetCoTopo( double gg, CMCoLocal *Site ) const;
      // compute coordinate changing equinoz
      CMCoEquatorial*                  ChangeEqu( double Newgg ) const;
      // set distance
      void                             SetDist( double Dist );
      // get distance
      double                           GetDist( void ) const;
   private:
      // right ascension
      CMHours                          *RA;
      // declination
      CMDegree                         *Dec;
      // equinoz epoch
      double                           gg;
      // distance
      double                           Dist;
};

#endif
