/*---------------------------------------------------------------------------

	CMCoLocal.cpp  Local coordinate computig object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <math.h>



// project include
#include "CMCoLocal.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMCoLocal::CMCoLocal()
{
   Longitude = new CMHours( );
   Latitude = new CMDegree( );
   Elevation = 0;
}



/*---------------------------------------------------------------------------

	copy object constructor

---------------------------------------------------------------------------*/

CMCoLocal::CMCoLocal( const CMCoLocal *Copy )
{
   CMHours::HOURS                      TempAr;              // temp objects
   CMDegree::DEGREE                    TempDec;

   Longitude = new CMHours( );
   Latitude = new CMDegree( );

   TempAr = *Copy->GetLongitude()->Get();
   Longitude->Set( &TempAr );
   TempDec = *Copy->GetLatitude()->Get( );
   Latitude->Set( &TempDec );

   Elevation = Copy->Elevation;
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMCoLocal::~CMCoLocal()
{
   delete Longitude;
   delete Latitude;
}



/*---------------------------------------------------------------------------

	get longitude

---------------------------------------------------------------------------*/

CMHours* const CMCoLocal::GetLongitude( void ) const
{
   return Longitude;
}



/*---------------------------------------------------------------------------

	get latitude

---------------------------------------------------------------------------*/

CMDegree* const CMCoLocal::GetLatitude( void ) const
{
   return Latitude;
}



/*---------------------------------------------------------------------------

	get elevation

---------------------------------------------------------------------------*/

double CMCoLocal::GetElevation( void ) const
{
   return Elevation;
}



/*---------------------------------------------------------------------------

	set elevation

---------------------------------------------------------------------------*/

void CMCoLocal::SetElevation( double ParElevation )
{
   Elevation = ParElevation;
}

