/*---------------------------------------------------------------------------

	CMCoRectagolar.cpp  Rectangolar coordinate computing object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include di sistema
#include <string.h>
#include <math.h>



// include miei
#include "CMCoRectangolar.h"
#include "CMCoEcliptic.h"



// definizioni
#define  RAD            (180/M_PI)




/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMCoRectangolar::CMCoRectangolar()
{
   Coo.X = 0;
   Coo.Y = 0;
   Coo.Z = 1;
}



/*---------------------------------------------------------------------------

	copy object constructor

---------------------------------------------------------------------------*/

CMCoRectangolar::CMCoRectangolar( CMCoRectangolar *Copy )
{
   Coo = Copy->Coo;
}



/*---------------------------------------------------------------------------

	set coordinate

---------------------------------------------------------------------------*/

int CMCoRectangolar::Set( RET_COO* ParCoo )
{
   Coo = *ParCoo;
   return 0;
}



/*---------------------------------------------------------------------------

	get coordinate

---------------------------------------------------------------------------*/

const CMCoRectangolar::RET_COO* CMCoRectangolar::Get( void )
{
   return &Coo;
}



/*---------------------------------------------------------------------------

	subtract rectangolar coordinate

---------------------------------------------------------------------------*/

void CMCoRectangolar::Sub( CMCoRectangolar *SubEle )
{
   Coo.X -= SubEle->Coo.X;
   Coo.Y -= SubEle->Coo.Y;
   Coo.Z -= SubEle->Coo.Z;
}



/*---------------------------------------------------------------------------

	add rectangolar coordinate

---------------------------------------------------------------------------*/

void CMCoRectangolar::Add( CMCoRectangolar *AddEle )
{
   Coo.X += AddEle->Coo.X;
   Coo.Y += AddEle->Coo.Y;
   Coo.Z += AddEle->Coo.Z;
}



/*---------------------------------------------------------------------------

	get ecliptic coordinate

---------------------------------------------------------------------------*/

CMCoEcliptic* CMCoRectangolar::GetCoEcl( void )
{
   CMCoEcliptic*                       Obj;                 // ecliptic coordinate object
   CMCoEcliptic::ECL_COO               Ecl;                 // ecliptic coordinate value
   double                              c;                   // temp

   // create object
   Obj = new CMCoEcliptic();

   // compute distance, longigude e latitude
   Ecl.d = sqrt( pow(Coo.X, 2) + pow( Coo.Y, 2) + pow (Coo.Z, 2) );
   c = atan ( Coo.Y / Coo.X) * RAD;
   if ( Coo.X < 0 )
      c += 180;
   if ( c < 0 )
      c += 360;
   Ecl.l = c;
   Ecl.b = atan ( Coo.Z / sqrt( pow(Coo.X, 2) + pow (Coo.Y, 2) ) ) * RAD;

   // set object
   Obj->Set(&Ecl);

   return Obj;
}



/*---------------------------------------------------------------------------

	get distance

---------------------------------------------------------------------------*/

double CMCoRectangolar::GetDist( void )
{
   double               d;

   d = pow( Coo.X, 2) + pow( Coo.Y, 2 ) + pow( Coo.Z, 2 );
   d = pow( d, 0.5);
   return d;
}

