/*---------------------------------------------------------------------------

	CMDeepSky.cpp Deep sky object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : November 2012

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>



// project include
#include <stdio.h>
#include "CmDeepSky.h"




/*---------------------------------------------------------------------------

	constructor

---------------------------------------------------------------------------*/

CMDeepSky::CMDeepSky() :
CMObject()
{
   BaseDate = 2451545;                                      // J2000
   BasePosition = 0;
   Date = 2451545;                                          // J2000
}



/*---------------------------------------------------------------------------

	copy constructor

---------------------------------------------------------------------------*/

CMDeepSky::CMDeepSky( CMDeepSky *Copy ) :
CMObject( Copy )
{
   BaseDate = Copy->BaseDate;
   BasePosition = Copy->BasePosition;
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMDeepSky::~CMDeepSky(  )
{
   if ( BasePosition != 0 )
      delete BasePosition;
}



/*---------------------------------------------------------------------------

	set base star position

---------------------------------------------------------------------------*/

void CMDeepSky::SetBase( double ParDataBase, CMCoEquatorial* ParPosizioneBase )
{
   if ( BasePosition != 0 )
      delete BasePosition;

   BaseDate = ParDataBase;
   BasePosition = new CMCoEquatorial ( ParPosizioneBase );
}



/*---------------------------------------------------------------------------

	get information

---------------------------------------------------------------------------*/

void CMDeepSky::GetInfo( char *PInfo, int Size )
{
   char                                Message[100];        // message for info adding
   int                                 Len;                 // string len

   // object type decoding
   strcpy( Message, "Oggetto non definito\r\n");
   if ( ObjectType == CMDeepSky::GALAXY )
      strcpy( Message, "Galassia\r\n");
   if ( ObjectType == CMDeepSky::OPENCLASTER )
      strcpy( Message, "Ammasso aperto\r\n");
   if ( ObjectType == CMDeepSky::GLOBULARCLUSTER )
      strcpy( Message, "Ammasso globulare\r\n");
   if ( ObjectType == CMDeepSky::NEBULA )
      strcpy( Message, "Nebulosa in emissione o rilessione\r\n");
   if ( ObjectType == CMDeepSky::PLANETARYNEBULLA )
      strcpy( Message, "Nebulosa planetaria\r\n");
   if ( ObjectType == CMDeepSky::NEBULACLUSTER )
      strcpy( Message, "Ammasso con nebulosa\r\n");
   if ( ObjectType == CMDeepSky::ASTERISM )
      strcpy( Message, "Asterismo\r\n");
   if ( ObjectType == CMDeepSky::KNOT )
      strcpy( Message, "Grovigio o nubolosa extragalattica\r\n");
   if ( ObjectType == CMDeepSky::TRIPLESTAR )
      strcpy( Message, "stella tripla\r\n");
   if ( ObjectType == CMDeepSky::DOUBLESTAR )
      strcpy( Message, "Stella doppia\r\n");
   if ( ObjectType == CMDeepSky::SINGLESTAR )
      strcpy( Message, "Stella singola\r\n");
   Len = strlen( Message);                                  // compute message lenght
   if ( Size >= Len )                                       // is there enought space ?
   {
      strcpy( PInfo, Message );                             // copy message
      Size -= Len;                                          // decrease lenght size
      PInfo += Len;                                         // move pointer
   }

   // object magnitude
   if ( Mag != 0 )
   {
      sprintf( Message, "Magnutudo %2.1f\r\n", Mag);
      Len = strlen( Message);                               // compute message lenght
      if ( Size >= Len )                                    // is there enought space ?
      {
         strcpy( PInfo, Message );                          // copy message
         Size -= Len;                                       // decrease lenght size
         PInfo += Len;                                      // move pointer
      }
   }

   // object extension
   if ( ApparentSize != 0 )
   {
      sprintf( Message, "Dimensione app. %4.1f arcsec\r\n", ApparentSize);
      Len = strlen( Message);                               // compute message lenght
      if ( Size >= Len )                                    // is there enought space ?
      {
         strcpy( PInfo, Message );                          // copy message
         Size -= Len;                                       // decrease lenght size
         PInfo += Len;                                      // move pointer
      }
   }
   CMObject::GetInfo( PInfo, Size );
}



/*---------------------------------------------------------------------------

	set object type

---------------------------------------------------------------------------*/

void CMDeepSky::SetType( TYPE Type )
{
   ObjectType = Type;
}



/*---------------------------------------------------------------------------

	get object type

---------------------------------------------------------------------------*/

CMDeepSky::TYPE CMDeepSky::GetType( void )
{
   return ObjectType;
}



/*---------------------------------------------------------------------------

	set apparent size

---------------------------------------------------------------------------*/

void CMDeepSky::SetSize( float Size )
{
   ApparentSize = Size;
}



/*---------------------------------------------------------------------------

	get apparent size

---------------------------------------------------------------------------*/

float CMDeepSky::GetSize( void )
{
   return ApparentSize;
}



/*---------------------------------------------------------------------------

	compute position

---------------------------------------------------------------------------*/

void CMDeepSky::ComputePosition( void )
{
   CMCoEquatorial                      *Base;

   // object with base coordinate
   Base = new CMCoEquatorial(BasePosition);

   // compute precession
   delete Position;
   Position = Base->ChangeEqu( Date );
   delete Base;
}




