/*---------------------------------------------------------------------------

	CMDeepSky.h  Header for deep sky objects

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : November 2012

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include antirepeat
#ifndef CMDeepSkyH
#define CMDeepSkyH



// system include
#include <math.h>



// project include
#include "CMObject.h"
#include "CMCoEquatorial.h"



class CMDeepSky : public CMObject
{
   public:

      // deep sky object type
      typedef enum
      {
         GALAXY,                                            // galaxy
         OPENCLASTER,                                       // open cluster
         GLOBULARCLUSTER,                                   // globular cluster
         NEBULA,                                            // emission or reflection nebula
         PLANETARYNEBULLA,                                  // planetary nebula
         NEBULACLUSTER,                                     // cluster associated with nebula
         ASTERISM,                                          // asterism
         KNOT,                                              // knot or nebula in external galaxy
         TRIPLESTAR,                                        // tripe star
         DOUBLESTAR,                                        // double star
         SINGLESTAR,                                        // single star
         NOTDEFINED,                                        // uncertain or not defined object
      } TYPE;
      // constructor
                                       CMDeepSky( );
      // copy constructor
                                       CMDeepSky( CMDeepSky *Star );
      // destroyer
      virtual                          ~CMDeepSky();
      // set base object position
      void                             SetBase( double DataBase, CMCoEquatorial* PosizioneBase );
      // get magnitude
      virtual void                     GetInfo( char *PInfo, int Size );
      // set object type
      virtual void                     SetType( TYPE Type );
      // get object type
      TYPE                             GetType( void );
      // set apparent size
      virtual void                     SetSize( float Size );
      // get apparent size
      float                            GetSize( void );
   protected:
      // compute position
      virtual void                     ComputePosition( void );

      double                           BaseDate;            // date for base position
      CMCoEquatorial*                  BasePosition;        // base position
      TYPE                             ObjectType;          // object type
      float                            ApparentSize;        // object apparent size in arc sec
};
#endif
