/*---------------------------------------------------------------------------

	CMDegree.cpp  Degree coordinate computing object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include di sistema
#include "string.h"



// include miei
#include "CMDegree.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMDegree::CMDegree(  )
{
   strcpy( Dec.sign, "+");
   Dec.gg = 0;
   Dec.mm = 0;
   Dec.ss = 0;
}



/*---------------------------------------------------------------------------

	object copy constructor

---------------------------------------------------------------------------*/

CMDegree::CMDegree( const CMDegree *Copy )
{
   Dec = Copy->Dec;
}



/*---------------------------------------------------------------------------

	set composed coordinate

---------------------------------------------------------------------------*/

void CMDegree::Set( const DEGREE *Par )
{
   Dec = *Par;
}



/*---------------------------------------------------------------------------

	set packed coordinate (in degrees with decimal)

---------------------------------------------------------------------------*/

void CMDegree::Set( double Par )
{
   double                              Residual;            // decimal part

   // assign sign
   strcpy( Dec.sign, "+");
   if ( Par < 0 )
   {
      strcpy( Dec.sign, "-");
      Par = - Par;
   }

   Dec.gg = Par;                                            // assign degrees
   Residual = Par - Dec.gg;                                 // residual degrees
   Residual *= 60;                                          // makes it minutes
   Dec.mm = Residual;                                       // assign minutes
   Residual = Residual - Dec.mm;                            // residual minutes
   Residual *= 60;                                          // makes it seconds
   Dec.ss = Residual;                                       // assign seconds
}



/*---------------------------------------------------------------------------

	get composed coordinate

---------------------------------------------------------------------------*/

const CMDegree::DEGREE* CMDegree::Get(void ) const
{
   return &Dec;
}



/*---------------------------------------------------------------------------

	get packed coordinate

---------------------------------------------------------------------------*/

double CMDegree::GetC( void ) const
{
   double                              Degree;              // composed coordinate

   Degree = Dec.ss;                                         // assign seconds
   Degree /= 60;                                            // makes it minutes
   Degree += Dec.mm;                                        // add minutes
   Degree /= 60;                                            // makes it degree
   Degree += Dec.gg;                                        // add degree
   if ( Dec.sign[0] == '-' )                                // make is negative if it needs
      Degree = - Degree;

   return Degree;
}



/*---------------------------------------------------------------------------

	add object coordinate to the object

---------------------------------------------------------------------------*/

void CMDegree::Add( CMDegree *Par )
{
   double                              CurrentDegree;       // current packed coordinate
   double                              ParDegree;           // parameter packed coordinate

   CurrentDegree = GetC( );                                 // get current packed coordinate
   ParDegree = Par->GetC( );                                // get parameter packed coordinate
   CurrentDegree += ParDegree;                              // sum coordinate
   Set( CurrentDegree );                                    // set new coordinate
}



/*---------------------------------------------------------------------------

	add composed coordinate to the object

---------------------------------------------------------------------------*/

void CMDegree::Add( CMDegree::DEGREE *Par )
{
   CMDegree                            *TempObj;            // temporary object

   TempObj = new CMDegree();                                // create temporary object
   TempObj->Set( Par );                                     // set parameter to temporary object
   Add( TempObj );                                          // add temporary object to current object
   delete TempObj;                                          // destroy temporary object
}



/*---------------------------------------------------------------------------

	aggiunge gradi indicati a quelli correnti

---------------------------------------------------------------------------*/

void CMDegree::Add( double Par )
{
   double                              CurrentDegree;       // current packed coordinate

   CurrentDegree = GetC();                                  // get current packed degree
   CurrentDegree += Par;                                    // add parameter
   Set( CurrentDegree );                                    // set new coordinate
}



/*---------------------------------------------------------------------------

	subctract object coordinate from the object

---------------------------------------------------------------------------*/

void CMDegree::Sub( CMDegree *Par )
{
   double                              CurrentDegree;       // current packed coordinate
   double                              ParDegree;           // parameter packed coordinate

   CurrentDegree = GetC( );                                 // get current packed coordinate
   ParDegree = Par->GetC( );                                // get parameter packed coordinate
   CurrentDegree -= ParDegree;                              // subctract coordinate
   Set( CurrentDegree );                                    // set new coordinate
}



/*---------------------------------------------------------------------------

	subctract composed coordinate from the object

---------------------------------------------------------------------------*/

void CMDegree::Sub( CMDegree::DEGREE *Par )
{
   CMDegree                            *TempObj;            // temporary object

   TempObj = new CMDegree();                                // create temporary object
   TempObj->Set( Par );                                     // set parameter to temporary object
   Sub( TempObj );                                          // subtract temporary object to current object
   delete TempObj;                                          // destroy temporary object
}



/*---------------------------------------------------------------------------

	subctract packed coordinate from the object

---------------------------------------------------------------------------*/

void CMDegree::Sub( double Par )
{
   double                              CurrentDegree;       // current packed coordinate

   CurrentDegree = GetC();                                  // get current packed degree
   CurrentDegree -= Par;                                    // subtract parameter
   Set( CurrentDegree );                                    // set new coordinate
}

