/*---------------------------------------------------------------------------

	CMEvent.h  Header for astronomical event object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : August 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include antirepeat
#ifndef CMEventH
#define CMEventH


#include "CMHours.h"
#include "CMDegree.h"



class CMEvent
{
   public:
      // event definition
      typedef struct
      {
         int                           Year;
         int                           Month;
         int                           Day;
         int                           Hour;
         int                           Minute;
         int                           Sec;
      } DATE;

      // constructor
                                       CMEvent();
      // copy constructor
                                       CMEvent( CMEvent *Copy );
      // set date
      int                              SetDate( DATE *Event );
      // set date in JD
      int                              SetGG( double Event );
      // get date
      const DATE*                      GetDate();
      // get date in JD
      double                           GetGG();
      // add days to current event
      void                             AddTime( double Delta );

   private:
      // compute JD from date
      void                             CalcGG(void);

      // compute date from JD
      void                             CalcDate();

      // date
      DATE                             EventC;
      // date in JD
      double                           EventGG;
      // date validity
      bool                             ValidC;
      // date in JD validy
      bool                             ValidGG;
};

#endif