/*---------------------------------------------------------------------------

	CMHours.h  Header object for hours coordinate computing

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// antirepeat include
#ifndef CMHoursH
#define CMHoursH



class CMHours
{
   public:
      // defines a righit ascension coordinate
      typedef struct
      {
         int                           hh;                  // hours
         int                           mm;                  // minutes
         float                         ss;                  // seconds
      }HOURS;

      // object constructor
                                       CMHours( );
      // object copy constructor
                                       CMHours( const CMHours *Copy );
      // set composed coordinate
      void                             Set( const HOURS *Par, bool Neg=false );
      // set packed coordinate
      void                             Set( double Par ) ;
      // get composed coordinate
      const HOURS *                    Get( void ) const;
      // get packed coordinate
      double                           GetC(void ) const;
      // add object coordinate to the object
      void                             Add( CMHours *Par );
      // add composed coordinate to the object
      void                             Add( CMHours::HOURS *Par );
      // add packed coordinate to the object
      void                             Add( double Par );
      // subctract composed coordinate from the object
      void                             Sub( CMHours::HOURS *Par );
      // subctract object coordinate from the object
      void                             Sub( CMHours *Par );
      // subctract packed coordinate from the object
      void                             Sub( double Par );
   protected:
      HOURS                            Ar;                  // objects coordinate
 };
#endif
