/*---------------------------------------------------------------------------

	CMObject.h  Header for planet objects

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include antirepeat
#ifndef CMMPlanetH
#define CMMPlanetH



// system include
#include <math.h>



// project include
#include "CMObject.h"
#include "CMCoEquatorial.h"
#include "CMFunctions.h"
#include "CMPlanet.h"




class CMMPlanet : public CMObject
{
   public:
      // constructor
                                       CMMPlanet( );
      // copy constructor
                                       CMMPlanet( CMMPlanet *Planet );
      // destroyer
      virtual                          ~CMMPlanet();
      // set date
      virtual void                     SetData( double PData );
      // set planet of view
      int                              SetPlanetView( CMPlanet *Planet );
      // set computing option
      void                             SetOpt( bool Nutation, bool TimeLight );
      // set orbital elements
      void                             SetEleOrb( double ParJEph, MEEUS_ELE_ORB* ParEleOrb, double ParM0, double ParN );
      // set base magnitude
      void                             SetMagBase( double PMagBase );
      // get orbital elements
      const MEEUS_ELE_ORB*             GetEleOrb( void );
      // get magnitude
      virtual float                    GetMag(void);
   protected:
      // compute position
      virtual void                     ComputePosition( void );
      // compute orbital parameters
      void                             CalcParamOrb( double Days, MEEUS_ELE_ORB &EleOrb );

      bool                             Nutation;            // compute nutation
      bool                             TimeLight;           // time light correction
      CMPlanet                         *PlanetView;         // plant of view
      double                           JEph;
      MEEUS_ELE_ORB                    EleOrb;              // orbital elements
      double                           M0;                  // mean anomaly
      double                           N;                   // mean daily motion
      double                           MagBase;             // base magnitude
      double                           Mag;                 // current magnitude
      CMCoEcliptic                     *PosizioneEclReale;  // real ecliptical position
      CMCoRectangolar                  *RealRetPosition;    // real rectangolar position
      CMCoRectangolar                  *PosizioneRetView;   // apparent rectangolar position
      CMCoEcliptic                     *PosizioneEclVSOP;   // apparent ecliptical position in VSOP
      CMCoEcliptic                     *PosizioneEclFK5;    // apparent ecliptical position in FK5
};

#endif
