/*---------------------------------------------------------------------------

	CMMoon.cpp  Moon object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : November 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>



// project include
#include "CMMoon.h"




/*---------------------------------------------------------------------------

	Object constructor

---------------------------------------------------------------------------*/

CMMoon::CMMoon(void) :
CMObject()
{
   SetName("Luna", CMObject::NAME_OWN );
   Nutation = false;
}



/*---------------------------------------------------------------------------

	Object copy constructor

---------------------------------------------------------------------------*/

CMMoon::CMMoon( CMMoon *Copy ) :
CMObject( Copy )
{
   Nutation = Copy->Nutation;
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMMoon::~CMMoon(  )
{
}



/*---------------------------------------------------------------------------

	computing setting

---------------------------------------------------------------------------*/

void CMMoon::SetOpt( bool ParNutation )
{
   Nutation = ParNutation;
}



/*---------------------------------------------------------------------------

	moon position computing

---------------------------------------------------------------------------*/

void CMMoon::ComputePosition( void )
{
   CMCoEcliptic          *PosEcl;

   // compute ecliptic position con Meeus method
   PosEcl = CM_MoonPositionComputing( Date );

   // get equatorial position
   delete Position;
   Position = PosEcl->GetCoEqu(Date, Nutation) ;
   delete PosEcl;
   return;
}


