/*---------------------------------------------------------------------------

	MCObject.cpp Generic abstract astronomical object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <stdlib.h>



// project include
#include "CMObject.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

CMObject::CMObject()
{
   // null names
   *Name[0] = 0;
   *Name[1] = 0;
   *Name[2] = 0;
   // null date
   Date = 0;
   Info = 0;
   // create object position
   Position = new CMCoEquatorial();
   // position to be computed
   Dirty = true;
}



/*---------------------------------------------------------------------------

	copy  object constructor

---------------------------------------------------------------------------*/

CMObject::CMObject( CMObject *Copy )
{
   // copy all
   strcpy(Name[0], Copy->Name[0]);
   strcpy(Name[1], Copy->Name[1]);
   strcpy(Name[2], Copy->Name[2]);
   Date = Copy->Date;
   Position = new CMCoEquatorial( Copy->Position );

   if ( Copy->Info != 0 )
   {
      Info = (char*)malloc( strlen( Copy->Info) + 1 );      // allocate memory for info
      strcpy( Info, Copy->Info);
   }

   Dirty = Copy->Dirty;
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMObject::~CMObject(  )
{
   delete Position;
   if ( Info != 0 )                                         // remove info if present
      free ( Info );
}



/*---------------------------------------------------------------------------

	set object name

---------------------------------------------------------------------------*/

void CMObject::SetName( char *PName, NAME_TYPE Type )
{
   int                                 Index;

   switch ( Type )
   {
      case NAME_CATALOG:
         Index = 0; break;
      case NAME_COST:
         Index = 1; break;
      case NAME_OWN:
         Index = 2; break;
      default:
         return;
   }
   // copy name
   strncpy( Name[Index], PName, CMOBJECT_NAMELEN);
   Name[Index][CMOBJECT_NAMELEN] = 0;
}



/*---------------------------------------------------------------------------

	get object name

---------------------------------------------------------------------------*/

const char* CMObject::GetName( NAME_TYPE Type )
{
   int                                 Index;

   switch ( Type )
   {
      case NAME_CATALOG:
         Index = 0; break;
      case NAME_COST:
         Index = 1; break;
      case NAME_OWN:
         Index = 2; break;
      default:
         return 0;
   }
   return Name[Index];
}



/*---------------------------------------------------------------------------

	set efemerid date

---------------------------------------------------------------------------*/

void CMObject::SetData( double gg )
{
   Date = gg;                          // update date
   Dirty = true;                       // position to be computed
   return;
}



/*---------------------------------------------------------------------------

	get efemerid date

---------------------------------------------------------------------------*/

double CMObject::GetData( void )
{
   return Date;
}



/*---------------------------------------------------------------------------

	get position

---------------------------------------------------------------------------*/

const CMCoEquatorial* CMObject::GetPosition( void )
{
   if ( Dirty == true )
   {
      Dirty = false;
      ComputePosition();
   }
   return Position;
}



/*---------------------------------------------------------------------------

	set magnitude

---------------------------------------------------------------------------*/

void CMObject::SetMag( float ParMag )
{
   Mag = ParMag;
}



/*---------------------------------------------------------------------------

	get magnitude

---------------------------------------------------------------------------*/

float CMObject::GetMag( void )
{
   return Mag;
}



/*---------------------------------------------------------------------------

	set information

---------------------------------------------------------------------------*/

void CMObject::SetInfo( char *PInfo )
{
   if ( Info != 0 )                                         // remove previous info if present
      free ( Info );

   Info = (char*)malloc( strlen( PInfo) + 1 );              // allocate memory for info
   strcpy( Info, PInfo);                                    // copy information
}



/*---------------------------------------------------------------------------

	get information

---------------------------------------------------------------------------*/

void CMObject::GetInfo( char *PInfo, int Size )
{
   *PInfo = 0;                                              // preventive erase
   if ( Info != 0 )                                         // only if information are present
   {
      if ( Size > (int)strlen(Info ) )                      // only if there is enought memory
         strcpy( PInfo, Info );
   }
}

