/*---------------------------------------------------------------------------

	MCObject.h  Header for astonomical generic object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include autirepeat
#ifndef CMObjectH
#define CMObjectH


// system include
#include <math.h>



// project include
#include "CMCoEquatorial.h"
#include "CMFunctions.h"
#include "CMVSOP.h"

#define                                CMOBJECT_NAMELEN 20

class CMObject
{
   public:

      // name type
      typedef enum
      {
         NAME_CATALOG,                                      // catalog name
         NAME_COST,                                         // costellation name
         NAME_OWN,                                          // own particola name
      } NAME_TYPE;

      // object constructor
                                       CMObject();
      // copy object constructor
                                       CMObject( CMObject *Copy );
      // destroyer
      virtual                          ~CMObject();
      // set object name
      void                             SetName(char *PName, NAME_TYPE Type);
      // get object name
      const char*                      GetName(NAME_TYPE Type);
      // set efemerid date
      virtual void                     SetData( double gg );
      // get efemerid date
      double                           GetData( void );
      // get position
      const CMCoEquatorial*            GetPosition( void );
      // set magnitude
      virtual void                     SetMag( float PMag );
      // get magnitude
      virtual float                    GetMag( void );
      // set information
      void                             SetInfo( char *PInfo );
      // get magnitude
      virtual void                     GetInfo( char *PInfo, int Size );
   protected:
      // compute position
      virtual void                     ComputePosition( void )=0;

      bool                             Dirty;                        // position to be computed
      char                             Name[3][CMOBJECT_NAMELEN+1];  // object names
      double                           Date;                         // efemerid date
      CMCoEquatorial*                  Position;                     // equatorial position
      float                            Mag;                          // magnitude
      char                             *Info;                        // object information
};

#endif
