/*---------------------------------------------------------------------------

	CmObjectList.cpp  List of generic astronomical objects

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : May 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// System include
#include <string.h>
#include <alloc.h>



// Project include
#include "CMObjectList.h"



/*---------------------------------------------------------------------------

   Object constructor

---------------------------------------------------------------------------*/

CMObjectList::CMObjectList( bool PByReference )
{
   ObjectCount = 0;
   FirstElement = 0;
   LastElement = 0;
   CurrentElement = 0;
   PrevElement = 0;
   ByReference = PByReference;
}



/*---------------------------------------------------------------------------

	object destroyer

---------------------------------------------------------------------------*/

CMObjectList::~CMObjectList( )
{
   RemoveAll();
}



/*---------------------------------------------------------------------------

	Add an object to list

---------------------------------------------------------------------------*/

void CMObjectList::Add(CMObject *Object)
{
   LIST_ELEMENT                        *NewElement;         // new added element

   // create and fill new element
   NewElement = (LIST_ELEMENT*)malloc(sizeof(LIST_ELEMENT));
   NewElement->Object = Object;
   NewElement->Next = 0;

   // when list is empty this is the first and the last
   if ( FirstElement == 0 )
   {
      LastElement = NewElement;
      FirstElement = NewElement;
   }
   else
   {
      // update and reassign last element
      LastElement->Next = NewElement;
      LastElement = NewElement;
   }

   ObjectCount++;                                           // increment object counter
}



/*---------------------------------------------------------------------------

	Remove current object from list

---------------------------------------------------------------------------*/

void CMObjectList::Remove(void)
{
   // put current element out of the link
   if ( PrevElement == 0 )
      FirstElement = CurrentElement->Next;
   else
      PrevElement->Next = CurrentElement->Next;

   // delete current object
   if ( ByReference == false )
      delete CurrentElement->Object;
   // delete currente element
   delete CurrentElement;
   // adjust count
   ObjectCount--;
}



/*---------------------------------------------------------------------------

	Remove all objects from list

---------------------------------------------------------------------------*/

void CMObjectList::RemoveAll(void)
{
   LIST_ELEMENT                        *NextElement;        // Next element to delete

   // only if list is not empty
   if ( FirstElement != 0 )
   {
      do
      {
         NextElement = FirstElement->Next;                  // save next element
         if ( ByReference == false )
            delete FirstElement->Object;                    // delete object in the list
         free( FirstElement);                               // free element list structure
         FirstElement = NextElement;                        // now the next is the first
      } while (FirstElement != 0 );
   }
   FirstElement = 0;

   ObjectCount = 0;
}


/*---------------------------------------------------------------------------

	Set list pointer to the first element

---------------------------------------------------------------------------*/

void CMObjectList::First( void )
{
   ScanElement = FirstElement;
   CurrentElement = 0;
   PrevElement = 0;
}



/*---------------------------------------------------------------------------

	get next object in the list

---------------------------------------------------------------------------*/

CMObject * CMObjectList::Next( void )
{
   CMObject                            *Object;             // obejct to yeld

   PrevElement = CurrentElement;
   CurrentElement = ScanElement;
   if ( ScanElement == 0 )
      Object = 0;
   else
   {
      Object = ScanElement->Object;
      ScanElement = ScanElement->Next;
   }

   return Object;
}



/*---------------------------------------------------------------------------

	get number of objects in the list

---------------------------------------------------------------------------*/

int CMObjectList::GetCount( void )
{
   return ObjectCount;
}

