/*---------------------------------------------------------------------------

	CMObjectList.h  Header for list of generic astronomical objects

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : May 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include antirepeat
#ifndef CMObjectListH
#define CMObjectListH


// system include
#include <math.h>



// project include
#include "CMObject.h"



class CMObjectList
{
   public:

      typedef struct LIST_ELEMENT_Tag
      {
         CMObject                      *Object;             // pointer to object in the list
         LIST_ELEMENT_Tag              *Next;               // pointer to next element
      } LIST_ELEMENT;

      // object constructor
                                       CMObjectList( bool PByReference=false);
      // object destroyer
      virtual                          ~CMObjectList();
      // Add an object to list
      void                             Add(CMObject *Object);
      // remove actual object from the list
      void                             Remove( void );
      // remove all objects from the list
      void                             RemoveAll( void );
      // Set list pointer to the first element
      void                             First(void) ;
      // get next object in the list
      CMObject *                       Next( void );
      // get number of objects in the list
      int                              GetCount( void );
   protected:
      int                              ObjectCount;
      bool                             ByReference;
      LIST_ELEMENT                     *FirstElement;
      LIST_ELEMENT                     *LastElement;
      LIST_ELEMENT                     *ScanElement;
      LIST_ELEMENT                     *CurrentElement;
      LIST_ELEMENT                     *PrevElement;
};

#endif
