/*---------------------------------------------------------------------------

	CMPlanet.h  Header for planet object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// include antirepeat
#ifndef CMPlanetH
#define CMPlanetH



// system  include
#include <math.h>



// project include
#include "CMObject.h"
#include "CMCoEquatorial.h"
#include "CMFunctions.h"
#include "CMVSOP.h"

class CMPlanet : public CMObject
{
   public:
      // planet definition
      typedef enum
      {
         SUN,
         MERCURY,
         VENUS,
         EARTH,
         EARTH2000,
         MARS,
         JUPITER,
         SATURN,
         URAN,
         NEPTUN,
      } PLANET;

      // compute mode
      typedef enum
      {
         MEEUS,
         VSOP,
      } COMP_MODE;

      // Object constructor
                                       CMPlanet( PLANET Planet);
      // Object copy constructor
                                       CMPlanet( CMPlanet *Planet );
      // destroyer
      virtual                          ~CMPlanet();
      // set efemerid date
      virtual void                     SetData( double PData );
      // set planet of view
      int                              SetPlanetView( CMPlanet *Planet );
      // set computing options
      void                             SetOpt( COMP_MODE Mode, bool Nutation, bool TimeLight );
      // get orbital elemets
      const MEEUS_ELE_ORB*             GetEleOrb( void );
      // comupute position
      virtual void                     ComputePosition( void );

      CMCoRectangolar                  *RealRetPosition;       // real rectangolar position
   protected:
      COMP_MODE                        Mode;                   // computing way
      bool                             Nutation;               // nutation
      bool                             TimeLight;              // time light correction
      CMPlanet                         *PlanetView;            // planet of view
      PLANET                           Planet;                 // planet code
      const MEEUS_COEFF_ELE            *CoeEleMeeus;           // Meeus coefficents
      VSOP_Object                      *Series;                // VSOP series for rectangolar elements
      MEEUS_ELE_ORB                    EleOrb;                 // orbital elementsi
      CMCoEcliptic                     *RealEclPos;            // real eclictical position
      CMCoRectangolar                  *ApparentRetPos;        // rectangolar postion from planet of view
      CMCoEcliptic                     *VSOPEclPos;            // eclictic postion from planet of view with VSOP system
      CMCoEcliptic                     *FK5EclPos;             // eclictic postion from planet of view with FK5 system
};

#endif
