/*---------------------------------------------------------------------------

	CmStar.cpp Star computing object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : September 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>



// project include
#include "CMStar.h"




/*---------------------------------------------------------------------------

	constructor

---------------------------------------------------------------------------*/

CMStar::CMStar() :
CMObject()
{
   BaseDate = 2451545;                                      // J2000
   BasePosition = 0;
   RAMotion = 0;
   DecMotion = 0;
   Date = 2451545;                                          // J2000
}



/*---------------------------------------------------------------------------

	copy constructor

---------------------------------------------------------------------------*/

CMStar::CMStar( CMStar *Copy ) :
CMObject( Copy )
{
   BaseDate = Copy->BaseDate;
   BasePosition = Copy->BasePosition;
   RAMotion = Copy->RAMotion;
   DecMotion = Copy->DecMotion;
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMStar::~CMStar(  )
{
   if ( BasePosition != 0 )
      delete BasePosition;
}



/*---------------------------------------------------------------------------

	set base star position

---------------------------------------------------------------------------*/

void CMStar::SetBase( double ParDataBase, CMCoEquatorial* ParPosizioneBase, double ParMotoAr, double ParMotoDec )
{
   if ( BasePosition != 0 )
      delete BasePosition;

   BaseDate = ParDataBase;
   BasePosition = new CMCoEquatorial ( ParPosizioneBase );
   RAMotion = ParMotoAr;
   DecMotion = ParMotoDec;
}



/*---------------------------------------------------------------------------

	compute position

---------------------------------------------------------------------------*/

void CMStar::ComputePosition( void )
{
   double                              Years;
   double                              RADelta;
   double                              DecDelta;
   double                              Delta;
   CMCoEquatorial                      *Base;


   // object with base coordinate
   Base = new CMCoEquatorial(BasePosition);

   // compute julian years difference
   Years = (Date - BaseDate) / 365.25;

   // compute and apply proper motion
   Delta = Base->GetDec()->GetC();
   RADelta = RAMotion * Years * cos( Delta / RAD );
   RADelta /= 15;
   DecDelta = DecMotion * Years;
   Base->GetRA()->Add( RADelta/3600 );
   Base->GetDec()->Add( DecDelta/3600 );

   // compute presession
   delete Position;
   Position = Base->ChangeEqu( Date );
   delete Base;
}




