/*---------------------------------------------------------------------------

	CmStarEB.cpp Eclipse Binary Star computing object

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : MArch 2012

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>



// project include
#include "CMStarEB.h"




/*---------------------------------------------------------------------------

	constructor

---------------------------------------------------------------------------*/

CMStarEB::CMStarEB() :
CMStar()
{
   BaseJD = 0;
   Period = 0;
   MinimumPhase = 0;
   NeedPrimary = false;
   NeedSecondary = false;
}



/*---------------------------------------------------------------------------

	copy constructor

---------------------------------------------------------------------------*/

CMStarEB::CMStarEB( CMStarEB *Copy ) :
CMStar( Copy )
{
   BaseJD = Copy->BaseJD;
   Period = Copy->Period;
   MinimumPhase = Copy->MinimumPhase;
   NeedPrimary = Copy->NeedPrimary;
   NeedSecondary = Copy->NeedSecondary;
}



/*---------------------------------------------------------------------------

	destroyer

---------------------------------------------------------------------------*/

CMStarEB::~CMStarEB(  )
{
}



/*---------------------------------------------------------------------------

	set base star position

---------------------------------------------------------------------------*/

void CMStarEB::SetEBParameters( double PBaseJD, double PPeriod, double PMinimumPhase, bool PNeedPrimary, bool PNeedSecondary)
{
   // check corret parameters

   BaseJD           = PBaseJD;
   Period           = PPeriod;
   MinimumPhase     = PMinimumPhase;
   NeedPrimary      = PNeedPrimary;
   NeedSecondary    = PNeedSecondary;
}



/*---------------------------------------------------------------------------

	compute next minimum

---------------------------------------------------------------------------*/

double CMStarEB::GetNextMinimum( double From, bool *Primary )
{
   double                              Current;             // current rotation and phase
   int                                 Rotation;            // current rotation
   double                              Phase;               // current phase
   bool                                CompPrimary;         // next needed minimum is primary
   double                              NextMinimum;         // next minimum JD

   // check for set star
   if ( BaseJD == 0 || Period == 0 )
      return 0;

   // increment input from in order to be sure to skip the current minimum (rouding issue may be)
   From += 0.001;                                           // applay increment

   // compute current period state
   Current = (From - BaseJD)/Period;
   Rotation = Current;
   Phase = Current - Rotation;

   // decide wich minimum is needed
   CompPrimary = NeedPrimary;                               // condition valid when only one minimum is needded
   if ( NeedPrimary == true && NeedSecondary == true )      // when both minimum are required need a choise
   {
      if ( Phase < MinimumPhase )                           // when actual phase is less than secondary minimum next minimum is secondary
         CompPrimary = false;
   }

   // compute next minimum
   if ( CompPrimary == true )
   {
      NextMinimum = BaseJD + double(Rotation+1)* Period;
      *Primary = true;
   }
   else
   {
      if ( Phase > MinimumPhase )
         Rotation++;
      NextMinimum = BaseJD + double(Rotation)* Period + MinimumPhase * Period;
      *Primary = false;
   }

   return NextMinimum;
}




