/******************************************************************************

	CCDServerMF.cpp | CCD server main form class

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : May 2010

******************************************************************************/



// System include
#include <vcl.h>
#pragma hdrstop



// project include
#include "CCDServerMF.h"
#pragma package(smart_init)
#pragma resource "*.dfm"



// form object
TForm1 *Form1;



/****************************************************************************

   Object constructor

*****************************************************************************/

__fastcall TForm1::TForm1(TComponent* Owner)
   : TForm(Owner)
{
}



/****************************************************************************

   Shows the form

*****************************************************************************/

void __fastcall TForm1::FormShow(TObject *Sender)
{
   // initialize object variabiles
   ExposureActive = false;
   AbortActive = false;
   Image = 0;
   Image2 = 0;
   HLib = 0;
   DpRam = 0;

   // create mapped file
	HDpRam = CreateFileMapping((HANDLE)0xFFFFFFFF, NULL, PAGE_READWRITE, 0, sizeof(DP_CCD_SERVER), FILENAME_CCD_SERVER);
   if ( HDpRam == NULL )
   {
      MessageBox( this->Handle, "Fallita creazione mapped file", "Errore", 0 );
   }
   else
	{
   	// open view on mapped file
		DpRam = (DP_CCD_SERVER*)MapViewOfFile(HDpRam, FILE_MAP_WRITE, 0, 0, 0);
   	if ( DpRam == NULL )
         MessageBox( this->Handle, "Fallita aperture view su mapped file", "Errore", 0 );
      else
      {
         memset ( DpRam, 0, sizeof(DP_CCD_SERVER) );
         DpRam->Command = DP_COM_NULL;
         DpRam->Error = 0;
         DpRam->ExposingStatus = DP_STATUS_IDLE;
         DpRam->Opened = false;
      }
   }
}



/****************************************************************************

   Destroy the form

*****************************************************************************/

void __fastcall TForm1::FormDestroy(TObject *Sender)
{
   // close ccd
   CloseCCD(0);

	// close view
   if (DpRam != NULL)
      UnmapViewOfFile(DpRam) ;

	// close mapped file
   if (HDpRam != NULL)
      CloseHandle(HDpRam) ;
}



/****************************************************************************

   Open selected ccd

*****************************************************************************/

void __fastcall TForm1::Open(TObject *Sender)
{
   char                                CompleteName[100];   // complete dll name for selected device
   bool                                Ok;                  // ccd open without error
   char                                *p;

   // if CCD is already open it is an error
   if ( HLib != 0 )
      return;

   // create real dll name
   strcpy( CompleteName, "d_" );
   strcat( CompleteName, CCDName->Text.c_str() );
   Ok = true;

   // open DLL
   HLib = LoadLibrary(CompleteName);
   if ( HLib == 0 )
   {
      Ok = false;
      if (DpRam != 0 )
         DpRam->Error = DP_ERR_CCD;
   }

   // load all function pointes
   if ( Ok == true )
   {
      ccd_initialize = (CCD_INITIALIZE) GetProcAddress( HLib, "ccd_initialize" );
      if ( ccd_initialize == 0 )
         Ok = false;
      ccd_showdialog = (CCD_SHOWDIALOG) GetProcAddress( HLib, "ccd_showdialog" );
      if ( ccd_showdialog == 0 )
         Ok = false;
      ccd_getok = (CCD_GETOK) GetProcAddress( HLib, "ccd_getok" );
      if ( ccd_getok == 0 )
         Ok = false;
      ccd_finalize = (CCD_FINALIZE) GetProcAddress( HLib, "ccd_finalize" );
      if ( ccd_finalize == 0 )
         Ok = false;
      ccd_gethint = (CCD_GETHINT) GetProcAddress( HLib, "ccd_gethint" );
      if ( ccd_gethint == 0 )
         Ok = false;
      ccd_getmodes = (CCD_GETMODES) GetProcAddress( HLib, "ccd_getmodes" );
      if ( ccd_getmodes == 0 )
         Ok = false;
      ccd_gettitle = (CCD_GETTITLE) GetProcAddress( HLib, "ccd_gettitle" );
      if ( ccd_gettitle == 0 )
         Ok = false;
      ccd_getstatistics = (CCD_GETSTATISTICT) GetProcAddress( HLib, "ccd_getstatistics" );
      if ( ccd_getstatistics == 0 )
         Ok = false;
      ccd_getfocusmodes = (CCD_GETFOCUSMODES) GetProcAddress( HLib, "ccd_getfocusmodes" );
      if ( ccd_getfocusmodes == 0 )
         Ok = false;
      ccd_modifywindow = (CCD_MODIFYWINDOW) GetProcAddress( HLib, "ccd_modifywindow" );
      if ( ccd_modifywindow == 0 )
         Ok = false;
      ccd_windowmode = (CCD_WINDOWMODE) GetProcAddress( HLib, "ccd_windowmode" );
      if ( ccd_windowmode == 0 )
         Ok = false;
      ccd_guidewindowmode = (CCD_GUIDEWINDOWMODE) GetProcAddress( HLib, "ccd_guidewindowmode" );
      if ( ccd_guidewindowmode == 0 )
         Ok = false;
      ccd_gettemperature = (CCD_GETTEMPERATURE) GetProcAddress( HLib, "ccd_gettemperature" );
      if ( ccd_gettemperature == 0 )
         Ok = false;
      ccd_getwidth = (CCD_GETWIDTH) GetProcAddress( HLib, "ccd_getwidth" );
      if ( ccd_getwidth == 0 )
         Ok = false;
      ccd_getheight = (CCD_GETHIGHT) GetProcAddress( HLib, "ccd_getheight" );
      if ( ccd_getheight == 0 )
         Ok = false;
      ccd_getblockmode = (CCD_GETBLOCKMODE) GetProcAddress( HLib, "ccd_getblockmode" );
      if ( ccd_getblockmode == 0 )
         Ok = false;
      ccd_telescope = (CCD_TELESCOPE) GetProcAddress( HLib, "ccd_telescope" );
      if ( ccd_telescope == 0 )
         Ok = false;
      ccd_filterwheel = (CCD_FILTERWHELL) GetProcAddress( HLib, "ccd_filterwheel" );
      if ( ccd_filterwheel == 0 )
         Ok = false;
      ccd_startdownload = (CCD_STARTDOWNLOAD) GetProcAddress( HLib, "ccd_startdownload" );
      if ( ccd_startdownload == 0 )
         Ok = false;
      ccd_readline = (CCD_READLINE) GetProcAddress( HLib, "ccd_readline" );
      if ( ccd_readline == 0 )
         Ok = false;
      ccd_readimage = (CCD_READIMAGE) GetProcAddress( HLib, "ccd_readimage" );
      if ( ccd_readimage == 0 )
         Ok = false;
      ccd_enddownload = (CCD_ENDDOWNLOAD) GetProcAddress( HLib, "ccd_enddownload" );
      if ( ccd_enddownload == 0 )
         Ok = false;
      ccd_startexposure = (CCD_STARTEXPOSURE) GetProcAddress( HLib, "ccd_startexposure" );
      if ( ccd_startexposure == 0 )
         Ok = false;
      ccd_endexposure = (CCD_ENDEXPOSURE) GetProcAddress( HLib, "ccd_endexposure" );
      if ( ccd_endexposure == 0 )
         Ok = false;
      ccd_stopexposure = (CCD_STOPEXPOSURE) GetProcAddress( HLib, "ccd_stopexposure" );
      if ( ccd_stopexposure == 0 )
         Ok = false;
      ccd_getstatus = (CCD_GETSTATUS) GetProcAddress( HLib, "ccd_getstatus" );
      if ( ccd_getstatus == 0 )
         Ok = false;
      ccd_exposing = (CCD_EXPOSING) GetProcAddress( HLib, "ccd_exposing" );
      if ( ccd_exposing == 0 )
         Ok = false;
      ccd_fastexposure = (CCD_FASTEXPOSURE) GetProcAddress( HLib, "ccd_fastexposure" );
      if ( ccd_fastexposure == 0 )
         Ok = false;
      ccd_autoguideexposure = (CCD_AUTOGUIDEEXPOSURE) GetProcAddress( HLib, "ccd_autoguideexposure" );
      if ( ccd_autoguideexposure == 0 )
         Ok = false;
      ccd_setfocusimaging = (CCD_SETFOCUSIMAGING) GetProcAddress( HLib, "ccd_setfocusimaging" );
      if ( ccd_setfocusimaging == 0 )
         Ok = false;
      ccd_setselfguiderows = (CCD_SETSELFGUIDEROWS) GetProcAddress( HLib, "ccd_setselfguiderows" );
      if ( ccd_setselfguiderows == 0 )
         Ok = false;
      ccd_getfamily = (CCD_GETFAMILY) GetProcAddress( HLib, "ccd_getfamily" );
      if ( ccd_getfamily == 0 )
         Ok = false;
   }

   // if not all is ok release library and exit
   if ( Ok == false )
   {
      if ( HLib != 0 )
         FreeLibrary( HLib );
      HLib = 0;
      if (DpRam != 0 )
         DpRam->Error = DP_ERR_CCD;
      return;
   }

   // initialize CCD device
   ccd_initialize( this->Handle );
   Sleep( 200 );
   ccd_showdialog( 0 );
   Sleep( 200 );
   Ok = ccd_getok();

   // if CCD is not corret initialized relase library and exit
   if ( Ok == false )
   {
      if ( HLib != 0 )
         FreeLibrary( HLib );
      HLib = 0;
      return;
   }

   // set ccd
   ccd_getfocusmodes( );
   Sleep( 200 );
   ccd_windowmode(0, 0, 100000, 100000, 0);
   Sleep( 200 );
   Width = ccd_getwidth();
   WidthEdit->Text = Width;
   Sleep( 200 );
   Height = ccd_getheight()-1;
   HeightEdit->Text = Height;
   Sleep( 200 );
   p = ccd_gettitle();
   Sleep( 200 );
   p = ccd_getstatistics();
   Sleep( 200 );
   ccd_gethint(1);

   // create image buffer
   Image = (unsigned short*)malloc( Width * Height * 2);    // memory allocation for internal image
   Image2 = (unsigned short*)malloc( Width * Height * 2);   // memory allocation for internal image

   if (DpRam != 0 )
   {
      DpRam->Error = DP_ERR_NONE;
      DpRam->Opened = true;
   }
}



/****************************************************************************

   close ccd

*****************************************************************************/

void __fastcall TForm1::CloseCCD(TObject *Sender)
{
   // if CCD is not open it is an error
   if ( HLib == 0 )
      return;

   free ( Image );                    // free image buffer
   free ( Image2 );
   ccd_finalize();                    // free ccd
   FreeLibrary( HLib );
   HLib = 0;
   if ( DpRam != 0 )
   {
      DpRam->Error = DP_ERR_NONE;
      DpRam->Opened = false;
   }
}



/****************************************************************************

   start shot with ccd

*****************************************************************************/

void __fastcall TForm1::Shot(TObject *Sender)
{
   int                                 i;
   float                               f;

   if ( HLib == 0 || ExposureActive == true )
   {
      if (DpRam != 0 )
         DpRam->Error = DP_ERR_SHOT;
      return;
   }
   i = ccd_getfamily();
   Sleep( 200 );
   ccd_windowmode(0, 0, 100000, 100000, 1);
   Sleep( 200 );
   i = ccd_gettemperature();

   f = atof( Exposure->Text.c_str());
   f *= 1000;
   i = f;
   ccd_startexposure(i);
   ExposureActive = true;
   StartShot = GetTickCount();

   if (DpRam != 0 )
   {
      DpRam->Error = DP_ERR_NONE;
      DpRam->ExposingStatus = DP_STATUS_EXPOSING;
   }
}



/****************************************************************************

   abort exposing

*****************************************************************************/

void __fastcall TForm1::Abort(TObject *Sender)
{
   AbortActive = true;
}



/****************************************************************************

   Polling for dpram command

*****************************************************************************/

void __fastcall TForm1::Polling(TObject *Sender)
{
   int                                 i;
   int                                 i2;
   unsigned short                      *p;
   unsigned short                      *pr;
   unsigned short                      *pw;
   char                                c[20];               // generic string
   DWORD                               Now;                 // actual tick time
   HANDLE                              hFile;               // handle del file
   DWORD                               ToWrite;             // bytes to write
   DWORD                               Written;             // written bytes
   float                               Time;
   int                                 Exp;

   // accept command via dpram
   if ( DpRam != 0 )
   {
      switch ( DpRam->Command )
      {
         case DP_COM_OPEN:
            CCDName->Text = AnsiString( DpRam->CCDName );
            Open(0);
            DpRam->Command = DP_COM_NULL;
            break;
         case DP_COM_CLOSE:
            CloseCCD(0);
            DpRam->Command = DP_COM_NULL;
            break;
         case DP_COM_SHOT:
            gcvt( DpRam->ExposureTime, 5, c );
            Exposure->Text = AnsiString ( c );
            Shot(0);
            DpRam->Command = DP_COM_NULL;
            break;
         case DP_COM_ABORT:
            Abort(0);
            DpRam->Command = DP_COM_NULL;
            break;
         case DP_COM_EXIT:
            Close();
            break;
      }
   }

   // set button enable
   Button1->Enabled = HLib == 0;
   Button2->Enabled = HLib != 0;
   Button3->Enabled = (HLib != 0) && ExposureActive == false;
   Button4->Enabled = (HLib != 0) && ExposureActive == true;

   // exposing management
   if ( ExposureActive == true )
   {
      if ( AbortActive == true )
      {
         AbortActive = false;
         ccd_stopexposure();
         Sleep( 200 );
      }
      Exp = atoi( Exposure->Text.c_str() );
      Now = GetTickCount();
      Now -= StartShot;
      Time = Now;
      Time /= 1000;
      if ( Time > Exp )
          Time = Exp;
      Exposing->Text = Time;
      if ( DpRam != 0 )
         DpRam->ExposingTime = Time;
      i = ccd_exposing();
      if ( i != 0 )
         return;

      if ( DpRam != 0 )
         DpRam->ExposingTime = DpRam->ExposureTime;
      ExposureActive = false;
      AbortActive = false;
      ccd_endexposure();
      Sleep( 200 );
      ccd_getfamily();
      Sleep( 200 );
      ccd_startdownload();
      Width = ccd_getwidth();
      WidthEdit->Text = Width;
      Sleep( 200 );
      Height = ccd_getheight()-1;
      HeightEdit->Text = Height;
      Sleep( 200 );
      ccd_getfamily();
      Sleep( 200 );

      i = ccd_getblockmode();
      p = Image;
      if ( Height >= 0 )
         ccd_readline(p);           // leggo a vuoto la prima riga che  probabilmente quella del dark interno
      for ( i=0; i<Height; i++ )
      {
         p = Image + i*Width;
         ccd_readline(p);
      }
      ccd_enddownload(Image, 0, 0);

      // gira immagine destra/sinistra
      Memo->Lines->Clear();
      AnsiString A;

      for ( i=0; i<Height; i++ )
      {
         pr = Image + (i+1) * Width;
         pw = Image2 +  i    * Width;
         A = "";
         for ( i2=0; i2<Width; i2++ )
         {
            --pr;
            *pw = *pr ;
            if ( i2 < 10 )
            {
               if ( i2 > 0 )
                  A += ",";
               A += AnsiString( *pw );
            }
            pw++;
         }
         if ( i < 10 )
            Memo->Lines->Add(A);
      }

      hFile = CreateFile( "RawImage", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
      if ( hFile != INVALID_HANDLE_VALUE )
      {
         ToWrite = Width * Height * 2;                      // image size
         WriteFile ( hFile, Image2, ToWrite, &Written, NULL);  // write image
         if ( Written != ToWrite )                             // verico se ho scritto tutto
         {
         }
         CloseHandle( hFile);
      }
      Sleep ( 200 );
      if ( DpRam != 0 )
      {
         DpRam->Width = Width;
         DpRam->Height = Height;
         DpRam->ExposingStatus = DP_STATUS_IDLE;
      }
   }
}

