/******************************************************************************

	CCDServerMF.h | Header of CCD server main form

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : May 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef CCDServerMFH
#define CCDServerMFH



// system include
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>



// project include
#include "dpCCDServer.h"



// definition of functions in CCD DLL
typedef void   _stdcall    (*CCD_INITIALIZE)  ( void* );
typedef int    _stdcall    (*CCD_FINALIZE)  ( void );
typedef void   _stdcall    (*CCD_SHOWDIALOG)  ( void* );
typedef int    _stdcall    (*CCD_GETOK)  ( void );
typedef int    _stdcall    (*CCD_GETHINT)  ( int );
typedef int    _stdcall    (*CCD_GETMODES)  ( char**, char**, char**, char**, char**, char**, char**, char** );
typedef char*  _stdcall    (*CCD_GETTITLE)  ( void );
typedef char*  _stdcall    (*CCD_GETSTATISTICT)  ( void );
typedef int    _stdcall    (*CCD_GETFOCUSMODES)  ( void );
typedef void   _stdcall    (*CCD_MODIFYWINDOW)  ( void );
typedef int    _stdcall    (*CCD_WINDOWMODE)  ( int, int, int, int, int );
typedef int    _stdcall    (*CCD_GUIDEWINDOWMODE)  ( int, int, int, int, int );
typedef int    _stdcall    (*CCD_GETTEMPERATURE)  ( void );
typedef int    _stdcall    (*CCD_GETWIDTH)  ( void );
typedef int    _stdcall    (*CCD_GETHIGHT)  ( void );
typedef int    _stdcall    (*CCD_GETBLOCKMODE)  ( void );
typedef int    _stdcall    (*CCD_TELESCOPE)  ( int, int );
typedef int    _stdcall    (*CCD_FILTERWHELL)  ( int, int );
typedef void   _stdcall    (*CCD_STARTDOWNLOAD)  ( void );
typedef void   _stdcall    (*CCD_READLINE)  ( unsigned short* );
typedef void   _stdcall    (*CCD_READIMAGE)  ( unsigned short* );
typedef void   _stdcall    (*CCD_ENDDOWNLOAD)  ( unsigned short*, int, int );
typedef int    _stdcall    (*CCD_STARTEXPOSURE)  ( int );
typedef void   _stdcall    (*CCD_ENDEXPOSURE)  ( void );
typedef void   _stdcall    (*CCD_STOPEXPOSURE)  ( void );
typedef void   _stdcall    (*CCD_GETSTATUS)  ( void );
typedef int    _stdcall    (*CCD_EXPOSING)  ( void );
typedef void   _stdcall    (*CCD_FASTEXPOSURE)  ( void );
typedef void   _stdcall    (*CCD_AUTOGUIDEEXPOSURE)  ( void );
typedef void   _stdcall    (*CCD_SETFOCUSIMAGING)  ( void );
typedef void   _stdcall    (*CCD_SETSELFGUIDEROWS)  ( void );
typedef int    _stdcall    (*CCD_GETFAMILY)  ( void );



/******************************************************************************

	CCD server main form class

******************************************************************************/

class TForm1 : public TForm
{
   __published:	// IDE-managed Components
      TButton                          *Button1;
      TButton                          *Button2;
      TLabel                           *Label1;
      TEdit                            *CCDName;
      TLabel                           *Label2;
      TEdit                            *Exposure;
      TEdit                            *Exposing;
      TButton                          *Button3;
      TButton                          *Button4;
      TTimer                           *Timer1;
   TMemo *Memo;
   TLabel *Label3;
   TEdit *WidthEdit;
   TEdit *HeightEdit;
   TLabel *Label4;
      void                             __fastcall Open(TObject *Sender);
      void                             __fastcall CloseCCD(TObject *Sender);
      void                             __fastcall Shot(TObject *Sender);
      void                             __fastcall Abort(TObject *Sender);
      void                             __fastcall FormShow(TObject *Sender);
      void                             __fastcall FormDestroy(TObject *Sender);
      void                             __fastcall Polling(TObject *Sender);

   private:	// User declarations
      HINSTANCE                        HLib;
      CCD_INITIALIZE                   ccd_initialize;
      CCD_FINALIZE                     ccd_finalize;
      CCD_SHOWDIALOG                   ccd_showdialog;
      CCD_GETOK                        ccd_getok;
      CCD_GETHINT                      ccd_gethint;
      CCD_GETMODES                     ccd_getmodes;
      CCD_GETTITLE                     ccd_gettitle;
      CCD_GETSTATISTICT                ccd_getstatistics;
      CCD_GETFOCUSMODES                ccd_getfocusmodes;
      CCD_MODIFYWINDOW                 ccd_modifywindow;
      CCD_WINDOWMODE                   ccd_windowmode;
      CCD_GUIDEWINDOWMODE              ccd_guidewindowmode;
      CCD_GETTEMPERATURE               ccd_gettemperature;
      CCD_GETWIDTH                     ccd_getwidth;
      CCD_GETHIGHT                     ccd_getheight;
      CCD_GETBLOCKMODE                 ccd_getblockmode;
      CCD_TELESCOPE                    ccd_telescope;
      CCD_FILTERWHELL                  ccd_filterwheel;
      CCD_STARTDOWNLOAD                ccd_startdownload;
      CCD_READLINE                     ccd_readline;
      CCD_READIMAGE                    ccd_readimage;
      CCD_ENDDOWNLOAD                  ccd_enddownload;
      CCD_STARTEXPOSURE                ccd_startexposure;
      CCD_ENDEXPOSURE                  ccd_endexposure;
      CCD_STOPEXPOSURE                 ccd_stopexposure;
      CCD_GETSTATUS                    ccd_getstatus;
      CCD_EXPOSING                     ccd_exposing;
      CCD_FASTEXPOSURE                 ccd_fastexposure;
      CCD_AUTOGUIDEEXPOSURE            ccd_autoguideexposure;
      CCD_SETFOCUSIMAGING              ccd_setfocusimaging;
      CCD_SETSELFGUIDEROWS             ccd_setselfguiderows;
      CCD_GETFAMILY                    ccd_getfamily;
      int                              Width;
      int                              Height;
      bool                             ExposureActive;
      bool                             AbortActive;
      unsigned short                   *Image;
      unsigned short                   *Image2;
      DWORD                            StartShot;
      HANDLE							      HDpRam;                 // dp-ram handle
      DP_CCD_SERVER					      *DpRam;						// dp ram pointer

   public:		// User declarations
      __fastcall                       TForm1(TComponent* Owner);
};
//---------------------------------------------------------------------------
extern PACKAGE TForm1 *Form1;
//---------------------------------------------------------------------------
#endif
