/******************************************************************************

	CloseSequence.h | Header of observatory closing sequence

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef CloseSequenceH
#define CloseSequenceH


// project include
#include "Observer.h"
#include "TentaclesMgr.h"



class CloseSequence : public TThread, public Observer
{
   public:

      // state of sequence
      typedef enum
      {
         SEQ_STATE_IDLE,                                    // idle, wait to start
         SEQ_STATE_START,                                   // start state
         SEQ_STATE_TELESCOPEHOME,                           // wait for telescope in home position
         SEQ_STATE_DOMEZERO,                                // wait for dome in zero position
         SEQ_STATE_CLOSEBSHUTTER,                           // wait for close balancing shutter
         SEQ_STATE_CLOSESSHUTTER,                           // wait for close sliding shutter
         SEQ_STATE_DOMEHOME,                                // wait for dome in home position
         SEQ_STATE_END,                                     // sequence ended
      } SEQ_STATE;

      // Object constructor
      __fastcall                       CloseSequence(void);
      // Object destroyer
      virtual _fastcall                ~CloseSequence();
      // set subject
      int                              SetSubject( TentaclesMgr *ParPuntSubject );
      // start sequence
      void                             StartSequence(HANDLE ParFormHandle, bool ParTelescopeHome, bool ParTrackingOff, bool ParDomeZero, bool ParCloseBShutter, bool ParCloseSShutter, bool ParDomeHome, bool ParPoweroff );
      // get sequence status
      SEQ_STATE                        GetState( int *ParCounter );

   private:
      // notify change in subject
      virtual void                     Update( int ANotifyCode );

   protected:
      TentaclesMgr                     *PuntSubject;        // pointer to subject
      HANDLE                           FormHandle;          // handle of the parent form
      bool                             TelescopeHome;       // required telescope in home position
      bool                             TrackingOff;         // required turn off tracking motor
      bool                             DomeZero;            // required dome in zero position
      bool                             CloseBShutter;       // required balancing shutter closing
      bool                             CloseSShutter;       // required sliding shutter closing
      bool                             DomeHome;            // required dome in home position
      bool                             PowerOff;            // required general power off
      SEQ_STATE                        State;               // actual sequence state
      bool                             Control;             // control activated
      int                              Counter;             // progress counter

      // Threaded function
      void __fastcall                  Execute(void);
};
//---------------------------------------------------------------------------
#endif
