/******************************************************************************

	Config.cpp : Polypus configuration form

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : july 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <vcl.h>
#pragma hdrstop
#include <dir.h>



// project include
#include "Config.h"
#include "PolypusMF.h"
#pragma package(smart_init)
#pragma resource "*.dfm"



// global variables
TForm1 *Form1;



/*---------------------------------------------------------------------------

   Costructor

---------------------------------------------------------------------------*/

__fastcall TForm1::TForm1(TComponent* Owner)
   : TForm(Owner)
{
}



/*---------------------------------------------------------------------------

   Show form

---------------------------------------------------------------------------*/

void __fastcall TForm1::FormShow(TObject *Sender)
{
   int                                 Index;               // index for observer scanning
   // select Observatory page
   ConfigPageControl->ActivePageIndex = 0;

   // fill Observatory page information with configuration data
   LongHoursEdit->Text = MainForm->ObsLongitude.hh;
   LongMinEdit->Text = MainForm->ObsLongitude.mm;
   LongSecEdit->Text = MainForm->ObsLongitude.ss;
   LatSignEdit->Text = MainForm->ObsLatitude.sign;
   LatGradEdit->Text = MainForm->ObsLatitude.gg;
   LatMinEdit->Text = MainForm->ObsLatitude.mm;
   LatSecEdit->Text = MainForm->ObsLatitude.ss;
   TelescopeEdit->Text = MainForm->Telescope;
   ObserversMemo->Clear();
   for ( Index=0; Index<MainForm->NumObserver; Index++)
      ObserversMemo->Lines->Add(MainForm->Observers[Index]);

   // fill directory page information win configuration data
   ShotDirEdit->Text = MainForm->ShotFolder;
   DarkDirEdit->Text = MainForm->DarkFolder;
   GSCDirEdit->Text = MainForm->GSCFolder;
   GSCResolutionDirEdit->Text = MainForm->GSCResolutionFolder;
   CatalogDirEdit->Text = MainForm->ObjectsFolder;
   ScriptDirEdit->Text = MainForm->ScriptFolder;
   SNDirEdit->Text = MainForm->SNFolder;

   // fill CCD page information win configuration data
   CCDNameComboBox->Text = MainForm->CCDName;
   CCDRowsEdit->Text = MainForm->FrameHeight;
   CCDColumnEdit->Text = MainForm->FrameWidth;
   CCDScaleEdit->Text = MainForm->PixelScale;

   // fill miscellaneus page information win configuration data
   SerialPowerEdit->Text = MainForm->SerialPower;
   SerialLX200Edit->Text = MainForm->SPortNumber;
   BaudRateEdit->Text = MainForm->SPortBaudRate;
   SelectionsRadiusEdit->Text = MainForm->SelectionRadius;
   CommTimeoutCheckBox->Checked = MainForm->EnableCommTimeout;
   DiaryCheckBox->Checked = MainForm->EnableDiary;
   SafeElevationEdit->Text = MainForm->SafeElevation;
   MaxMagResolutionEdit->Text = MainForm->MaxMagResolution;
   PECCheckBox->Checked = MainForm->EnablePEC;
   PECAmplitudeEdit->Text = MainForm->PECAmplitude;
   PECPhaseEdit->Text = MainForm->PECPhase;
}



/*---------------------------------------------------------------------------

   Select shot dir

---------------------------------------------------------------------------*/

void __fastcall TForm1::ShotDirButtonClick(TObject *Sender)
{
   AnsiString                             Path;             // selected path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   if ( SelectDir->Execute() == false )
      return;

   Path = SelectDir->FileName;                              // get file name
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path
   ShotDirEdit->Text = Path;                                // update directory
}



/*---------------------------------------------------------------------------

   Select dark dir

---------------------------------------------------------------------------*/

void __fastcall TForm1::DarkDirButtonClick(TObject *Sender)
{
   AnsiString                             Path;             // selected path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   if ( SelectDir->Execute() == false )
      return;

   Path = SelectDir->FileName;                              // get file name
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path
   DarkDirEdit->Text = Path;                                // update directory
}



/*---------------------------------------------------------------------------

   Select GSC dir

---------------------------------------------------------------------------*/

void __fastcall TForm1::GSCDirButtonClick(TObject *Sender)
{
   AnsiString                             Path;             // selected path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   if ( SelectDir->Execute() == false )
      return;

   Path = SelectDir->FileName;                              // get file name
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path
   GSCDirEdit->Text = Path;                                 // update directory
}



/*---------------------------------------------------------------------------

   Select GSC resolution dir

---------------------------------------------------------------------------*/

void __fastcall TForm1::GSCPinpointDirButtonClick(TObject *Sender)
{
   AnsiString                             Path;             // selected path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   if ( SelectDir->Execute() == false )
      return;

   Path = SelectDir->FileName;                              // get file name
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path
   GSCResolutionDirEdit->Text = Path;                       // update directory
}



/*---------------------------------------------------------------------------

   Select catalogs dir

---------------------------------------------------------------------------*/

void __fastcall TForm1::CatalogDirButtonClick(TObject *Sender)
{
   AnsiString                             Path;             // selected path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   if ( SelectDir->Execute() == false )
      return;

   Path = SelectDir->FileName;                              // get file name
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path
   CatalogDirEdit->Text = Path;                             // update directory
}



/*---------------------------------------------------------------------------

   Select script dir

---------------------------------------------------------------------------*/

void __fastcall TForm1::ScriptDirButtonClick(TObject *Sender)
{
   AnsiString                             Path;             // selected path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   if ( SelectDir->Execute() == false )
      return;

   Path = SelectDir->FileName;                              // get file name
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path
   ScriptDirEdit->Text = Path;                              // update directory
}



/*---------------------------------------------------------------------------

   conferme new parameters

---------------------------------------------------------------------------*/

void __fastcall TForm1::OkButtonClick(TObject *Sender)
{
   int                                 Index;               // index for observer scanning
   AnsiString                          Observer;            // observer from memo list

   // update Observatory page information
   MainForm->ObsLongitude.hh = atoi( LongHoursEdit->Text.c_str() );
   MainForm->ObsLongitude.mm = atoi( LongMinEdit->Text.c_str() );
   MainForm->ObsLongitude.ss = atoi( LongSecEdit->Text.c_str() );
   strcpy( MainForm->ObsLatitude.sign, LatSignEdit->Text.c_str() );
   MainForm->ObsLatitude.gg = atoi( LatGradEdit->Text.c_str() );
   MainForm->ObsLatitude.mm = atoi( LatMinEdit->Text.c_str() );
   MainForm->ObsLatitude.ss = atoi( LatSecEdit->Text.c_str() );
   strcpy( MainForm->Telescope, TelescopeEdit->Text.c_str() );

   MainForm->NumObserver = 0;                               // re count observers number
   for ( Index=0; Index<ObserversMemo->Lines->Count; Index ++ )
   {
      Observer = ObserversMemo->Lines->Strings[Index];      // get observer
      if ( Observer == "" )
         continue;                                          // skip unused lines
      strcpy( MainForm->Observers[MainForm->NumObserver], Observer.c_str() );  // copy name
      MainForm->NumObserver++;                              // count observer
   }

   // update directory page information
   strcpy( MainForm->ShotFolder, ShotDirEdit->Text.c_str() );
   strcpy( MainForm->DarkFolder, DarkDirEdit->Text.c_str() );
   strcpy( MainForm->GSCFolder, GSCDirEdit->Text.c_str() );
   strcpy( MainForm->GSCResolutionFolder, GSCResolutionDirEdit->Text.c_str() );
   strcpy( MainForm->ObjectsFolder, CatalogDirEdit->Text.c_str() );
   strcpy( MainForm->ScriptFolder, ScriptDirEdit->Text.c_str() );
   strcpy( MainForm->SNFolder, SNDirEdit->Text.c_str() );

   // update CCD page information
   strcpy( MainForm->CCDName, CCDNameComboBox->Text.c_str() );
   MainForm->FrameHeight = atoi( CCDRowsEdit->Text.c_str() );
   MainForm->FrameWidth = atoi( CCDColumnEdit->Text.c_str() );
   MainForm->PixelScale = atof( CCDScaleEdit->Text.c_str() );

   // update miscellaneus page information
   MainForm->SerialPower = atoi( SerialPowerEdit->Text.c_str() );
   MainForm->SPortNumber = atoi( SerialLX200Edit->Text.c_str() );
   MainForm->SPortBaudRate = atoi( BaudRateEdit->Text.c_str() );
   MainForm->SelectionRadius = atoi( SelectionsRadiusEdit->Text.c_str() );
   MainForm->EnableCommTimeout = CommTimeoutCheckBox->Checked;
   MainForm->EnableDiary = DiaryCheckBox->Checked;
   MainForm->SafeElevation = atoi( SafeElevationEdit->Text.c_str() );
   MainForm->MaxMagResolution = atoi( MaxMagResolutionEdit->Text.c_str() );
   MainForm->EnablePEC = PECCheckBox->Checked;
   MainForm->PECAmplitude = atoi( PECAmplitudeEdit->Text.c_str() );
   MainForm->PECPhase = atoi( PECPhaseEdit->Text.c_str() );
}


void __fastcall TForm1::SNDirButtonClick(TObject *Sender)
{
   AnsiString                             Path;             // selected path
   AnsiString                             FileName;         // configuration file name
   int                                    Pos;              // delimiter postion

   // let use to select a file to define the directory
   if ( SelectDir->Execute() == false )
      return;

   Path = SelectDir->FileName;                              // get file name
   Pos = Path.LastDelimiter("\\");                          // find last '\' before it there is the path
   Path = Path.SubString(1,Pos);                            // get path
   SNDirEdit->Text = Path;                                  // update directory
}
//---------------------------------------------------------------------------


