/******************************************************************************

	ElevationGraph.h | Header of Elevation Graphics manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2012

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef ELEVATIONGRAPHH
#define ELEVATIONGRAPHH



// system include
#include <vcl.h>
#pragma hdrstop



// project include
#include "CMPlanet.h"
#include "CMObjectList.h"



#define                       ELEVATION_PRECISION  0.5     // elevation precision
#define                       MAX_OBJECT  20               // maximum elevation object


/******************************************************************************

	ElevationGraph manager class

******************************************************************************/

class ElevationGraph
{
   public:
      // mixed definition
      typedef enum
      {
         RULER_SIZEY = 20,                                  // time ruler vertical size
      } DEFINE;

      // object constructor
                                       ElevationGraph( );
      // object destroyer
      virtual                          ~ElevationGraph();
      // set X axel ( hours )
      void                             SetX( int Start, int Stop, int Grid );
      // set Y Axel ( elevation)
      void                             SetY( int Start, int Stop, int Grid );
      // set date
      void                             SetDate( double PJD );
      // set site position
      void                             SetSite ( CMCoLocal *PSite );
      // set object list
      void                             SetList( CMObjectList *PList );
      // set data color
      void                             SetColor( TColor *AColor, int AObjectCount );
      // get graph bitmap
      void                             GetBmp( Graphics::TBitmap ** );

  protected:
      double                           JD;                  // graphic JD
      CMCoLocal                        *Site;               // site position
      CMPlanet                         *Sun;                // sun object
      CMPlanet                         *Earth;              // earth object
      double                           StartX;              // minimum value in X axel
      double                           StopX;               // maximum value in X axel
      double                           GridX;               // grid size in X axel
      double                           StartY;              // minimum value in Y axel
      double                           StopY;               // maximum value in Y axel
      double                           GridY;               // grid size in Y axel
      double                           SunsetTime;
      double                           NightIniTime;
      double                           NightEndTime;
      double                           SunriseTime;
      CMObjectList                     *List;
      int                              ObjectCount;         // object to manage
      TColor                           TabColor[MAX_OBJECT];
};

#endif
