/*---------------------------------------------------------------------------

	Fits.cpp  oggetto di gestine file fits

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : March 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus and SNF
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>
#include <math.h>



// project include
#include "Fits.h"




/*---------------------------------------------------------------------------

   contructor

---------------------------------------------------------------------------*/

Fits::Fits()
{
   Image=0;
   First=0;
   SizeX=0;
   SizeY=0;
   Base=0;
}



/*---------------------------------------------------------------------------

	copy constructor

---------------------------------------------------------------------------*/

Fits::Fits( Fits *Copy )
{
   int                                 TotSize;             // total image size
   FITS_CHAIN*                         NowOld;              // old element in block chain
   FITS_CHAIN*                         NowNew;              // current element in new block chain
   FITS_CHAIN*                         PrevNew;             // previous element in new block chain

   // initialization
   Image=0;
   First=0;
   SizeX=0;
   SizeY=0;
   Base=0;

   // if image is present copy all
   if ( Copy->Image != 0 )
   {
      // copy object parameters
      SizeX = Copy->SizeX;
      SizeY = Copy->SizeY;
      Base = Copy->Base;
      // copy image
      TotSize = SizeX * SizeY*2;
      Image = (unsigned short*) malloc(TotSize);            // image memory allocation
      memcpy( Image, Copy->Image, TotSize );
      // copy block parameter
      NowOld = Copy->First;                                 // first parameter
      PrevNew = 0;
      while ( NowOld != 0 )                                 // until paramater are present
      {
         NowNew = (FITS_CHAIN*)malloc( sizeof(FITS_CHAIN) );// allocation for a new block
         memcpy( NowNew, NowOld, sizeof(FITS_CHAIN) );      // block copy
         NowNew->Next = 0;                                  // reser next element
         if ( First == 0 )                                  // there is the first?
            First = NowNew;                                 // this is the fisrt
         else
            PrevNew->Next = NowNew;                         // this is the next of previous
         PrevNew = NowNew;                                  // this is next previouso
         NowOld = NowOld->Next;                             // go to the next block
      }
   }
}



/*---------------------------------------------------------------------------

   destroyer

---------------------------------------------------------------------------*/

Fits::~Fits()
{
   FreeChain();                                             // free chain
   if ( Image != 0 )                                        // free buffer image
      free( Image );
}



/*---------------------------------------------------------------------------

   load file from disk

---------------------------------------------------------------------------*/

Fits::ERR Fits::LoadFile( char *FileName)
{
   ERR                                 Err;                 // error code
   HANDLE                              hFile;               // file handle
   FITS_BLOCK                          Block;               // parameter block
   DWORD                               ToRead;              // byte to read
   DWORD                               Read;                // byte read
   bool                                End;                 // end tag found
   int                                 Index;               // index for paramater scan
   FITS_CHAIN*                         Prev;                // previous block element
   FITS_CHAIN*                         Now;                 // current block element
   double                              Value;               // value from file
   int                                 BitPix;              // number of bit per pixel
   unsigned short*                     TempImage;           // temp image
   unsigned short                      *p1,*p2;             // pointer for image conversion
   unsigned long                       *p3;                 // pointer for 32 bit image read
   int                                 TotPixel;            // total pixel in the image
   char                                ValueStr[71];        // single value;
   bool                                Pixel32;             // every pixel use 32 bits

   // free previous imformation
   FreeChain();                                             // free block chain
   if ( Image != 0 )                                        // free buffer image
   {
      free( Image );
      Image = 0;
   }

   // open file
   hFile = CreateFile( FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile == INVALID_HANDLE_VALUE )
      return ERR_FILENOTOPEN;

   // load block until END tag found
   Err = ERR_NONE;
   End = false;
   Base = 0;
   Prev = 0;
   Pixel32 = false;
   do
   {
      ToRead = sizeof(Block);                               // block size
      ReadFile ( hFile, &Block, ToRead, &Read, NULL);       // block read
      if ( Read != ToRead )                                 // read check
      {
         Err = ERR_FILENOTREAD;
         break;
      }
      for ( Index=0; Index<FITS_NUMVALUES; Index++)
      {
         if ( strncmp( Block.Param[Index].Key, "END     ", 8 ) == 0 )   // check for END tag
         {
            End = true;
            break;
         }
         Now = (FITS_CHAIN*)malloc( sizeof(FITS_CHAIN) );   // block allocation
         memcpy( Now->Actual.Key, Block.Param[Index].Key, 8 );    // values copy
         memcpy( Now->Actual.Value, Block.Param[Index].Value, 70);
         Now->Next = 0;
         if ( First == 0 )                                  // is the first?
            First = Now;                                    // this is the first
         else
            Prev->Next = Now;                               // this the next of previous
         Prev = Now;
      }
   } while ( End == false);


   // check for single image
   if ( Err == 0 )
   {
      if ( GetValue( "SIMPLE  ", ValueStr) != ERR_NONE )
         Err = ERR_IMAGETOOCOMPLEX;
      else
      {
         if ( ValueStr[19] != 'T' )                         // it should be true
            Err = ERR_IMAGETOOCOMPLEX;
      }
   }

   // get image sise
   if ( Err == 0 )
   {
      // get axis number (should be 2)
      if ( GetValueNum( "NAXIS   ", &Value) != 0 )
         Err = ERR_TAGNOTFOUND;
      else
         if ( Value != 2 )
            Err = ERR_IMAGETOOCOMPLEX;
   }
   if ( Err == 0 )
   {
      // get x axis size
      if ( GetValueNum( "NAXIS1  ", &Value) != 0 )
         Err = ERR_TAGNOTFOUND;
      SizeX = Value;
   }
   if ( Err == 0 )
   {
      // get y axis size
      if ( GetValueNum( "NAXIS2  ", &Value) != 0 )
         Err = ERR_TAGNOTFOUND;
      SizeY = Value;
   }
   if ( Err == 0 )
   {
      // get pixel number (it should be 16)
      if ( GetValueNum( "BITPIX  ", &Value) != 0 )
         Err = ERR_TAGNOTFOUND;
      else
      {
         BitPix = Value;
         if ( BitPix != 16 && BitPix != 32 )
            Err = ERR_IMAGETOOCOMPLEX;
         if ( BitPix == 32 )
         {
            Pixel32 = true;
            strncpy( Current->Actual.Value+18, "16",2 );
         }
      }
   }
   // get base value
   if ( GetValueNum( "BZERO   ", &Value) == 0 )
      Base = Value;

   // get image
   TempImage = 0;
   if ( Err == 0 )
   {
      ToRead = SizeX*SizeY*2;                               // image size
      if ( Pixel32 == true )                                // double size for 32 bits image
         ToRead *= 2;
      TempImage = (unsigned short*) malloc(ToRead);         // temporary image buffer allocation
      ReadFile ( hFile, TempImage, ToRead, &Read, NULL);    // get image
      if ( Read != ToRead)
      {
         Err = ERR_FILENOTREAD;
      }
   }

   // tranform data in integer mode and arrange base value
   if ( Err == 0 )
   {
      ToRead = SizeX*SizeY*2;                               // image size
      Image = (unsigned short*) malloc(ToRead);             // final image buffer allocation
      TotPixel = SizeX*SizeY;                               // image size (in pixel)
      p1 = Image;                                           // write pointer
      p2 = TempImage;                                       // read pointer
      p3 = (unsigned long*)TempImage;                       // read pointer for 32 bit image

      if ( Pixel32 == false )                               // depending on image size
      {
         for ( Index=0; Index<TotPixel; Index++ )
         {
            *p1++ = (unsigned short)(GiraW( *p2++ ) - Base);   // convert pixel value
         }
      }
      else
      {
         for ( Index=0; Index<TotPixel; Index++ )
         {
            *p1++ = (unsigned short)(GiraL( *p3++ ) - Base);   // convert pixel value
         }
         Base = 32768;
         SetValueNum( "BZERO   ", Base);
      }
   }
   if ( TempImage != 0 )                                    // free temporaty image
      free ( TempImage );

   // file close
   CloseHandle( hFile);
   
   // yeld error code
   return Err;
}



/*---------------------------------------------------------------------------

   save file to disk

---------------------------------------------------------------------------*/

Fits::ERR Fits::SaveFile( char *FileName)
{
   ERR                                 Err;                 // error code
   HANDLE                              hFile;               // file handle
   FITS_BLOCK                          Block;               // block parameter
   DWORD                               ToWrite;             // bytes to write
   DWORD                               Written;             // written bytes
   bool                                End;                 // added END tag
   int                                 Index;               // index for paramater scan
   FITS_CHAIN*                         Now;                 // current block element
   unsigned short*                     TempImage;           // temporary image
   unsigned short                      *p1,*p2;             // pointer for image conversion
   int                                 TotPixel;            // total pixel in the image

   if ( Image == 0 )                                        // image not present
      return ERR_NOIMAGE;

   // open file
   hFile = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,	FILE_ATTRIBUTE_ARCHIVE, NULL );
   if ( hFile == INVALID_HANDLE_VALUE )
      return ERR_FILENOTOPEN;

   // write block with all parameters
   Err = ERR_NONE;
   End = false;
   Now = First;
   do
   {
      memset( &Block, ' ', sizeof(Block));                  // block initialization
      for ( Index=0; Index<FITS_NUMVALUES; Index++)         // fill block
      {
         if ( Now == 0 )                                    // if block chain is finished no other parameter
            break;
         memcpy( Block.Param[Index].Key,   Now->Actual.Key ,8 );  // value copy
         Block.Param[Index].Ass[0] = '=';
         memcpy( Block.Param[Index].Value, Now->Actual.Value,70);
         Now = Now->Next;                                   // goto next parameter
      }
      if ( Now == 0 && Index<FITS_NUMVALUES )               // if finish and there is space add END tag
      {
         memcpy( Block.Param[Index].Key, "END     " ,8 );   // END tag
         End = true;                                        // finisch
      }
      ToWrite = sizeof(Block);                              // block size
      WriteFile ( hFile, &Block, ToWrite, &Written, NULL);  // block write
      if ( Written != ToWrite )                             // check write
      {
         Err = ERR_FILENOTWRITTEN;
         break;
      }
   } while ( End == false);                                 // contine until END tag is written

   // put data in Motorola format and add base
   TempImage = 0;
   if ( Err == 0 )
   {
      TotPixel = SizeX*SizeY;                               // number of pixel
      TempImage = (unsigned short*) malloc(TotPixel*2);     // write buffer allocation
      p1 = TempImage;                                       // write pointer
      p2 = Image;                                           // read pointer
      for ( Index=0; Index<TotPixel; Index++ )
      {
         *p1++ = GiraW( (unsigned short)(*p2++ + Base) );     // arrange pixel value
      }
   }

   // image write
   if ( Err == 0 )
   {
      ToWrite = TotPixel*2;
      WriteFile ( hFile, TempImage, ToWrite, &Written, NULL);
      if ( Written != ToWrite )                             // check write
         Err = ERR_FILENOTWRITTEN;
   }
   if ( TempImage != 0 )
      free ( TempImage );

   // file close
   CloseHandle( hFile);

   // yeld error code
   return Err;
}



/*---------------------------------------------------------------------------

   set image

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetImage( int DimY, int DimX, int ParBase, unsigned short *ImageBuffer)
{
   int                                 TotSize;                // image size
   ERR                                 Err;                    // error code

   // remove previous information
   FreeChain();                                                // remove parameter chain
   if ( Image != 0 )                                           // remove buffer image
   {
      free( Image );
      Image = 0;
   }

   // set object values
   SizeX = DimX;
   SizeY = DimY;
   Base = (unsigned short) ParBase;

   // set image
   TotSize = SizeX * SizeY * 2;
   Image = (unsigned short*) malloc(TotSize);                  // image buffer allocation
   memcpy( Image, ImageBuffer, TotSize );

   // create mandatory parameters
   Err = SetValueBool( "SIMPLE  ", true);
   if ( Err == 0 )
      Err = SetValueNum( "BITPIX  ", 16);
   if ( Err == 0 )
      Err = SetValueNum( "NAXIS   ", 2);
   if ( Err == 0 )
      Err = SetValueNum( "NAXIS1  ", SizeX);
   if ( Err == 0 )
      Err = SetValueNum( "NAXIS2  ", SizeY);
   if ( Err == 0 )
      SetValueNum( "BZERO   ", Base);

   return Err;
}



/*---------------------------------------------------------------------------

   get image size

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetImageSize( int* DimY, int* DimX )
{
   if ( Image == 0 )
      return ERR_NOIMAGE;
      
   *DimY = SizeY;
   *DimX = SizeX;
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   get image

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetImage( unsigned short *ImageBuffer )
{
   int                                 TotalSize;

   // check image
   if ( Image == 0 )
      return ERR_NOIMAGE;

   // compute image size
   TotalSize = SizeX*SizeY*2;

   memcpy(ImageBuffer, Image, TotalSize);
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   set temperature

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetTemperature( double Temp)
{
   ERR                                 Err;

   Err = SetValueNum( "TEMPERAT", Temp );

   return Err;
}



/*---------------------------------------------------------------------------

   get temperature

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetTemperature( double *Temp)
{
   if ( GetValueNum( "TEMPERAT", Temp ) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   set shot parameters

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetShotPar( SYSTEMTIME *Time, double Exp)
{
   ERR                                 Err;                 // error code
   char                                ValueStr[71];        // string value
   char                                Str[50];             // temp string
   char                                Str2[50];
   char                                c[10];
   int                                 Len;

   // set exposure time
   Err = SetValueNum( "EXPOSURE", Exp);

   // set timestamp in UT
   if ( Err == 0 )
   {
      memset( ValueStr, ' ', sizeof(ValueStr));             // clear value
      strcpy( Str, "'");
      itoa ( Time->wYear+10000, c, 10 );                    // year conversion
      strcat( Str, c+1);
      strcat( Str, "-");
      itoa ( Time->wMonth+100, c, 10 );                     // month conversion
      strcat( Str, c+1);
      strcat( Str, "-");
      itoa ( Time->wDay+100, c, 10 );                       // day convserion
      strcat( Str, c+1);
      strcat( Str, "T");
      itoa ( Time->wHour+100, c, 10 );                      // hour conversion
      strcat( Str, c+1);
      strcat( Str, ":");
      itoa ( Time->wMinute+100, c, 10 );                    // minute conversion
      strcat( Str, c+1);
      strcat( Str, ":");
      itoa ( Time->wSecond+100, c, 10 );                    // second conversion
      strcat( Str, c+1);
      strcat( Str, ".");
      itoa ( Time->wMilliseconds/100, c, 10 );              // millisecond conversion
      strcat( Str, c);
      strcpy( Str2, Str );
      strcat( Str, "' / UT");
      Len = strlen(Str);
      memcpy( ValueStr, Str, Len );
      Err = SetValue( "DATE-OBS", ValueStr );
   }

   // set time stamp un local time
   if ( Err == 0 )
   {
      memset( ValueStr, ' ', sizeof(ValueStr));    // clear value
      strcat( Str2, "' / LT");
      memcpy( ValueStr, Str2, Len );
      Err = SetValue( "DATE-LOC", ValueStr );
   }

   return Err;
}



/*---------------------------------------------------------------------------

   get shot parameters

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetShotPar( SYSTEMTIME *Time, double *Exp)
{
   char                                ValueStr[71];        // string value
   char                                c[10];               // temp string

   // get exposure time
   if ( GetValueNum( "EXPOSURE", Exp) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // get timestamo
   if ( GetValue( "DATE-OBS", ValueStr ) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // year conversion
   c[0] = ValueStr[1];
   c[1] = ValueStr[2];
   c[2] = ValueStr[3];
   c[3] = ValueStr[4];
   c[4] = 0;
   Time->wYear = (WORD)atoi( c);

   // mount convesion
   c[0] = ValueStr[6];
   c[1] = ValueStr[7];
   c[2] = 0;
   Time->wMonth = (WORD)atoi( c);

   // day conversion
   c[0] = ValueStr[9];
   c[1] = ValueStr[10];
   c[2] = 0;
   Time->wDay = (WORD)atoi( c);

   // hour conversion
   c[0] = ValueStr[12];
   c[1] = ValueStr[13];
   c[2] = 0;
   Time->wHour = (WORD)atoi( c);

   // minute conversion
   c[0] = ValueStr[15];
   c[1] = ValueStr[16];
   c[2] = 0;
   Time->wMinute = (WORD)atoi( c);

   // second conversion
   c[0] = ValueStr[18];
   c[1] = ValueStr[19];
   c[2] = 0;
   Time->wSecond = (WORD)atoi( c);

   // millisecond conversion
   c[0] = ValueStr[21];
   c[1] = 0;
   Time->wMilliseconds = (WORD)(atoi( c)*100);

   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   set coordinate

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetPosition( CMHours::HOURS *Ar, CMDegree::DEGREE *Dec, double YearEquinox)
{
   ERR                                 Err;                 // error code
   char                                ValueStr[71];        // string value
   char                                Str[50];             // temp string
   char                                c[10];
   int                                 Len;

   // set right ascension
   memset( ValueStr, ' ', sizeof(ValueStr));                // clear value
   strcpy( Str, "'");
   itoa ( Ar->hh+100, c, 10 );                              // hour conversion
   strcat( Str, c+1);
   strcat( Str, " ");
   itoa ( Ar->mm+100, c, 10 );                              // minute conversion
   strcat( Str, c+1);
   strcat( Str, " ");
   gcvt ( Ar->ss+100, 5, c );                               // second conversion
   strcat( Str, c+1);
   strcat( Str, "'");
   Len = strlen(Str);
   memcpy( ValueStr, Str, Len );
   Err = SetValue( "IMAGERA ", ValueStr );
   if ( Err == 0 )
      Err = SetValue( "OBJCTRA ", ValueStr );

   // set declination
   if ( Err == 0 )
   {
      memset( ValueStr, ' ', sizeof(ValueStr));             // clear value
      strcpy( Str, "'");
      Str[1] = Dec->sign[0];                                // put sign
      Str[2] = 0;
      itoa ( Dec->gg+100, c, 10 );                          // degree conversion
      strcat( Str, c+1);
      strcat( Str, " ");
      itoa ( Dec->mm+100, c, 10 );                          // minute conversion
      strcat( Str, c+1);
      strcat( Str, " ");
      gcvt ( Dec->ss+100, 4, c );                           // second conversion
      strcat( Str, c+1);
      strcat( Str, "'");
      Len = strlen(Str);
      memcpy( ValueStr, Str, Len );
      Err = SetValue( "IMAGEDEC", ValueStr );
      if ( Err == 0 )
         Err = SetValue( "OBJCTDEC ", ValueStr );
   }

   // set epoch
   if ( Err == 0 )
      Err = SetValueNum( "EQUINOX ", YearEquinox);

   return Err;
}



/*---------------------------------------------------------------------------

   get coordinate

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetPosition( CMHours::HOURS *Ar, CMDegree::DEGREE *Dec, double *YearEquinox)
{
   char                                ValueStr[71];        // string value
   char                                c[10];               // temp

   // get righ ascension
   if ( GetValue( "IMAGERA ", ValueStr) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // hour conversion
   c[0] = ValueStr[1];
   c[1] = ValueStr[2];
   c[2] = 0;
   Ar->hh = atoi( c);

   // minute conversion
   c[0] = ValueStr[4];
   c[1] = ValueStr[5];
   c[2] = 0;
   Ar->mm = atoi( c);

   // second conversion
   c[0] = ValueStr[7];
   c[1] = ValueStr[8];
   c[2] = ValueStr[9];
   c[3] = ValueStr[10];
   c[4] = ValueStr[11];
   c[5] = 0;
   Ar->ss = atof( c);

   // get declination
   if ( GetValue( "IMAGEDEC", ValueStr) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // get sign
   Dec->sign[0] = ValueStr[1];
   Dec->sign[1] = 0;

   // degree conversion
   c[0] = ValueStr[2];
   c[1] = ValueStr[3];
   c[2] = 0;
   Dec->gg = atoi( c);

   // minute conversion
   c[0] = ValueStr[5];
   c[1] = ValueStr[6];
   c[2] = 0;
   Dec->mm = atoi( c);

   // second conversion
   c[0] = ValueStr[8];
   c[1] = ValueStr[9];
   c[2] = ValueStr[10];
   c[3] = ValueStr[11];
   c[4] = 0;
   Dec->ss = atof( c);

   // get epoch
   if ( GetValueNum( "EQUINOX ", YearEquinox) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   set object name

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetObject( char *Object)
{
   ERR                                 Err;                 // error code
   char                                ValueStr[71];        // string value
   char                                Str[50];             // temp string
   int                                 Len;

   // set object name
   memset( ValueStr, ' ', sizeof(ValueStr));
   strcpy( Str, "'");
   strcat( Str, Object);
   strcat( Str, "'");
   Len = strlen(Str);
   memcpy( ValueStr, Str, Len );
   Err = SetValue( "OBJECT  ", ValueStr );

   return Err;
}



/*---------------------------------------------------------------------------

   get object name

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetObject( char *Object)
{
   char                                ValueStr[71];        // string value
   int                                 i;                   // index

   // search tag
   if ( GetValue( "OBJECT  ", ValueStr) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // searching for second ' in the string
   for ( i=1; i<71; i++)
   {
      if ( ValueStr[i] == '\'' )
         ValueStr[i] = 0;                                   // replace with string terminator
   }
   strcpy( Object, ValueStr+1);

   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   set observatory parameters

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetObservatoryPar( char *Telescope, char *Instrument, char *Observer)
{
   ERR                                 Err;                 // error code
   char                                ValueStr[71];        // string value
   char                                Str[50];             // temp string
   int                                 Len;

   // set telescope
   memset( ValueStr, ' ', sizeof(ValueStr));
   strcpy( Str, "'");
   strcat( Str, Telescope);
   strcat( Str, "'");
   Len = strlen(Str);
   memcpy( ValueStr, Str, Len );
   Err = SetValue( "TELESCOP", ValueStr );

   // set instrument
   if ( Err == 0 )
   {
      memset( ValueStr, ' ', sizeof(ValueStr));
      strcpy( Str, "'");
      strcat( Str, Instrument);
      strcat( Str, "'");
      Len = strlen(Str);
      memcpy( ValueStr, Str, Len );
      Err = SetValue( "INSTRUME", ValueStr );
   }

   // set observers
   if ( Err == 0 )
   {
      memset( ValueStr, ' ', sizeof(ValueStr));
      strcpy( Str, "'");
      strcat( Str, Observer);
      strcat( Str, "'");
      Len = strlen(Str);
      memcpy( ValueStr, Str, Len );
      Err = SetValue( "OBSERVER", ValueStr );
   }

   return Err;
}



/*---------------------------------------------------------------------------

   get observatory parameters

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetObservatoryPar( char *Telescope, char *Instrument, char *Observer)
{
   char                                ValueStr[71];        // string value
   int                                 i;

   // get telescope
   if ( GetValue( "TELESCOP", ValueStr) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // searching for second ' in the string
   for ( i=1; i<71; i++)
   {
      if ( ValueStr[i] == '\'' )
         ValueStr[i] = 0;                                   // replace with string terminator
   }
   strcpy( Telescope, ValueStr+1);

   // get CCD
   if ( GetValue( "INSTRUME", ValueStr) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // searching for second ' in the string
   for ( i=1; i<71; i++)
   {
      if ( ValueStr[i] == '\'' )
         ValueStr[i] = 0;                                   // replace with string terminator
   }
   strcpy( Instrument, ValueStr+1);

   // get observers
   if ( GetValue( "OBSERVER", ValueStr) != ERR_NONE )
      return ERR_TAGNOTFOUND;

   // searching for second ' in the string
   for ( i=1; i<71; i++)
   {
      if ( ValueStr[i] == '\'' )                            // replace with string terminator
         ValueStr[i] = 0;
   }
   strcpy( Observer, ValueStr+1);

   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   Set generic value

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetValue( char *Key, char *Value )
{
   FITS_CHAIN*                         Last;                // last element i block chain
   FITS_CHAIN*                         New;                 // new element in blocl chain

   // get last chain element
   Last = First;
   if ( Last != 0 )
   {
      while ( Last->Next != 0 )
         Last = Last->Next;
   }

   // create new parameter
   New = (FITS_CHAIN*)malloc( sizeof(FITS_CHAIN) );         // alloco spazio per parametro caricato
   memcpy( New->Actual.Key, Key, 8 );                       // copio i valori
   memcpy( New->Actual.Value, Value, 70);
   New->Next = 0;

   // put at the end
   if ( Last == 0 )
      First = New;
   else
      Last->Next = New;
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   get generic value

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetValue( char *Key, char *Value)
{
   FITS_CHAIN*                         Now;                 // actual element

   Now = First;                                             // start at begining

   while ( Now != 0 )
   {
      if ( strncmp( Now->Actual.Key, Key, 8 ) == 0 )        // it is my key?
      {
         memcpy( Value, Now->Actual.Value, 70);             // copy value
         Current = Now;
         return ERR_NONE;                                   // succesfull exit
      }
      Now = Now->Next;                                      // goto next
   }
   return ERR_VALUENOTFOUND;                                // not found
}



/*---------------------------------------------------------------------------

   get to first value

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetFirst( FITS_VALUE **Now )
{
   Previous = NULL;                                         // no previous
   Current = First;                                         // set first as current
   *Now = &Current->Actual;                                 // copy value
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   get next value element

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetNext( FITS_VALUE **Now )
{
   if ( Current == NULL )                                   // check if current is already over
      return ERR_ENDPARAMETERS;

   Previous = Current;                                      // save current as previous
   Current = Current->Next;                                 // set on next element
   if ( Current == NULL )                                   // check if current is over
      return ERR_ENDPARAMETERS;

   *Now = &Current->Actual;                                 // copy value
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   remove current value

---------------------------------------------------------------------------*/

Fits::ERR Fits::Remove( void )
{
   if ( Current == NULL )                                   // check if current is already over
      return ERR_ENDPARAMETERS;

   if ( Previous == NULL )                                  // first element remove ?
   {
      First = Current->Next;                                // assign first
      free( Current );                                      // free actual element
      Current = First;                                      // new current is the new first
   }
   else
   {
      Previous->Next = Current->Next;                       // assign new next to previouis
      free( Current );                                      // free actual element
      Current = Previous;                                   // go back on previous element
   }
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   free current block chain

---------------------------------------------------------------------------*/

Fits::ERR Fits::FreeChain( void )
{
   FITS_CHAIN*                         Now;                 // actual block chain element
   FITS_CHAIN*                         Next;                // next block chain element

   Now = First;                                             // first element
   while ( Now != 0 )
   {
      Next = Now->Next;                                     // get next element
      free( Now );                                          // free actual element
      Now = Next;                                           // goto to the next
   }

   First = 0;                                               // no first element
   return ERR_NONE;
}



/*---------------------------------------------------------------------------

   change word order (Motorola to Intel or vice versa)

---------------------------------------------------------------------------*/

unsigned short Fits::GiraW(unsigned short i)
{
   unsigned short                      o;

   o =  (unsigned short )   (i >> 8);
   o |= (unsigned short ) ( (i & 0xff) << 8 );
   return o;
}



/*---------------------------------------------------------------------------

   change long order (Motorola to Intel or vice versa)

---------------------------------------------------------------------------*/

unsigned long Fits::GiraL(unsigned long i)
{
   unsigned long                      o;

   o =  (unsigned long )   (i >> 24);
   o |= (unsigned long ) ( (i & 0xff0000) >> 8 );
   o |= (unsigned long ) ( (i & 0xff00) << 8 );
   o |= (unsigned long ) ( (i & 0xff) << 24 );
   return o;
}



/*---------------------------------------------------------------------------

   add a boolern value

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetValueBool( char *Key, bool Value )
{
   char                                ValueStr[71];        // string value
   ERR                                 Err;                 // error code

   memset( ValueStr, ' ', sizeof(ValueStr));
   if ( Value == true )
      ValueStr[19] = 'T';
   else
      ValueStr[19] = 'F';

   Err = SetValue ( Key, ValueStr);
   return Err;
}



/*---------------------------------------------------------------------------

   add a numeric value

---------------------------------------------------------------------------*/

Fits::ERR Fits::SetValueNum( char *Key, double Value )
{
   char                                ValueStr[71];        // string value
   ERR                                 Err;                 // error code
   char                                c[20];               // temp string
   int                                 Len;

   memset( ValueStr, ' ', sizeof(ValueStr));
   gcvt( Value, 6, c);
   Len = strlen(c);
   memcpy( ValueStr+20-Len, c, Len);
   Err = SetValue ( Key, ValueStr);
   return Err;
}



/*---------------------------------------------------------------------------

   recupero valore numerico

---------------------------------------------------------------------------*/

Fits::ERR Fits::GetValueNum( char *Key, double *Value)
{
   char                 ValueStr[71];           // valore trovato;
   int                  Err;
   char                 *p1;
   char                 *p2;

   // get raw value
   Err = GetValue( Key, ValueStr);
   if ( Err != ERR_NONE )
      return ERR_VALUENOTFOUND;
   ValueStr[70] = 0;

   // bypass initial blank
   p1 = ValueStr;
   while ( *p1 == ' ' )
      p1++;

   // put string terminator
   p2 = p1;
   while ( *p2 != ' ' && *p2 != 0 )
      p2++;
   *p2 = 0;

   *Value = atof( p1 );
   return ERR_NONE;
}

