/******************************************************************************

	GRB.h | Header of GRB alert detector

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : May 2011

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// include antirepeat
#ifndef GRBH
#define GRBH



// system include
#include <vcl.h>
#include <mapi.h>
#pragma hdrstop



// project include
#include "CMDegree.h"
#include "CMHours.h"
#include "Tokenizer.h"



/******************************************************************************

	GRB alert detector

******************************************************************************/

class GRB
{
   public:

      // mail information
      typedef struct
      {
         CMHours::HOURS                RA;                  // GRB right ascension
         CMDegree::DEGREE              Dec;                 // GRB declination
         int                           TriggerIndex;        // GRB trigger index
      }MAIL_INFO;

      typedef struct LIST_MAIL_Tag
      {
         char                          From[50];            // source of mail
         char                          Subject[100];        // mail subject
         char                          Date[30];            // mail date
         char                          *Text;               // pointer to mail text
         LIST_MAIL_Tag                 *Prev;               // pointer to previous element
         LIST_MAIL_Tag                 *Next;               // pointer to next element
      } LIST_MAIL;

      // error definition
      typedef enum
      {
         ERR_NONE,                                          // no error
         ERR_NOLOG,                                         // is it not possibile mail login
         ERR_NOTFOUND,                                      // element not found
         ERR_ENDOFLIST,                                     // end of list
      } ERR;

      // object constructor
                                       GRB( void);
      // object destroyer
      virtual                          ~GRB();
      // set option
      void                             SetOption ( int MinElevation, int MaxDelay );
      // set status edit
      void                             SetStatusEdit( TEdit *StatusEdit );
      // download mail
      ERR                              DownloadMail(void);
      // get mail count
      int                              GetMailsCount( void );
      // Set list pointer to the first element
      void                             First(void) ;
      // Set list pointer to the last element
      void                             Last(void) ;
      // get previous mail information
      ERR                              GetNextMail( char *From, char *Subject, char *Date, char **Text );
      // get previous mail information
      ERR                              GetPrevMail( char *From, char *Subject, char *Date, char **Text );
      // check if mail is already present
      bool                             IsMailPresent(char *From, char *Subject, char *Date);
      // understand mail
      bool                             UnderstandMail( char *Text, MAIL_INFO *Info );

  protected:
      HANDLE                           hlibMAPI;            // MAPI libary handle
      MAPILOGON                        *MAPILogon;          // MAPI log on function pointer
      MAPILOGOFF                       *MAPILogoff;         // MAPI log off function pointer
      MAPIREADMAIL                     *MAPIReadMail;       // MAPI read mail function pointer
      MAPIFINDNEXT                     *MAPIFindNext;       // MAPI read next mail function pointer

      TEdit                            *StatusEdit;         // status edit
      int                              MinElevation;        // minimum elevation in degree for alert taking care
      int                              MaxDelay;            // maximum delay in minutes for alert taking take
      LIST_MAIL                        *FirstElement;       // first element in the list
      LIST_MAIL                        *LastElement;        // last element in the list
      LIST_MAIL                        *ScanElement;        // scanning element
      int                              MailCount;           // mail in the object

      // clear list
      void                             ClearList(void);
      // Add an object to list
      void                             Add(char *From, char *Subject, char *Date, char *Text);

};

#endif
