/******************************************************************************

	CCD.h | Header of GSC browser

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : May 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef GSCH
#define GSCH



// system include
#include <vcl.h>
#include <dir.h>
#pragma hdrstop



// project include
#include "CMObjectList.h"



/******************************************************************************

	GSC browser class

******************************************************************************/

class GSC
{
   public:

      // mixed definition
      typedef enum
      {
         MAX_REGIONS = 10000,                               // maximun region number
      } DEFINE;

      // regione header inforrmation
      typedef struct
      {
         int                           Len;                 // header len
         int                           Version;             // file vesion
         int                           Region;              // region number
         int                           ObjectNumber;        // object number
         double                        RAMin;               // minimum right ascension
         double                        RAMax;               // maximum right ascension
         double                        DecMin;              // minimum declination
         double                        DecMax;              // maximum declination
         double                        MagOffset;           // magnitude offset
         double                        RAScale;             // scale in right ascension
         double                        DecScale;            // scale in declination
         double                        ErrorScale;          // scale in position error
         double                        MagScale;            // scale in magnitude
         int                           PlateNum;            // number of plates covering region
         char                          *PlateList;          // list of plate names
      } HEADER;

      // star record
      typedef struct
      {
         double                        RA;                  // right ascension
         double                        Dec;                 // declination
         int                           Region;              // region number
         int                           Id;                  // star identifier
         float                         PosError;            // position error
         float                         Mag;                 // magnitude
         float                         MagError;            // magnitude error
         int                           Band;                // band code
         int                           Class;               // Object class (0=star, 3=no star)
         char                          PlateName[5];        // name of plate
         char                          MultipCode;          // ?
         float                         PlateData;           // plate shot data
      } GSCREC ;

      // region record
      typedef struct
      {
         int                           Number;              // region number
         double                        RAMin;               // minimum region right ascension
         double                        RAMax;               // maximum region right ascensione
         double                        DecMin;              // minimum region declination
         double                        DecMax;              // maximum region declination
      } REGION;

      // object constructor
                                       GSC( char *PGSCPath );
      // object destroyer
      virtual                          ~GSC();
      // get star list inside a sky window
      CMObjectList *                   GetStars( CMHours::HOURS *ARCenter, CMDegree::DEGREE *DecCenter, int Radius );  // radius in arcsec (it isn't a circle it is a square)


  protected:
      // detect intersetion from indexed region and given sky window
      bool                             RegionIntersect( int IndexRegion, double Right, double Left, double Top, double Bottom );
      // get header information from file buffer
      unsigned char *                  GetHeader( unsigned char *Buffer, HEADER *Header );
      // get next star from file buffer
      unsigned char *                  GetStar( unsigned char *Punt, HEADER *Header, GSCREC *Star );

      char                             GSCPath[MAXPATH];    // catalog path
      int                              NumRegions;          // loaded regions
      REGION                           Regions[MAX_REGIONS];// regions list
};

#endif
