/******************************************************************************

	ImageAnalyse.h | Header of image analyser

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : October 2012

   This header C++ object is used in more than one astromical software developed at
   Bassano Bresciano Observatory  
   Copyright (C) 2009,2010,2011,2012  Luca Pietro Strabla
   This file is part of Polipus and SNF
   Polypus ans SNF are free software: you can redistribute it and/or modify
   them under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus ans SNF are distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it

******************************************************************************/



// include antirepeat
#ifndef ImageAnalyseH
#define ImageAnalyseH



// system include
#include <vcl.h>
#include <dir.h>
#pragma hdrstop



/******************************************************************************

	Image compare class

******************************************************************************/

class ImageAnalyse
{
   public:
      // analyse parameter
      typedef struct
      {
         int                           XStart;              // start analyse column
         int                           XStop;               // stop analyse column
         int                           YStart;              // start analyse line
         int                           YStop;               // stop analyse line
         int                           MinDetectionPixel;   // minimum pixel count for over threshold for object detection
         double                        DetectionThreshold;  // detection threshold in sigma RMS
         double                        ClassThreshold;      // classification threshold in sigma RMS
         int                           SaturationLevel;     // CCD saturation level
         double                        SeeinfFWHM;          // seeing FWHM in pixel
         bool                          MostBright;          // filtered for most bright
         int                           MaxObjectCount;      // maximum needed object count
         bool                          NoGalaxies;          // galaxy not into the list
         double                        GalaxyFWHM;          // limit for galaxy discrimation
      }ANALYSE;

      // star found information
      typedef struct
      {
         double                        X;                   // x centroid position
         double                        Y;                   // y centroid position
         double                        Flux;                // flux
         double                        Magnitude;           // instrumental magnitude
         double                        FWHM;                // full width at half maximum
         int                           Flags;               // extraction flags
         double                        Classification;      // classification index
      }OBJECT;

      // block chain element
      typedef struct OBJECT_CHAINTag
      {
         OBJECT                        Actual;              // actual object
         OBJECT_CHAINTag*              Next;                // pounter to next object chain element
      }OBJECT_CHAIN;

      // error definition
      typedef enum
      {
         ERR_NONE,                                          // no error
         ERR_NOPARAMETERS,                                  // no parameters set
         ERR_SEXFILES,                                      // was not possibile create sextractor files
         ERR_SEXOUTPUT,                                     // sextractor output file not found
         ERR_MEMORY,                                        // not enought memory
      } ERR;

      // object constructor
                                       ImageAnalyse( char* PPath, char* PName );
      // object destroyer
      virtual                          ~ImageAnalyse();
      // set analyse parameters
      void                             SetAnalyseParameters( ANALYSE* PAnalyseParameters );
      // image analyse
      ERR                              Analyse( char *ImagePath );
      // build object list
      ERR                              BuildObjectList( void );
      // save object list
      ERR                              UpdateOutputFile( void );
      // get object list
      OBJECT_CHAIN *                   GetObjects( int *PObjectCount );
      // save object list
      ERR                              SaveObject( void );

  protected:
      char                             Path[MAXPATH];       // path for all files
      char                             Name[20];            // name for all SExtractor file
      bool                             HasParameters;       // analyse parameters was received
      bool                             ParametersChanged;   // analyse parameters was changed
      ANALYSE                          AnalyseParameters;   // analyse parameters
      OBJECT_CHAIN                     *FirstObject;        // first object in the chain
      OBJECT_CHAIN                     *LastObject;         // last object in chain
      int                              ObjectCount;         // object count in the list

      // free current object chain
      ERR                              FreeChain(void);
      // Add object to chain
      void                             AddObject( OBJECT *NewObject  );
      // create SExtractor files
      ERR                              CreateSExtractorFiles ( void );
      // macro expansion
      void                             MacroExpansion( char *Input, char *Output, char **Param );
};

#endif
