/******************************************************************************

	LX200.h | Header of LX200 protocol management

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef LX200H
#define LX200H



// system include
#include <vcl.h>
#pragma hdrstop



// project include
#include "CMDegree.h"
#include "CMHours.h"



// definition
#define LX200_RXTOUT      3000



/******************************************************************************

	LX200 protocol management class

******************************************************************************/

class LX200 
{
   public:
      // Object constructor
                                       LX200( );
      // Object destroyer
      virtual                          ~LX200();
      // Serial line opening
      int                              Open( int Line, int BaudDate );
      // Serial line close
      int                              Close( void );
      // Get current telescope position
      int                              GetTelescopePosition( CMHours::HOURS *RA, CMDegree::DEGREE *Dec );
      // set telescope velocity
      int                              SetTelescopeVelocity( int Velocity );
      // Set telecope position
      int                              SetTelescopePosition( CMHours::HOURS *RA, CMDegree::DEGREE *Dec );
      // Reset telecope position
      int                              ResetTelescopeAlign( void );
      // telescope goto command
      int                              TelescopeGoto( CMHours::HOURS *RA, CMDegree::DEGREE *Dec );
      // telescope in home position
      int                              TelescopeHome( );
      // manual telescope movement to north
      int                              GoTelescopeNorth( void );
      // manual telescope movement to south
      int                              GoTelescopeSouth( void );
      // manual telescope movement to east
      int                              GoTelescopeEast( void );
      // manual telescope movement to west
      int                              GoTelescopeWest( void );
      // stop telescope movement
      int                              StopTelescope( void );
      // set controller clock
      int                              SetClock( SYSTEMTIME *Time );
      // get telescope status
      int                              GetTelescopeStatus( bool *Retain, bool *Calib, bool *MotAR, bool *MotDec, bool *MotoO, bool *Movimento, int *PECValue, int *PECIndex );
      // reset motor fault
      int                              ResetMotorFault( int Mot );
      // tun on/off telescope tracking movement
      int                              SetTelescopeTracking( bool Stato );
      // tun on/off dome automatic movement
      int                              SetAutomaticDome( bool Stato );
      // turn on/off flat light
      int                              SetFlatLight( bool Stato );
      // get dome status
      int                              GetDomeStatus( int *Azimut, int *Scor, int *Riba, bool *Flat, bool *Auto, bool *Calib, int *ComCupola, bool *Phone, bool *Alarm, bool *Home, bool *Zero );
      // get dome temperature
      int                              GetDomeTemperature( int *TempAmb );
      // open/close sliding shutter
      int                              SetSShutter( bool Open );
      // open/close balancing shutter
      int                              SetBShutter( bool Open );
      // CW dome rotation
      int                              SetDomeCW( void );
      // CCW dome rotation
      int                              SetDomeCCW( void );
      // Stop dome movement
      int                              SetDomeStop( void );
      // dome in home position
      int                              SetDomeHome( void );
      // dome in zero position
      int                              SetDomeZero( void );
      // turn on/off decondence
      int                              SetDecondence( bool Stato );
      // turn on/off timeout
      int                              SetTimeoutControl( bool Stato );
      // set automatic velocity
      int                              SetAutoVelocity( int Velocity );
      // turn on/off periodic error correction
      int                              SetPEC( bool Stato, int Amplitude, int Phase );

   protected:
      HANDLE                           hCom;                // handle della linea seriale
      CRITICAL_SECTION                 CriticalSection;

      // communication execution
      int                              ExecCom( char *BufferTx, int LenBufTx, char *BufferRx, int LenBufRx, int LenExpected);
      // send buffer on serial line
      int                              TxBuffer( char *Buffer, int LenBuf );
      // receive buffer from serial line
      int                              RxBuffer( char *Buffer, int LenBuf, int LenExpected);
};

#endif
