/******************************************************************************

	LX200Thread.cpp | LX200 Communication thread

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// System include
#include <vcl.h>
#pragma hdrstop
                                                 


// project include
#include "LX200Thread.h"
#pragma package(smart_init)





/*---------------------------------------------------------------------------

   Object constructor

---------------------------------------------------------------------------*/

__fastcall LX200Thread::LX200Thread(LX200 *ParObjCom, int ParRefreschTime )
   : TThread( false )
{
   Active = true;
   StatusReceived = false;
   CommStatus = false;
   ObjCom = ParObjCom;
   RefreschTime = ParRefreschTime;
   memset ( &InternalStatus, 0, sizeof( InternalStatus ) );
}



/*---------------------------------------------------------------------------

   thread execution

---------------------------------------------------------------------------*/

void __fastcall LX200Thread::Execute(void)
{
   int                                 Ret;                 // communication retun value
   int                                 Temp;                // dome temperature
   bool                                TempCommStatus;      // temporary status in the loop

   while (  Terminated == false )
   {
      // get telescope position
      Ret = ObjCom->GetTelescopePosition(&InternalStatus.TeleRA, &InternalStatus.TeleDec);
      if ( Terminated == true )
         break;
      TempCommStatus = Ret == 0;

      Sleep ( RefreschTime/4 );

      // get telescope state
      Ret = ObjCom->GetTelescopeStatus( &InternalStatus.DataRetained, &InternalStatus.TeleAlligned,
                                   &InternalStatus.MotorRAFault, &InternalStatus.MotorDecFault,
                                   &InternalStatus.TeleTracking, &InternalStatus.TeleMoving,
                                   &InternalStatus.PECValue,     &InternalStatus.PECIndex);
      if ( Terminated == true )
         break;
      TempCommStatus &= Ret == 0;

      Sleep ( RefreschTime/4 );

      // get dome state
      Ret = ObjCom->GetDomeStatus( &InternalStatus.DomeAzimut,   &InternalStatus.DomeSlidingShutter, &InternalStatus.DomeBalancingShutter,
                                   &InternalStatus.FlatLight,    &InternalStatus.DomeLock,           &InternalStatus.DomeAlligned,
                                   &InternalStatus.DomeMovement, &InternalStatus.DeCondenses,        &InternalStatus.TeleAlarm,
                                   &InternalStatus.DomeHome,     &InternalStatus.DomeZero );
      if ( Terminated == true )
         break;
      TempCommStatus &= Ret == 0;

      Sleep ( RefreschTime/4 );

      Ret = ObjCom->GetDomeTemperature( &Temp);
      if ( Terminated == true )
         break;
      if ( Ret == 0 )
         InternalStatus.DomeTemp = Temp;
      TempCommStatus &= Ret == 0;

      Sleep ( RefreschTime/4 );

      CommStatus = TempCommStatus;                          // activate status
      if ( TempCommStatus == true )                         // when communication is ok status is declared received
         StatusReceived = true;
   }

   Active = false;
}



/*---------------------------------------------------------------------------

   get telescope/dome status

---------------------------------------------------------------------------*/

bool LX200Thread::GetStatus( STATUS * InfoStatus )
{
   *InfoStatus = InternalStatus;
   return StatusReceived;
}



/*---------------------------------------------------------------------------

   get communication status

---------------------------------------------------------------------------*/

bool LX200Thread::GetCommStatus(void)
{
   return CommStatus;
}

