/******************************************************************************

	LX200Thread.h | Header of LX200 communication thread

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef LX200ThreadH
#define LX200ThreadH



// system include
#include <Classes.hpp>


// project include
#include "LX200.h"



class LX200Thread : public TThread
{
   public:
      // observatory status structure
      typedef struct
      {
         bool                          DataRetained;        // Data retained correctly by controller
         CMHours::HOURS                TeleRA;              // Current telescope right ascension
         CMDegree::DEGREE              TeleDec;             // Current telescope declination
         bool                          TeleAlligned;        // Telescopel is assligned
         bool                          TeleTracking;        // Telescope tracking motor state
         bool                          TeleMoving;          // Telescope is moving
         bool                          TeleAlarm;           // Telescope alarm position
         int                           DomeAzimut;          // Current dome azimut
         bool                          DomeLock;            // Dome lock to telescope azimut state
         bool                          DomeAlligned;        // Dome is alligned
         int                           DomeMovement;        // Dome movement state
         int                           DomeSlidingShutter;  // Sliding shutter state
         int                           DomeBalancingShutter;// Balancing shutter state
         float                         DomeTemp;            // Temperature inside dome
         bool                          DomeHome;            // Dome in home position
         bool                          DomeZero;            // Dome in zero position
         bool                          FlatLight;           // Flat light state
         bool                          DeCondenses;         // De condenses device state
         bool                          MotorRAFault;        // Fault on right ascension motor
         bool                          MotorDecFault;       // Fault on declination motor
         int                           PECValue;            // periodic error correction value
         int                           PECIndex;            // periodic error correction phase index
      }STATUS;

      // Object constructor
      __fastcall                       LX200Thread(LX200 *ParObjCom, int ParRefreschTime);
      // get telescope/dome status
      bool                             GetStatus( STATUS * InfoStatus );
      // get communication status
      bool                             GetCommStatus(void);

      bool                             Active;              // thread is running

   protected:
      // thread execution
      void __fastcall                  Execute(void);

      LX200                            *ObjCom;             // communication object
      STATUS                           InternalStatus;      // internal status
      bool                             CommStatus;          // communication status
      int                              RefreschTime;        // status refresc time (millisec)
      bool                             StatusReceived;      // status completelly received
};
//---------------------------------------------------------------------------
#endif
