/******************************************************************************

	ManualControlObserver.cpp Main form observer. It should be part of Main form class
                             but VCL doesn't support multiple inheritance

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : 		April 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

*****************************************************************************/



// project include
#include "PolypusMF.h"
#include "ManualControlObserver.h"



/******************************************************************************

   constructor

******************************************************************************/

ManualControlObserver::ManualControlObserver( )
{
   // inizialize internal data
   PuntSubject=NULL;
   PuntMF=NULL;
}



/******************************************************************************

   Destroyer

******************************************************************************/

ManualControlObserver::~ManualControlObserver( )
{
   // if obderver is attached to subject
   if ( PuntSubject != NULL )
      PuntSubject->Detach( this );                          // release itself from subject
}



/******************************************************************************

   Set subject

******************************************************************************/

int ManualControlObserver::SetSubject( TentaclesMgr *APuntSubject )
{
   PuntSubject = APuntSubject;                              // store pointer
   return PuntSubject->Attach( this );                      // attach itself to subject
}



/******************************************************************************

   Get subject

******************************************************************************/

TentaclesMgr * ManualControlObserver::GetSubject( void )
{
   return PuntSubject;
}



/******************************************************************************

   Set main form object

******************************************************************************/

void ManualControlObserver::SetMF( TMainForm *APuntMF )
{
   PuntMF = APuntMF;
}



/******************************************************************************

   Send to main form all notifications

******************************************************************************/

void ManualControlObserver::Update( int ANotifyCode )
{
   // if mainform is set
   if ( PuntMF != NULL )
      PuntMF->ObsUpdate( ANotifyCode );                     // sent it all notificantions
}




