/******************************************************************************

	OpenSequence.h | Header of observatory opening sequence

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef OpenSequenceH
#define OpenSequenceH


// project include
#include "Observer.h"
#include "TentaclesMgr.h"




class OpenSequence : public TThread, public Observer
{
   public:

      // state of sequence
      typedef enum
      {
         SEQ_STATE_IDLE,                                    // idle, wait to start
         SEQ_STATE_START,                                   // start state
         SEQ_STATE_DOMEZERO,                                // wait for dome allign
         SEQ_STATE_OPENSSHUTTER,                            // wait for open sliding shutter
         SEQ_STATE_OPENBSHUTTER,                            // wait for open balancing shutter
         SEQ_STATE_END,                                     // sequence ended
      } SEQ_STATE;

      // Object constructor
      __fastcall                       OpenSequence(void);
      // Object destroyer
      virtual _fastcall                ~OpenSequence();
      // set subject
      int                              SetSubject( TentaclesMgr *ParPuntSubject );
      // start sequence
      void                             StartSequence(HANDLE ParFormHandle, bool ParDomeZero, bool ParOpenSShutter, bool ParOpenBShutter, bool ParDomeLock, bool ParSyncClock, bool ParTrackingOn );
      // get sequence status
      SEQ_STATE                        GetState( void );

   private:
      // notify change in subject
      virtual void                     Update( int ANotifyCode );

   protected:
      TentaclesMgr                     *PuntSubject;        // pointer to subject
      HANDLE                           FormHandle;          // handle of the parent form
      bool                             DomeZero;            // required dome in zero position
      bool                             OpenSShutter;        // required sliding shutter opening
      bool                             OpenBShutter;        // required balancing shutter opening
      bool                             DomeLock;            // required dome lock on telescope azimut position
      bool                             SyncClock;           // required controller clock sincronization
      bool                             TrackingOn;          // required turn on tracking motor
      SEQ_STATE                        State;               // actual sequence state
      bool                             Control;             // control activated

      // Threaded function
      void __fastcall                  Execute(void);
};
//---------------------------------------------------------------------------
#endif
