/******************************************************************************

	PolipoMF.h | Hader for polypus main form

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : March 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef PolypusMFH
#define PolypusMFH



// System include
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <ExtCtrls.hpp>
#include <Menus.hpp>
#include <Buttons.hpp>
#include <ImgList.hpp>
#include <CheckLst.hpp>
#include <Graphics.hpp>
#include <Dialogs.hpp>
#include <NMpop3.hpp>
#include <Psock.hpp>
#include <Grids.hpp>



// project include
#include "TentaclesMgr.h"
#include "ManualControlObserver.h"
#include "LogObserver.h"
#include "FlatSequence.h"
#include "DarkSequence.h"
#include "ShotSequence.h"
#include "OpenSequence.h"
#include "CloseSequence.h"
#include "ScriptEditor.h"
#include "ScriptCPU.h"
#include "XYGraph.h"
#include "GRB.h"
#include "ElevationGraph.h"


// definitions
#define                                BITMAP_OFF     0
#define                                BITMAP_ON      1
#define                                BITMAP_LOCK    2
#define                                BITMAP_UNLOCK  3
#define                                BITMAP_HOME    4
#define                                BITMAP_ZERO    5
#define                                BITMAP_STOP    6
#define                                BITMAP_LEFT    7
#define                                BITMAP_RIGHT   8
#define                                BITMAP_UP      9
#define                                BITMAP_DOWN    10
#define                                BITMAP_OPEN    11
#define                                BITMAP_CLOSE   12
#define                                BITMAP_SYNC    13
#define                                BITMAP_RESET   14

#define                                MAX_OBSERVER   10

#define                                OPTION_OBSERVATORY   "OBSERVATORY"
#define                                OPTION_DIRECTORIES   "DIRECTORIES"
#define                                OPTION_CCD           "CCD"
#define                                OPTION_MISC          "MISC"
#define                                OPTION_GRB           "GRB"

#define                                MAX_PLANNEDOBJ       20



class TMainForm : public TForm
{
   __published:	// IDE-managed Components
      TPanel                           *TelescopePanel;
      TPanel                           *DomePanel;
      TPanel                           *ShotPanel;
      TLabel                           *TelescopeMotorsLabel;
      TMainMenu                        *MainMenu;
      TMenuItem                        *File1;
      TLabel                           *TelescopeAllignedLabel;
      TLabel                           *TelescopeTrackingLabel;
      TLabel                           *TelescopeElevationLabel;
      TLabel                           *TelescopeAzimutLabel;
      TLabel                           *TelescopeDeclinationLabel;
      TLabel                           *TelescopeRALabel;
      TLabel                           *TelescopeLabel;      
      TLabel                           *DomeLabel;           
      TLabel                           *SShutterLabel;       
      TLabel                           *BShutterLabel;       
      TLabel                           *DomeAzimutLabel;     
      TLabel                           *FlatLightLabel;      
      TLabel                           *DecondLabel;         
      TLabel                           *DomeAllignedLabel;
      TLabel                           *DomeTemperatureLabel;
      TLabel                           *TimeLabel;           
      TLabel                           *SideralTimeLabel;    
      TLabel                           *ShotLabel;           
      TLabel                           *ShotNameLabel;
      TLabel                           *ShotExposureLabel;
      TLabel                           *VisibilityIndexLabel;
      TImageList                       *ImageList;              
      TPanel                           *GeneralPanel;        
      TLabel                           *DateLabel;           
      TLabel                           *RetainLabel;         
      TLabel                           *PhotoBackgroundLabel;
      TLabel                           *GeneralLabel;
      TLabel                           *CommLabel;     
      TTimer                           *PollingTimer;
      TLabel                           *PowerLabel;
      TLabel                           *DomeStateLabel; 
      TMenuItem                        *ItemConfigura;     
      TMenuItem                        *ItemEsci;
      TPopupMenu                       *ChartMenu;
      TMenuItem                        *ChartAllignmentItem;
      TMenuItem                        *LogDeviationItem;
      TPopupMenu                       *ShotMenu;
      TMenuItem                        *TelescopeAllignmentItem;       
      TMenuItem                        *StarCenterItem;
      TImage                           *ShotExposingImage;
      TOpenDialog                      *OpenDialog;
      TLabel                           *Label52;
      TLabel                           *Label53;
      TMenuItem                        *Licence;
      TLabel                           *GRBLabel;
      TTimer                           *MailTimer;
   TPageControl *Pages;
   TTabSheet *ManualPage;
   TGroupBox *ControllerBox;
   TSpeedButton *PowerOffButton;
   TSpeedButton *PowerOnButton;
   TSpeedButton *LX200OnButton;
   TSpeedButton *LX200OffButton;
   TSpeedButton *ClockSyncButton;
   TPanel *Panel2;
   TPanel *Panel11;
   TPanel *Panel13;
   TGroupBox *OpenSequenceBox;
   TSpeedButton *OpenSequenceButton;
   TCheckListBox *OpenSequenceListBox;
   TGroupBox *DomeBox;
   TSpeedButton *SShutterStopButton;
   TSpeedButton *SShutterCloseButton;
   TSpeedButton *SShutterOpenButton;
   TSpeedButton *BShutterCloseButton;
   TSpeedButton *BShutterStopButton;
   TSpeedButton *BShutterOpenButton;
   TSpeedButton *DomeLockButton;
   TSpeedButton *DomeUnlockButton;
   TSpeedButton *DomeHomeButton;
   TSpeedButton *DomeZeroButton;
   TSpeedButton *DomeStopButton;
   TSpeedButton *DomeLeftButton;
   TSpeedButton *DomeRightButton;
   TSpeedButton *FlatLightOffButton;
   TSpeedButton *FlatLightOnButton;
   TSpeedButton *DeCondOffButton;
   TSpeedButton *DeCondOnButton;
   TPanel *Panel6;
   TPanel *Panel7;
   TPanel *Panel12;
   TPanel *Panel4;
   TPanel *Panel5;
   TGroupBox *TelescopeBox;
   TSpeedButton *TelescopeDownButton;
   TSpeedButton *TelescopeHomeButton;
   TSpeedButton *TelescopeUpButton;
   TSpeedButton *TelescopeRightButton;
   TSpeedButton *TelescopeStopButton;
   TSpeedButton *TelescopeLeftButton;
   TSpeedButton *TrackingOffButton;
   TSpeedButton *TrackingOnButton;
   TSpeedButton *RAMotorResetButton;
   TSpeedButton *DecMotorResetButton;
   TPanel *Panel1;
   TPanel *Panel3;
   TPanel *Panel9;
   TTrackBar *TelescopeVelocity;
   TGroupBox *CloseSequenceBox;
   TSpeedButton *CloseSequenceButton;
   TLabel *Label38;
   TCheckBox *CloseSequenceCheckBox;
   TEdit *CloseSequenceHHEdit;
   TEdit *CloseSequenceMMEdit;
   TCheckListBox *CloseSequenceListBox;
   TTabSheet *ShotPage;
   TGroupBox *BoxCCD;
   TImage *CCDImage;
   TLabel *Label2;
   TLabel *Label1;
   TImage *ZoomImage;
   TLabel *Label35;
   TLabel *Label36;
   TLabel *Label37;
   TSpeedButton *AlignmentButton;
   TBitBtn *ActivateCCDButton;
   TBitBtn *ReleaseCCDButton;
   TEdit *ObjectNameEdit;
   TBitBtn *ImageSaveButton;
   TComboBox *ExposureComboBox;
   TBitBtn *ShotStartButton;
   TBitBtn *ShotStopButton;
   TEdit *ExposingEdit;
   TEdit *PointerYEdit;
   TEdit *PointerXEdit;
   TEdit *PointerValueEdit;
   TPageControl *ShotPages;
   TTabSheet *TabImage;
   TGroupBox *GroupFlat;
   TSpeedButton *FlatStartButton;
   TLabel *Label13;
   TLabel *Label14;
   TLabel *Label15;
   TEdit *FlatExposingEdit;
   TComboBox *FlatExposureComboBox;
   TEdit *FlatTargetEdit;
   TEdit *DarkFlatTargetEdit;
   TEdit *DarkFlatProgEdit;
   TEdit *FlatProgEdit;
   TButton *FlatStopButton;
   TGroupBox *GroupDark;
   TLabel *Label16;
   TLabel *Label17;
   TEdit *DarkExposingEdit;
   TBitBtn *DarkStartButton;
   TEdit *DarkTargetEdit;
   TListBox *DarkLibListBox;
   TEdit *DarkProgEdit;
   TComboBox *DarkExposureComboBox;
   TButton *DarkStopButton;
   TGroupBox *GroupScaling;
   TImage *HistogramImage;
   TLabel *Label5;
   TLabel *Label4;
   TLabel *Label18;
   TScrollBar *MaximumScrollBar;
   TEdit *MaximumEdit;
   TEdit *MinimumEdit;
   TScrollBar *MinimumScrollBar;
   TCheckBox *AutoScalingCheckBox;
   TEdit *SigmaMinusEdit;
   TEdit *SigmaPlusEdit;
   TGroupBox *GroupStatistics;
   TLabel *Label6;
   TLabel *Label7;
   TLabel *Label8;
   TLabel *Label9;
   TLabel *Label10;
   TLabel *Label11;
   TLabel *Label12;
   TEdit *ImageSigmaEdit;
   TEdit *BackgrondValueEdit;
   TEdit *BackgroundSigmaEdit;
   TEdit *VisibilityIndexEdit;
   TEdit *ImageFWHMEdit;
   TEdit *MinimumValueEdit;
   TEdit *MaximumValueEdit;
   TGroupBox *SequenceGroup;
   TLabel *Label39;
   TLabel *Label40;
   TLabel *Label3;
   TLabel *Label41;
   TLabel *Label42;
   TLabel *Label43;
   TCheckBox *SafeActive;
   TEdit *SafeTimeEdit;
   TEdit *SafeThresholdEdit;
   TEdit *SequenceProgEdit;
   TEdit *SequenceTargetEdit;
   TBitBtn *SequenceStartButton;
   TBitBtn *SequenceStopButton;
   TEdit *VentilationIntervalEdit;
   TEdit *VentilationTimeEdit;
   TCheckBox *VentilationActive;
   TEdit *VentilationPauseEdit;
   TTabSheet *TabField;
   TImage *ChartImage;
   TLabel *Label19;
   TLabel *Label20;
   TLabel *Label21;
   TLabel *Label22;
   TLabel *Label23;
   TLabel *Label24;
   TLabel *Label27;
   TLabel *Label26;
   TEdit *ChartRA_hhEdit;
   TEdit *ChartRA_mmEdit;
   TEdit *ChartRA_ssEdit;
   TEdit *ChartDec_SignEdit;
   TEdit *ChartDec_ggEdit;
   TEdit *ChartDec_mmEdit;
   TEdit *ChartDec_ssEdit;
   TCheckBox *CharFollowsTelescopeCheckBox;
   TButton *ChartGotoButton;
   TTabSheet *TabObjects;
   TLabel *Label25;
   TLabel *Label28;
   TLabel *Label29;
   TLabel *Label30;
   TLabel *Label31;
   TLabel *Label32;
   TLabel *Label33;
   TLabel *Label34;
   TLabel *Label47;
   TCheckBox *OnlyVisibleCheckBox;
   TListBox *ObjListBox;
   TButton *ObjGotoButton;
   TBitBtn *ObjAllignmentButton;
   TMemo *ObjectInfoMemo;
   TButton *ObjCharGotoButton;
   TEdit *ObjRA_hhEdit;
   TEdit *ObjRA_mmEdit;
   TEdit *ObjRA_ssEdit;
   TEdit *ObjDec_SignEdit;
   TEdit *ObjDec_ggEdit;
   TEdit *ObjDec_mmEdit;
   TEdit *ObjDec_ssEdit;
   TButton *AddButton;
   TButton *RemoveButton;
   TEdit *TelecopeAutoVelocityEdit;
   TTabSheet *GRBPage;
   TGroupBox *MailGroupBox;
   TLabel *Label79;
   TLabel *Label78;
   TCheckBox *GRBEnableCheckBox;
   TMemo *MailTextMemo;
   TEdit *MailStatus;
   TStringGrid *MailGrid;
   TEdit *LoadDownloadEdit;
   TButton *GRBGotoButton;
   TEdit *PollingEdit;
   TCheckBox *SwiftEnableCheckBox;
   TGroupBox *GroupBox1;
   TLabel *Label66;
   TLabel *Label67;
   TLabel *Label68;
   TLabel *Label69;
   TLabel *Label70;
   TLabel *Label71;
   TLabel *Label72;
   TLabel *Label73;
   TLabel *Label74;
   TLabel *Label75;
   TLabel *Label76;
   TLabel *Label77;
   TEdit *GRBRA_hhEdit;
   TEdit *GRBRA_mmEdit;
   TEdit *GRBRA_ssEdit;
   TEdit *GRBDec_SignEdit;
   TEdit *GRBDec_ggEdit;
   TEdit *GRBDec_mmEdit;
   TEdit *GRBDec_ssEdit;
   TEdit *GRBAzimutEdit;
   TEdit *GRBElevationEdit;
   TButton *GRBResetButton;
   TGroupBox *GRBScriptBox;
   TSpeedButton *LoadGRBScriptButton;
   TSpeedButton *SaveGRBScriptButton;
   TEdit *GRBScriptEdit;
   TTabSheet *ScriptPage;
   TSpeedButton *StepScriptButton;
   TSpeedButton *RewindScriptButton;
   TSpeedButton *RunScriptButton;
   TSpeedButton *StopScriptButton;
   TLabel *Label45;
   TGroupBox *MainScriptBox;
   TSpeedButton *LoadMainScriptButton;
   TSpeedButton *SaveMainScriptButton;
   TEdit *MainScriptEdit;
   TGroupBox *TimeScriptBox;
   TSpeedButton *LoadTimeScriptButton;
   TSpeedButton *SaveTimeScriptButton;
   TLabel *Label44;
   TEdit *TimeScriptEdit;
   TCheckBox *EnableTimeScriptCheck;
   TEdit *IntervalTimeScriptEdit;
   TGroupBox *CloseScriptBox;
   TSpeedButton *LoadCloseScriptButton;
   TSpeedButton *SaveCloseScriptButton;
   TLabel *Label46;
   TEdit *CloseScriptEdit;
   TEdit *HourCloseScriptEdit;
   TEdit *MinuteCloseScriptEdit;
   TMemo *ErrorListMemo;
   TTabSheet *LogPage;
   TMemo *LogMemo;
   TTabSheet *ServicePage;
   TMemo *ServiceMemo;
   TTabSheet *PlanePage;
   TImage *PlanningImage;
   TComboBox *ObjectTypeComboBox;
   TButton *PlaneButton;
   TGroupBox *GroupPlanningPar;
   TMonthCalendar *MonthCalendar1;
   TButton *NextDayButton;
   TButton *PreviousDayButton;
   TListBox *PlanningListBox;
   TDrawGrid *PlanningDrawGrid;
   TLabel *Label48;
   TLabel *Label49;
   TButton *ErasePlanningButton;
   TLabel *TelescopePECLabel;
   TCheckBox *PECheckBox;
   TComboBox *NGCTypeComboBox;
   TGroupBox *GroupBox2;
   TButton *IncDecButton;
   TButton *DecDecButton;
   TEdit *DeltaDecEdit;
   TEdit *DeltaRAEdit;
   TButton *IncRAButton;
   TButton *DecRAButton;

      void __fastcall                  FormShow(TObject *Sender);
      void __fastcall                  FormDestroy(TObject *Sender);                                           
      void __fastcall                  StatusUpdate(TObject *Sender);                                          
      void __fastcall                  PowerOnButtonClick(TObject *Sender);                                    
      void __fastcall                  PowerOffButtonClick(TObject *Sender);                                   
      void __fastcall                  LX200OnButtonClick(TObject *Sender);                                    
      void __fastcall                  LX200OffButtonClick(TObject *Sender);                                   
      void __fastcall                  FlatLightOnButtonClick(TObject *Sender);                                
      void __fastcall                  FlatLightOffButtonClick(TObject *Sender);
      void __fastcall                  TrackingOnButtonClick(TObject *Sender);                                 
      void __fastcall                  TrackingOffButtonClick(TObject *Sender);                                
      void __fastcall                  DeCondOnButtonClick(TObject *Sender);                                   
      void __fastcall                  DeCondOffButtonClick(TObject *Sender);                                  
      void __fastcall                  ClockSyncButtonClick(TObject *Sender);
      void __fastcall                  DomeLockButtonClick(TObject *Sender);                                   
      void __fastcall                  DomeUnlockButtonClick(TObject *Sender);                                 
      void __fastcall                  DomeHomeButtonClick(TObject *Sender);                                   
      void __fastcall                  DomeZeroButtonClick(TObject *Sender);                                   
      void __fastcall                  DomeStopButtonClick(TObject *Sender);                                   
      void __fastcall                  DomeLeftButtonClick(TObject *Sender);                                   
      void __fastcall                  DomeRightButtonClick(TObject *Sender);                                  
      void __fastcall                  RAMotorResetButtonClick(TObject *Sender);                               
      void __fastcall                  DecMotorResetButtonClick(TObject *Sender);                              
      void __fastcall                  SShutterOpenButtonClick(TObject *Sender);                               
      void __fastcall                  SShutterStopButtonClick(TObject *Sender);                               
      void __fastcall                  SShutterCloseButtonClick(TObject *Sender);                              
      void __fastcall                  BShutterOpenButtonClick(TObject *Sender);                               
      void __fastcall                  BShutterStopButtonClick(TObject *Sender);                               
      void __fastcall                  BShutterCloseButtonClick(TObject *Sender);
      void __fastcall                  TelescopeLeftButtonClick(TObject *Sender);                              
      void __fastcall                  TelescopeRightButtonClick(TObject *Sender);                             
      void __fastcall                  TelescopeUpButtonClick(TObject *Sender);                                
      void __fastcall                  TelescopeDownButtonClick(TObject *Sender);                              
      void __fastcall                  TelescopeStopButtonClick(TObject *Sender);                              
      void __fastcall                  TelescopeHomeButtonClick(TObject *Sender);
      void __fastcall                  ItemConfiguraClick(TObject *Sender);                                    
      void __fastcall                  ReleaseCCDButtonClick(TObject *Sender);                                 
      void __fastcall                  ShotStartButtonClick(TObject *Sender);                                  
      void __fastcall                  MinimumEditChange(TObject *Sender);
      void __fastcall                  MaximumEditChange(TObject *Sender);
      void __fastcall                  MaximumScrollBarChange(TObject *Sender);                                
      void __fastcall                  MinimumScrollBarChange(TObject *Sender);                                
      void __fastcall                  ActivateCCDButtonClick(TObject *Sender);                                
      void __fastcall                  CCDImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y);
      void __fastcall                  CCDImageMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
      void __fastcall                  ImageSaveButtonClick(TObject *Sender);                                 
      void __fastcall                  SequenceStartButtonClick(TObject *Sender);                             
      void __fastcall                  FlatStartButtonClick(TObject *Sender);                                 
      void __fastcall                  DarkStartButtonClick(TObject *Sender);                                 
      void __fastcall                  ShotStopButtonClick(TObject *Sender);                                  
      void __fastcall                  ChartGotoButtonClick(TObject *Sender);                                 
      void __fastcall                  ChartImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y);
      void __fastcall                  ChartImageMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
      void __fastcall                  ChartImageMouseUp(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
      void __fastcall                  ObjGotoButtonClick(TObject *Sender);                                               
      void __fastcall                  ObjAllignmentButtonClick(TObject *Sender);                                         
      void __fastcall                  UpdateObjectList(TObject *Sender);                                                 
      void __fastcall                  ObjListBoxClick(TObject *Sender);                                                  
      void __fastcall                  LogDeviation(TObject *Sender);                                                     
      void __fastcall                  ChartAllignmentFromShot(TObject *Sender);                                          
      void __fastcall                  TelescopeAllignmentItemClick(TObject *Sender);                                     
      void __fastcall                  StarCenterItemClick(TObject *Sender);                                              
      void __fastcall                  ObjCharGotoButtonClick(TObject *Sender);                                           
      void __fastcall                  FlatStopButtonClick(TObject *Sender);                                              
      void __fastcall                  FormCloseQuery(TObject *Sender, bool &CanClose);                                   
      void __fastcall                  OpenSequenceButtonClick(TObject *Sender);                                          
      void __fastcall                  CloseSequenceButtonClick(TObject *Sender);                                         
      void __fastcall                  SequenceStopButtonClick(TObject *Sender);                                          
      void __fastcall                  DarkStopButtonClick(TObject *Sender);
      void __fastcall                  AddMinorPlanet(TObject *Sender);
      void __fastcall                  SetGotoCoordinates(TObject *Sender);
      void __fastcall                  RemoveMinorPlanet(TObject *Sender);
      void __fastcall                  LoadMainScript(TObject *Sender);
      void __fastcall                  SaveMainScript(TObject *Sender);
      void __fastcall                  LoadTimeScript(TObject *Sender);
      void __fastcall                  SaveTimeScript(TObject *Sender);
      void __fastcall                  LoadCloseScript(TObject *Sender);
      void __fastcall                  SaveCloseScript(TObject *Sender);
      void __fastcall                  StepScriptButtonClick(TObject *Sender);
      void __fastcall                  RunScriptButtonClick(TObject *Sender);
      void __fastcall                  StopScriptButtonClick(TObject *Sender);
      void __fastcall                  RewindScriptButtonClick(TObject *Sender);
      void __fastcall                  CharFollowsTelescopeCheckBoxClick(TObject *Sender);
      void __fastcall                  LicenceClick(TObject *Sender);
      void __fastcall                  MailCheck(TObject *Sender);
      void __fastcall                  GRBEnableCheckBoxClick(TObject *Sender);
      void __fastcall                  MailGridSelectCell(TObject *Sender, int ACol, int ARow, bool &CanSelect);
      void __fastcall                  SwiftEnableCheckBoxClick(TObject *Sender);
      void __fastcall                  LoadGRBScriptButton2Click(TObject *Sender);
      void __fastcall                  GRBResetButtonClick(TObject *Sender);
      void __fastcall                  AligmentButtonClick(TObject *Sender);
      void __fastcall                  GRBGotoButtonClick(TObject *Sender);
      void __fastcall                  SaveGRBScript(TObject *Sender);
      void __fastcall                  PreviousDayButtonClick(TObject *Sender);
      void __fastcall                  NextDayButtonClick(TObject *Sender);
      void __fastcall                  MonthCalendar1Click(TObject *Sender);
      void __fastcall                  PlaneButtonClick(TObject *Sender);
      void __fastcall                  PlanningDrawGridDrawCell(TObject *Sender, int ACol, int ARow, TRect &Rect, TGridDrawState State);
      void __fastcall                  ErasePlanningButtonClick(TObject *Sender);
   void __fastcall IncDecButtonClick(TObject *Sender);
   void __fastcall DecDecButtonClick(TObject *Sender);
   void __fastcall IncRAButtonClick(TObject *Sender);
   void __fastcall DecRAButtonClick(TObject *Sender);

   private:

      CMCoLocal                        *Site;               // observatory position
      CMEvent                          *Now;                // current data and time event object
      char                             PatShotFolder[MAXPATH];    // nightly flder name
      ManualControlObserver            *MFObserver;         // observer of main form
      LogObserver                      *ObjLogObserver;     // observer for log
      TentaclesMgr::STATUS             Status;              // stutus of all devices managed by tentacles object
      TentaclesMgr::STATUS             StatusOld;           // previous stutus of all devices managed by tentacles object
      double                           AzimutOld;           // previous telescope azimut
      bool                             FirstStatus;         // first staus flag
      bool                             ExposureActive;      // single exposure is active
      bool                             ZoomLocked;          // zoom image is locked
      int                              ZoomX;               // zoom x center position
      int                              ZoomY;               // zoom y center position
      bool                             AdjustChatFrame;     // enable chart frame adjustment by mouse
      bool                             StarShotSelected;    // selected a star on the shot image
      bool                             StarChartSelected;   // selected a star on the chart image
      CMHours::HOURS                   RAStarShot;          // selected star RA on shot image
      CMDegree::DEGREE                 DecStarShot;         // selected star declination on shot image
      CMHours::HOURS                   RAStarChart;         // selected star RA on chart image
      CMDegree::DEGREE                 DecStarChart;        // selected star declination on chart image
      CMHours::HOURS                   RAChart;             // RA on chart image
      CMDegree::DEGREE                 DecChart;            // declination on chart image
      Graphics::TBitmap                *BmpOriginalChart;   // original chart bitmap without CCD frame
      Graphics::TBitmap                *BmpFramedChart;     // chart bitmap with CCD frame
      int                              XMouseDown;          // X chart mouse down position
      int                              YMouseDown;          // Y chart mouse down position
      FlatSequence                     *ObjFlatSequence;    // object for flat sequence management
      DarkSequence                     *ObjDarkSequence;    // object for dark sequence management
      ShotSequence                     *ObjShotSequence;    // object for shot sequence management
      OpenSequence                     *ObjOpenSequence;    // object for observatory opening  sequence management
      CloseSequence                    *ObjCloseSequence;   // object for observatory closing sequence management
      bool                             EnableScaligChangeDetect; // enable scaling value change detect (disabled when automatic change)
      bool                             EnableRefreshShot;   // enable refresh shot information
      int                              SetChartFromTelescope;  // enable chart set from telescope position
      AnsiString                       Diary;               // string with all diary
      bool                             StartedDiary;        // start diary already done
      char                             SelectedObject[51];  // selected objcet on object page
      ScriptEditor                     *GRBScriptEditor;    // editor object for GRB script
      ScriptEditor                     *MainScriptEditor;   // editor object for main script
      ScriptEditor                     *TimeScriptEditor;   // editor object for time script
      ScriptEditor                     *CloseScriptEditor;  // editor object for close script
      ScriptErrorList                  *ErrorList;          // error in the scripts
      ScriptCPU                        *CPU;                // thread executing script
      bool                             CPULoaded;           // script CPU is loaded
      bool                             GRBScriptChanged;    // GRB script needs save
      bool                             MainScriptChanged;   // main script needs save
      bool                             TimeScriptChanged;   // time script needs save
      bool                             CloseScriptChanged;  // close script needs save
      int                              OldLineRunning;      // old line running
      int                              OldScriptRunning;    // old script running
      ElevationGraph                   *PlaningGraph;       // night planning graph
      bool                             CPURunning;          // script CPU is running
      bool                             OldGRBDetect;        // value of GRBDetected at previous evaluation
      bool                             OldGRBOn;            // value of GRBOn at previous evaluation
      GRB                              *GRBDetector;        // object detecting GRB
      int                              ChartField;          // arcsec chart horizontal field
      CMObjectList                     *PlanningList;       // planning objects list
      TColor                           TabColor[MAX_PLANNEDOBJ];// table color for planning obejcts
      double                           NowJD;               // current julian day

      // load configuration parameter from ini file
      void                             LoadConfigurationData( void );
      // save configiration parameter to ini file
      void                             SaveConfigurationData( void );
      // save diary file
      void                             SaveDiary( void );
      // show error message box
      void                             ShowError ( TentaclesMgr::ERR Error );
      // update shot images with current scaling values
      void                             UpdateShotImages(void);
      // update Histogram image with current scaling values
      void                             UpdateHistogram( void );
      // Refresh shot information
      void                             RefreshShot( void );
      // Load retain data
      void                             LoadRetainData( void );
      // Save retain data
      void                             SaveRetainData( void );
      // load file in the main script editor
      void                             LoadGRBScriptFile(void);
      // load file in the main script editor
      void                             LoadMainScriptFile(void);
      // load file in the time script editor
      void                             LoadTimeScriptFile(void);
      // load file in the close script editor
      void                             LoadCloseScriptFile(void);
      // load script in CPU
      void                             LoadScript(void);
      // Load dark list from library
      void                             LoadDarkList( void );
      // Update mail grid
      void                             UpdateMailGrid(void);
      // detect GRB
      void                             DetectGRB(void);
      // update planning graph
      void                             UpdatePlanningGraph(void);

   public:
      // configuration data
      CMHours::HOURS                   ObsLongitude;        // observatory longitude in hours
      CMDegree::DEGREE                 ObsLatitude;         // observatory latitude
      double                           PixelScale;          // arcsec for a pixel in shot image
      int                              SerialPower;         // serial port number for controller power on
      int                              SPortNumber;         // serial port number for LX200 connection
      int                              SPortBaudRate;       // serial port baud rate for LX200 connection
      char                             ProgramFolder[MAXPATH];  // folder wher program started
      char                             GSCFolder[MAXPATH];  // folder containig GSC catalog
      char                             GSCResolutionFolder[MAXPATH];  // folder containig GSC catalog for plate resolution
      char                             ObjectsFolder[MAXPATH]; // folder contaning sky object catalogs
      char                             ShotFolder[MAXPATH]; // base folder for nighlty shot
      char                             DarkFolder[MAXPATH]; // base folder for dark shot
      char                             ScriptFolder[MAXPATH]; // folder contaning scripts
      char                             SNFolder[MAXPATH];   // folder contaning reference SN images
      bool                             EnableCommTimeout;   // communication timeout enable
      bool                             EnableDiary;         // diary enable
      bool                             EnablePEC;           // PEC enable
      int                              PECAmplitude;        // PEC sin amplitude
      int                              PECPhase;            // PEC sin phase
      int                              SafeElevation;       // safe elevation in degree for program working
      char                             CCDName[40];         // name of CCD
      int                              SelectionRadius;     // radius of selection circle
      int                              FrameWidth;          // frame over chart width
      int                              FrameHeight;         // frame over chart height
      char                             Telescope[80];       // telecope type
      int                              NumObserver;         // number of valid observer in the array
      char                             Observers[MAX_OBSERVER][80];  // observer array
      char                             ObserverList[200];   // observers list
      int                              MinElevation;        // minimum elevation in degree for alert taking care
      int                              MaxDelay;            // maximum delay in minutes for alert taking take
      int                              MaxMagResolution;    // maximum magnitude per plate resolution
      bool                             GRBDetected;         // GRB detected
      CMHours::HOURS                   GRBRA;               // GRB right ascension
      CMDegree::DEGREE                 GRBDec;              // GRB declination
      bool                             AutomaticRun;        // automatic progam start

      // Form constructor
      __fastcall                       TMainForm(TComponent* Owner);
      // manages notify messages from observer pattern
      void                             ObsUpdate( int ANotifyCode );

};
//---------------------------------------------------------------------------
extern PACKAGE TMainForm *MainForm;
//---------------------------------------------------------------------------
#endif
