/******************************************************************************

	SNFMF.cpp : SN finder main form

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : December 2010

   SNF - SN automatic finder
   Copyright (C) 2009,2010,2012  Luca Pietro Strabla
   This file is part of SNF.
   SNF is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   SNF is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it

******************************************************************************/



// System include
#include <vcl.h>
#include <math.h>
#include <registry.hpp>
#pragma hdrstop



// project include
#include "PulisceMF.h"
#include "SNFConfig.h"
#include "LicenceForm.h"
#pragma package(smart_init)
#pragma link "PinPoint_OCX"
#pragma resource "*.dfm"



// global variables
TForm1                                 *Form1;



/*---------------------------------------------------------------------------

   Form constructor

---------------------------------------------------------------------------*/

__fastcall TForm1::TForm1(TComponent* Owner)
   : TForm(Owner)
{
   Plate = new Fits();                                      // create fits object
}



/*---------------------------------------------------------------------------

   Form destroyer

---------------------------------------------------------------------------*/

void __fastcall TForm1::FormDestroy(TObject *)
{
   // destroy internal objects
   delete Plate;
}



/*---------------------------------------------------------------------------

   Folder selection

---------------------------------------------------------------------------*/

void __fastcall TForm1::FolderButtonClick(TObject *Sender)
{
   DirDialog->DefaultExt = "fit";                           // assign extension
   DirDialog->FileName = "*.fit";                           // assign filter
   DirDialog->InitialDir = FolderEdit->Text;                // set initial dir
   if ( DirDialog->Execute() == true )                      // show dialog
      FolderEdit->Text = ExtractFilePath(DirDialog->FileName);    // get selected folder
}



/*---------------------------------------------------------------------------

   File clining
                                 
---------------------------------------------------------------------------*/

void __fastcall TForm1::Button1Click(TObject *Sender)
{
   Fits::FITS_VALUE                    *CurrentValue;
   Fits::ERR                           Error;
   TSearchRec                          SearchRec;
   AnsiString                          DirName;
   AnsiString                          FileName;
   int                                 Counter;
   char                                *pComment;
   int                                 Index;

   Counter = 0;
   StatusBar->Panels->Items[2]->Text = "Operazione in corso";
   DirName = FolderEdit->Text + "*.fit";
   FindFirst(DirName, 0, SearchRec);
   do
   {
      FileName = FolderEdit->Text + SearchRec.Name;
      Counter++;
      StatusBar->Panels->Items[0]->Text = SearchRec.Name;
      StatusBar->Panels->Items[1]->Text = AnsiString(Counter);
      StatusBar->Update();
      Error = Plate->LoadFile(FileName.c_str() );
      if ( Error != Fits::ERR_NONE )
         continue;
      Plate->GetFirst( &CurrentValue );

      while ( Plate->GetNext(&CurrentValue) == Fits::ERR_NONE )
      {
         pComment = CurrentValue->Value;
         Index = 0;
         while ( *pComment != '/' && Index < sizeof( CurrentValue->Value ) )
         {
            Index++;
            *pComment++;
         }
         while ( Index < sizeof( CurrentValue->Value ) )
         {
            Index++;
            *pComment++ = ' ';
         }

         if ( strcmp( CurrentValue->Key, "XORGSUBF" ) == 0 )
            Plate->Remove();
         if ( strcmp( CurrentValue->Key, "YORGSUBF" ) == 0 )
            Plate->Remove();
         if ( strcmp( CurrentValue->Key, "SBSTDVER" ) == 0 )
            Plate->Remove();
         if ( strcmp( CurrentValue->Key, "FLIPSTAT" ) == 0 )
            Plate->Remove();
         if ( strncmp( CurrentValue->Value+1, "MaxIm", 5 ) == 0 )
            Plate->Remove();
         if ( strncmp( CurrentValue->Value+1, "JeRRy", 5 ) == 0 )
            Plate->Remove();
      }
      Plate->SaveFile(FileName.c_str());

   } while (FindNext(SearchRec) == 0);
   FindClose(SearchRec);
   StatusBar->Panels->Items[2]->Text = "Operazione completata";
}


