/******************************************************************************

	SNFMF.cpp : SN finder main form

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : November 2011

   SNF - SN automatic finder
   Copyright (C) 2009,2010,2012  Luca Pietro Strabla
   This file is part of SNF.
   SNF is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   SNF is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it

******************************************************************************/



// System include
#include <vcl.h>
#include <math.h>
#include <stdio.h>
#include <registry.hpp>
#pragma hdrstop



// project include
#include "SNFMF.h"
#include "SNFConfig.h"
#include "LicenceForm.h"
#pragma package(smart_init)
#pragma resource "*.dfm"



// global variables
TForm1                                 *Form1;
int                                    TabPixel[]={ 285,148};



/*---------------------------------------------------------------------------

   Form constructor

---------------------------------------------------------------------------*/

__fastcall TForm1::TForm1(TComponent* Owner)
   : TForm(Owner)
{
   int                                 Size;                // struct size
   int                                 Index;               // scanning index
   char                                Drive[MAXDRIVE];     // test image driver
   char                                Dir[MAXDIR];         // test image directory
   char                                Name[MAXFILE];       // test image file name
   char                                Ext[MAXEXT];         // test image file extension

   // initializations
   TestImageBuffer = 0;                                     // no test image
   ReferenceImageBuffer = 0;                                // no reference image
   TestLock = false;                                        // test image not locked
   ReferenceLock = false;                                   // reference image not locked

   FitsImage = new Fits();                                  // create fits object

   // gray palette creation for test and reference bitmaps
   Size = sizeof( GrayPalette );
   memset ( &GrayPalette, 0, Size );                        // erase palette definition
   GrayPalette.PalHeader.palVersion = 0x300;                // set version
   GrayPalette.PalHeader.palNumEntries = GRAY_PALETTE_ENTRY;
   for ( Index=0; Index<GRAY_LEVELS; Index++ )
   {
      GrayPalette.PalHeader.palPalEntry[Index].peRed = (BYTE)Index;
      GrayPalette.PalHeader.palPalEntry[Index].peGreen = (BYTE)Index;
      GrayPalette.PalHeader.palPalEntry[Index].peBlue = (BYTE)Index;
      GrayPalette.PalHeader.palPalEntry[Index].peFlags = 0;
   }

   // add an extracolor (green) for lines signing stars
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE].peRed = 0;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE].peGreen = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE].peBlue = 0;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE].peFlags = 0;

   // add an extracolor (red) for lines signing stars
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE2].peRed = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE2].peGreen = 0;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE2].peBlue = 0;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE2].peFlags = 0;

   // add an extracolor (yellow) for lines signing stars
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE3].peRed = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE3].peGreen = 255;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE3].peBlue = 0;
   GrayPalette.PalHeader.palPalEntry[COLOR_CIRCLE3].peFlags = 0;

   HPalette = CreatePalette((const tagLOGPALETTE *)&GrayPalette); // palette creation from structure

   // create bitmaps
   InternalBmp = new Graphics::TBitmap();                   // create bitmap for image
   InternalBmp->PixelFormat = pf8bit;                       // bitmap with 8 bit pixel format
   InternalBmp->Palette = HPalette;                         // assign gray palette to bitmap

  // gray palette creation for test and reference zoomed bitmaps
   GrayZoomPalette.PalHeader.palVersion = 0x300;            // set version
   GrayZoomPalette.PalHeader.palNumEntries = GRAY_PALETTE_ENTRY;// se entry count
   for ( Index=0; Index<GRAY_LEVELS; Index++ )              // create all gray level
   {
      GrayZoomPalette.PalHeader.palPalEntry[Index].peRed = (BYTE)Index;
      GrayZoomPalette.PalHeader.palPalEntry[Index].peGreen = (BYTE)Index;
      GrayZoomPalette.PalHeader.palPalEntry[Index].peBlue = (BYTE)Index;
      GrayZoomPalette.PalHeader.palPalEntry[Index].peFlags = 0;
   }

   // add an extracolor (Brown) for out border indication
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peRed = 128;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peGreen = 64;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peBlue = 0;
   GrayZoomPalette.PalHeader.palPalEntry[COLOR_OUTBORDER].peFlags = 0;
   HPaletteZoom = CreatePalette((const tagLOGPALETTE *)&GrayZoomPalette); // palette creation from structure

   // create bitmaps
   InternalZoomBmp = new Graphics::TBitmap();               // create bitmap for zoomed image
   InternalZoomBmp->PixelFormat = pf8bit;                   // bitmap with 8 bit pixel format
   InternalZoomBmp->Palette = HPaletteZoom;                 // assign gray palette to bitmap
   InternalZoomBmp->Width = PIXEL_ZOOMED*ZOOM_FACTOR;       // assign size to zoomed bitmap
   InternalZoomBmp->Height = PIXEL_ZOOMED*ZOOM_FACTOR;

   // get current folder
   fnsplit ( ParamStr(0).c_str(), Drive, Dir, Name, Ext);
   strcpy( CurrentPath, Drive );
   strcat( CurrentPath, Dir );
}



/*---------------------------------------------------------------------------

   Shows main form

---------------------------------------------------------------------------*/

void __fastcall TForm1::FormShow(TObject *)
{
   TRegistry                           *Reg;                // registry obejct
   AnsiString                          Licence;             // licence enable

   // ask for licence if not jet accepted
   Reg = new TRegistry();                                   // create registry object
   Licence = "";                                            // licence not enabled
   if ( Reg->OpenKey( "Software\\SNF", false ) == true )    // open SNF registry key
      Licence = Reg->ReadString("Licence");                 // read licence acepted
   delete Reg;                                              // destroy registry object

   // check if licence was acepted
   if ( Licence != "Y")
      LicenceItemClick( 0 );                                // show licence form

   LoadRetainData();                                        // load retain data (comparing parameters)
   LoadTestFiles();                                         // load test file list
}



/*---------------------------------------------------------------------------

   Form destroyer

---------------------------------------------------------------------------*/

void __fastcall TForm1::FormDestroy(TObject *)
{
   // destroy internal objects
   delete FitsImage;
   delete InternalBmp;
   delete InternalZoomBmp;

   SaveRetainData();                                        // save retain data

   // free allocated buffer is they was
   if ( TestImageBuffer != 0 )
      free ( TestImageBuffer );
   if ( ReferenceImageBuffer != 0 )
      free ( ReferenceImageBuffer );
}



/*---------------------------------------------------------------------------

   an image is selected: show it, show reference image and allign them if not jet

---------------------------------------------------------------------------*/

void __fastcall TForm1::ShowImages(TObject *)
{
   bool                                Ok;                  // images was loaded and aligned
   int                                 ItemIndex;           // index for list item accessing
   ImageCompare                        *Compare;            // image compare object for current image
   ImageCompare::ERR                   CompareError;        // image compare error code
   AnsiString                          FileName;            // fit file name
   Fits::ERR                           Error;               // Fits error code
   char                                TestImagePath[MAXPATH]; // test image name
   char                                RefImagePath[MAXPATH]; // reference image name
   char                                Temp1[20];
   char                                Temp2[20];
   double                              Faintest;            // faintest object matched
   double                              Maximum;             // maximum magnitude used in compatation

   // initialization
   Ok = true;
   memset( &ManualParam, -1, sizeof(ManualParam));          // remove manual parameters
   StatusBar->Panels->Items[0]->Text = "";                  // remove alignment parameters
   StatusBar->Panels->Items[1]->Text = "";
   StatusBar->Panels->Items[2]->Text = "";
   StatusBar->Panels->Items[3]->Text = "";                  // remove activity from status bar
   StatusBar->Panels->Items[4]->Text = "";                  // remove magnitude information
   DifferenceList = 0;                                      // invalidate lists
   AlignmentList = 0;
   // remove previous images
   if ( TestImageBuffer != 0 )
   {
      free ( TestImageBuffer );                             // release previous buffer
      TestImageBuffer = 0;                                  // erase pounter
   }
   if ( ReferenceImageBuffer != 0 )
   {
      free ( ReferenceImageBuffer );                        // release previous buffer
      ReferenceImageBuffer = 0;                             // erase pounter
   }

   ItemIndex = TestFileListBox->ItemIndex;                  // get current index
   if ( ItemIndex == -1 )                                   // is should be set
      Ok = false;

   if ( Ok == true )
   {
      // get image compare for current image
      Compare = (ImageCompare*) TestFileListBox->Items->Objects[ItemIndex];
      if ( Compare == 0 )
      {
         strcpy( TestImagePath, TestFolder.c_str() );          // build test image file name
         strcat( TestImagePath, TestFileListBox->Items->Strings[ItemIndex].c_str() );
         strcpy( RefImagePath, RefFolder.c_str() );            // build reference image file name
         strcat( RefImagePath, TestFileListBox->Items->Strings[ItemIndex].c_str() );
         Compare = new ImageCompare( CurrentPath );            // create comparing object
         Compare->SetImages(RefImagePath, TestImagePath );     // assign images name
         Compare->SetSolvingParam(&Solving);
         TestFileListBox->Items->Objects[ItemIndex] = (TObject*)Compare;
      }
 
      DifferenceList = Compare->GetDifferenceObjects();
      AlignmentList = Compare->GetAlignmentObjects();

      // set allignment activity in status bar
      StatusBar->Panels->Items[4]->Text = "Allinamento in corso";
      StatusBar->Update();                                  // force show text (this function can be called in automatic)

      // load test image
      TestImageName = TestFileListBox->Items->Strings[ItemIndex];  // get selected file name
      FileName = TestFolder + TestImageName;                // create complete file name
      Error = FitsImage->LoadFile(FileName.c_str());        // load image from file
      if ( Error != Fits::ERR_NONE )                        // check error code
      {
         StatusBar->Panels->Items[4]->Text = "Immagine di test non presente";
         Ok = false;
      }
   }

   if ( Ok == true )
   {
      // compute test image optimal scaling values
      GetInfoFromFits( &TestInfo );                         // get image info
      TestImageBuffer = GetImageFromFits ( &TestInfo );     // get image buffer
      if ( TestInfo.SizeX == 376 && TestInfo.SizeY == 290 )
         Compare->SetHotPixel(sizeof(TabPixel)/8, TabPixel);// set hot pixel table

      // load reference image
      RefImageName = TestImageName;                         // basic test name is the same
      FileName = RefFolder + RefImageName;                  // create complete file name
      Error = FitsImage->LoadFile(FileName.c_str());        // load image from file
      if ( Error != Fits::ERR_NONE )                        // check error code
      {
         StatusBar->Panels->Items[4]->Text = "Immagine di riferimento non presente";
         Ok = false;
      }
   }

   if ( Ok == true )
   {
      // compute reference image optimal scaling values
      GetInfoFromFits( &ReferenceInfo );                    // get image info
      ReferenceImageBuffer = GetImageFromFits ( &ReferenceInfo );      // get image buffer
   }

   if ( Ok == true )
   {
      // check for allignement
      if ( Compare->GetAllignementStatus() == ImageCompare::ALIGNEMENT_NOTDONE )
      {
         CompareError = Compare->ImageAllignment();         // execute allignment
         if ( CompareError == ImageCompare::ERR_NONE )
            AlignmentList = Compare->GetAlignmentObjects(); // renew the alignment list
         else
         {
            AlignmentList = 0;
            Ok = false;
         }
      }
   }

   // build gray images with the current parameters ( or show black image )
   BuildGrayImages(0);
   UpdateAllignment( Compare );                             // update allignment information

   if ( Ok == true )
   {
      if ( Compare->GetCompareStatus() == false )
         ExecCompare();                                     // execute comparation

      if ( DifferenceList == 0 )                            // check difference object
         StatusBar->Panels->Items[4]->Text = "Nessuna differenza rilevata";
      else
      {
         // show message depending on the type of activity
         if ( Solving.SkipReferencMatch == false )
            StatusBar->Panels->Items[4]->Text = "Rilevate differenze";
         else
            StatusBar->Panels->Items[4]->Text = "Evidenziati oggetti";
      }

      // get magnitude involved in comparation
      Compare->GetMagnitude( &Faintest, &Maximum );
      gcvt(Faintest, 3, Temp1);
      gcvt(Maximum, 3, Temp2);
      StatusBar->Panels->Items[4]->Text = AnsiString("Mag: ") + AnsiString(Temp1) + " " + AnsiString(Temp2);
   }
   MarkAlignmentObjects();                                  // mark objects used for alignment
   MarkDifferenceObjects();                                 // mark objects with differences
   ListImagesWhitDifferences();                             // list images with at leat one differences
   ListCurrentDifferences();                                // list differences in current image
}



/*---------------------------------------------------------------------------

   Pagecontrol resize

---------------------------------------------------------------------------*/

void __fastcall TForm1::PageControlResize(TObject *Sender)
{
   int                                 SizeX;               // horizontal size

   SizeX = Generale->Width - CommandPanel->Width - 6;       // compute size for both reference and test panels
   RefPanel->Width = SizeX/2;                               // half space for reference panel (other is automatic )
}



/*---------------------------------------------------------------------------

   Show setup form

---------------------------------------------------------------------------*/

void __fastcall TForm1::SetupItemClick(TObject *Sender)
{
   TForm2                              *FormCfg;            // configuration dialog box
   int                                 ReturnCode;          // return dialog value

   FormCfg = new TForm2(this);                              // create configuration dialog box
   ReturnCode = FormCfg->ShowModal();                       // show  dialog box
   delete FormCfg;                                          // delete dialog box

   if ( ReturnCode == mrOk )                                // when new information
      LoadTestFiles();                                      // load again test file list

   SetCurrentDirectory(CurrentPath);                        // set again current directory changed by Configuration window
}



/*---------------------------------------------------------------------------

   show licence form

---------------------------------------------------------------------------*/

void __fastcall TForm1::LicenceItemClick(TObject *Sender)
{
   TForm4                              *LicenceForm;        // licence dialog box
   int                                 ReturnCode;          // return value
   TRegistry                           *Reg;                // registry obejct
   AnsiString                          Licence;             // licence enable

   LicenceForm = new TForm4(this);                          // create licence dialog box
   ReturnCode = LicenceForm->ShowModal();                   // show dialog box
   delete LicenceForm;                                      // delete dialog box

   // store selection
   Reg = new TRegistry();
   Licence = "";
   if ( Reg->OpenKey( "Software\\SNF", false ) == true )
   {
      if ( ReturnCode == mrOk )                             // licence accepted
         Reg->WriteString( "Licence", "Y" );
      else
         Reg->WriteString( "Licence", "N" );
   }
   delete Reg;

   if ( ReturnCode != mrOk )                                // licence not accepted
      Close();                                              // program stop
}



/*---------------------------------------------------------------------------

   application close

---------------------------------------------------------------------------*/

void __fastcall TForm1::ExitItemClick(TObject *Sender)
{
   Close();
}



/*---------------------------------------------------------------------------

   build and show gray images

---------------------------------------------------------------------------*/

void __fastcall TForm1::BuildGrayImages(TObject *Sender)
{
   Graphics::TBitmap                   *Bmp;                // woriking bit map
   int                                 SigmaCount;          // sigma count for automatic scaling
   int                                 Limit;               // limit for automatic scaling

   // show test image if it is loaded
   if ( TestImageBuffer != 0 )
   {
      SigmaCount = atoi ( SigmaMinusEdit->Text.c_str() ) ;  // get minimum sigma number
      Limit = TestInfo.BackgrondLevel - SigmaCount * TestInfo.BackgroundSigma;   // compute lower limit
      if ( Limit < 0 )                                      // clamp to 0
         Limit = 0;
      TestMinimumValue = Limit;                             // this the minimum scaling level
      SigmaCount = atoi ( SigmaPlusEdit->Text.c_str() ) ;   // get maximum sigma number
      Limit = TestInfo.BackgrondLevel + SigmaCount * TestInfo.BackgroundSigma;   // compute upper limit
      if ( Limit > VALUE_RANGE )                            // clamp to maximum ccd value
         Limit = VALUE_RANGE;
      TestMaximumValue = Limit;                             // this the maximum scaling level

      // show test bitmap
      TestImage->Width = TestInfo.SizeX;                    // adapt bitmap size to image
      TestImage->Height = TestInfo.SizeY;
      GetBmp( &Bmp, TestImageBuffer, &TestInfo, TestMinimumValue, TestMaximumValue);  // get scaled bitmap for
      TestImage->Picture->Bitmap = Bmp;                     // assign to test bitmap
      TestImage->Update();                                  // force immediate update
   }
   else
   {
      // no image: black image showed
      TestImage->Canvas->Brush->Color = clBlack;            // black image
      TestImage->Canvas->Rectangle( 0, 0, TestImage->Width, TestImage->Height);
      TestZoomImage->Canvas->Brush->Color = clBlack;        // black zoom image
      TestZoomImage->Canvas->Rectangle( 0, 0, TestZoomImage->Width, TestZoomImage->Height);
   }

   // show reference image if it is loaded
   if ( ReferenceImageBuffer != 0 )
   {
      SigmaCount = atoi ( SigmaMinusEdit->Text.c_str() ) + atoi ( Edit1->Text.c_str() );  // get minimum sigma count
      Limit = ReferenceInfo.BackgrondLevel - SigmaCount * TestInfo.BackgroundSigma;   // compute lower limit
      if ( Limit < 0 )                                      // clamp to 0
         Limit = 0;
      ReferenceMinimumValue = Limit;                        // this the minimum scaling level
      SigmaCount = atoi ( SigmaPlusEdit->Text.c_str() ) + atoi ( Edit1->Text.c_str() );   // get maximum sigma count
      Limit = ReferenceInfo.BackgrondLevel + SigmaCount * TestInfo.BackgroundSigma;   // compute upper limit
      if ( Limit > VALUE_RANGE )                            // clamp to maximum ccd value
         Limit = VALUE_RANGE;
      ReferenceMaximumValue = Limit;                        // this the maximum scaling level

      // show reference bitmap
      ReferenceImage->Width = ReferenceInfo.SizeX;          // adapt bitmap size to image
      ReferenceImage->Height = ReferenceInfo.SizeY;
      GetBmp( &Bmp, ReferenceImageBuffer, &ReferenceInfo, ReferenceMinimumValue, ReferenceMaximumValue); // get scaled bitmap for
      ReferenceImage->Picture->Bitmap = Bmp;                // assign to reference bitmap
      ReferenceImage->Update();                             // force immediate update
   }
   else
   {
     // no image: black image showed
      ReferenceImage->Canvas->Brush->Color = clBlack;       // black image
      ReferenceImage->Canvas->Rectangle( 0, 0, ReferenceImage->Width, ReferenceImage->Height);
      RefZoomImage->Canvas->Brush->Color = clBlack;         // black zoom image
      RefZoomImage->Canvas->Rectangle( 0, 0, RefZoomImage->Width, RefZoomImage->Height);
   }

   MarkAlignmentObjects();                                  // mark objects used for alignment
   MarkDifferenceObjects();                                 // mark objects with differences
}



/*---------------------------------------------------------------------------

   detect mouse move in test image

---------------------------------------------------------------------------*/

void __fastcall TForm1::TestImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y)
{
   Graphics::TBitmap                   *Bmp;                // zoomed image
   unsigned short                      *pPixel;             // pointer for image scan
   double                              XCent;               // horizontal from center position
   double                              YCent;               // vertical from center posiiton
   int                                 XRot;                // horizontal rotated posiiton
   int                                 YRot;                // vertical rotated posiiton

   if ( TestLock == true )                                  // skip when zoom image was locked
      return;
   if ( TestImageBuffer == 0 )                              // skip when no image
      return;

   // get zoomed image
   GetZoomBmp( &Bmp, TestImageBuffer, &TestInfo, TestMinimumValue, TestMaximumValue, X, Y);

   TestZoomImage->Picture->Bitmap = Bmp;                    // assign bitmap to image
   XTestEdit->Text = X;                                     // show image position
   YTestEdit->Text = Y;
   pPixel = TestImageBuffer + Y * TestInfo.SizeX + X;       // show image value
   ValueTestEdit->Text = *pPixel;

   // simulate mouse move on reference image in alligned position
   XCent = X - ShiftX;                                      // apply reverse shift
   YCent = Y - ShiftY;
   XCent = XCent - TestInfo.SizeX/2;                        // convert bitmap coordinates in center related coordinates
   YCent = -(YCent - TestInfo.SizeY/2);
   XRot = XCent * cos( -Rotation/RAD ) - YCent * sin ( -Rotation/RAD ); // apply reverse rotation
   YRot = XCent * sin( -Rotation/RAD ) + YCent * cos ( -Rotation/RAD );
   XRot += ReferenceInfo.SizeX/2;                           // convert center related coordinates to bitmap coordinates
   YRot = TestInfo.SizeY/2 - YRot;
   ReferenceImageMouseMove( Sender, Shift, XRot, YRot);     // simulate a mouse move on reference image
}



/*---------------------------------------------------------------------------

   mouse down in test image

---------------------------------------------------------------------------*/

void __fastcall TForm1::TestImageMouseDown
(
   TObject                             *Sender,
   TMouseButton                        Button,
   TShiftState                         Shift,
   int                                 X,
   int                                 Y
)
{
   TPoint                              Click;               // click point position
   int                                 ItemIndex;           // index for list item accessing

   // left click management
   if ( Button == mbLeft )
      TestLock = ! TestLock;                                // toggle look condition

   // click with right button shows shot popup menu
   if ( Button == mbRight )
   {
      ItemIndex = TestFileListBox->ItemIndex;              // get current index
      if ( ItemIndex != -1 )                               // is should be set
      {
         ImageClick = 0;                                   // click on test image
         XClick = X;                                       // save click position
         YClick = Y;
         Click.x = X;                                      // fill point with mouse position
         Click.y = Y;
         Click = TestImage->ClientToScreen(Click);         // convert position in screen coordinates
         // enable menu item depending on selections
         AllignMenuItem->Enabled = ManualParam.X1Test != -1 && ManualParam.X2Test != -1 && ManualParam.X1Ref != -1 && ManualParam.X2Ref != -1;
         AllignPopupMenu->Popup(Click.x+10, Click.y+10);   // shows shot popup menu
      }
   }
}



/*---------------------------------------------------------------------------

   detect mouse move in reference image

---------------------------------------------------------------------------*/

void __fastcall TForm1::ReferenceImageMouseMove(TObject *,
      TShiftState , int X, int Y)
{
   Graphics::TBitmap                   *Bmp;                // zoomed image
   unsigned short                      *pPixel;             // pointer for image scan

   if ( ReferenceLock == true )                             // skip when zoom image was locked
      return;
   if ( ReferenceImageBuffer == 0 )                         // skip when no image
      return;
   if ( X < 0 || X >= ReferenceImage->Width || Y < 0 || Y >= ReferenceImage->Height )  // skip out of image
      return;

   // get zoomed image
   GetZoomBmp( &Bmp, ReferenceImageBuffer, &ReferenceInfo, ReferenceMinimumValue, ReferenceMaximumValue, X, Y);

   RefZoomImage->Picture->Bitmap = Bmp;                     // assign bitmap to image
   XRefEdit->Text = X;                                      // show image position
   YRefEdit->Text = Y;
   pPixel = ReferenceImageBuffer + Y * ReferenceInfo.SizeX + X;  // show image value
   ValueRefEdit->Text = *pPixel;
}



/*---------------------------------------------------------------------------

   mouse down in reference image

---------------------------------------------------------------------------*/

void __fastcall TForm1::ReferenceImageMouseDown(TObject *,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   TPoint                              Click;               // click point position
   int                                 ItemIndex;           // index for list item accessing

   // manage left click
   if ( Button == mbLeft )
      ReferenceLock = ! ReferenceLock;                      // toggle look condition

   // click with right button shows shot popup menu
   if ( Button == mbRight )
   {
      ItemIndex = TestFileListBox->ItemIndex;              // get current index
      if ( ItemIndex == -1 )                               // is should be set
         return;

      ImageClick = 1;                                       // click on reference image
      XClick = X;                                           // save click position
      YClick = Y;
      Click.x = X;                                          // fill point with mouse position
      Click.y = Y;
      Click = ReferenceImage->ClientToScreen(Click);        // convert position in screen coordinates
      // enable menu item depending on selections
      AllignMenuItem->Enabled = ManualParam.X1Test != -1 && ManualParam.X2Test != -1 && ManualParam.X1Ref != -1 && ManualParam.X2Ref != -1;
      AllignPopupMenu->Popup(Click.x+10, Click.y+10);       // shows shot popup menu
   }
}



/*---------------------------------------------------------------------------

   whole image comparing

---------------------------------------------------------------------------*/

void __fastcall TForm1::AutomaticButtonClick(TObject *Sender)
{
   int                                 Index;               // image index from the list
   DWORD                               Start;               // init comparing time
   DWORD                               Stop;                // end comparing time

   TestFileListBox->ItemIndex = TestFileListBox->ItemIndex + 1;   // goto next imgage

   // initialize
   ComparedImages = 0;                                      // erase compared image counter
   StopRequest= false;                                      // erase stop request
   Start = GetTickCount();                                  // get initial time

   do
   {
      ShowImages(0);                                        // show curent images

      // stop autimatic scan if there are difference and it was wanted
      if ( StopCheckBox->Checked == true && DifferenceList != 0 )
         break;

      Stop = GetTickCount();                                // get curent time
      ComparedImageEdit->Text = ComparedImages;             // update compared image count
      ComparedTimeEdit->Text = AnsiString ( (Stop-Start)/1000 ); // update comparing time
      Index = TestFileListBox->ItemIndex + 1;               // goto next image
      TestFileListBox->ItemIndex = Index;                   // update image list selection
      Application->ProcessMessages();                       // update window items
   } while (Index < TestFileListBox->Items->Count && StopRequest == false);  // go until stop or list end

   MarkDifferenceObjects();                                 // show object lits
   ListImagesWhitDifferences();                             // update the new object list
}



/*---------------------------------------------------------------------------

   Stop automatic comparing

---------------------------------------------------------------------------*/

void __fastcall TForm1::StopButtonClick(TObject *Sender)
{
   StopRequest = true;                                      // activate stop request
}



/*---------------------------------------------------------------------------

   copy test image in reference image

---------------------------------------------------------------------------*/

void __fastcall TForm1::CopyButtonClick(TObject *Sender)
{
   AnsiString                          SourceFile;          // source file name with path
   AnsiString                          DestFile;            // destination file name with path

   SourceFile = TestFolder + TestImageName;                 // get source name
   DestFile = RefFolder + RefImageName;                     // get destination name

   CopyFile( SourceFile.c_str(), DestFile.c_str(), false ); // copy file
   ShowImages(0);                                           // update images
}



/*---------------------------------------------------------------------------

   Show next image with difference

---------------------------------------------------------------------------*/

void __fastcall TForm1::DownButtonClick(TObject *Sender)
{
   ImageCompare                        *Compare;            // image compare object for current image
   int                                 ItemIndex;           // index for list item accessing
   ImageCompare::OBJECT_CHAIN          *CurrentDifferenceList; // difference list for current image

   ItemIndex = TestFileListBox->ItemIndex;                  // get current image index

   // scan until the list is finisched
   while (ItemIndex < TestFileListBox->Items->Count-1 )
   {
      ItemIndex++;                                          // goto next image

      Compare = (ImageCompare*) TestFileListBox->Items->Objects[ItemIndex];   // get current image compare object
      if ( Compare != 0 )
      {
         CurrentDifferenceList = Compare->GetDifferenceObjects(); // get difference objects list
         if ( CurrentDifferenceList != 0 )                  // check for any difference
         {
            TestFileListBox->ItemIndex  = ItemIndex;        // select image on the list
            ShowImages(0);                                  // show image
            break;
         }
      }
   }
}



/*---------------------------------------------------------------------------

   Show previous image with difference

---------------------------------------------------------------------------*/

void __fastcall TForm1::UpButtonClick(TObject *Sender)
{
   ImageCompare                        *Compare;            // image compare object for current image
   int                                 ItemIndex;           // index for list item accessing
   ImageCompare::OBJECT_CHAIN          *CurrentDifferenceList; // difference list for current image

   ItemIndex = TestFileListBox->ItemIndex;                  // get current image index

   // scan until the list is finisched
   while (ItemIndex > 0)
   {
      ItemIndex--;                                          // goto previous image

      Compare = (ImageCompare*) TestFileListBox->Items->Objects[ItemIndex];   // get current image compare object
      if ( Compare != 0 )
      {
         CurrentDifferenceList = Compare->GetDifferenceObjects(); // get difference objects list
         if ( CurrentDifferenceList != 0 )                  // check for any difference
         {
            TestFileListBox->ItemIndex  = ItemIndex;        // select image on the list
            ShowImages(0);                                  // show image
            break;
         }
      }
   }
}



/*---------------------------------------------------------------------------

   Set object 1 position for manuale alignment

---------------------------------------------------------------------------*/

void __fastcall TForm1::SetObject1Click(TObject *Sender)
{
   // wich image ?
   if ( ImageClick == 0 )
   {
      // test image position
      ManualParam.X1Test = XClick;
      ManualParam.Y1Test = YClick;
   }
   else
   {
      // reference image position
      ManualParam.X1Ref = XClick;
      ManualParam.Y1Ref = YClick;
   }
}



/*---------------------------------------------------------------------------

   Set object 2 position for manuale alignment

---------------------------------------------------------------------------*/

void __fastcall TForm1::SetObject2Click(TObject *Sender)
{
   // wich image ?
   if ( ImageClick == 0 )
   {
      // test image position
      ManualParam.X2Test = XClick;
      ManualParam.Y2Test = YClick;
   }
   else
   {
      // reference image position
      ManualParam.X2Ref = XClick;
      ManualParam.Y2Ref = YClick;
   }
}



/*---------------------------------------------------------------------------

   execute manuale alignment

---------------------------------------------------------------------------*/

void __fastcall TForm1::AllignMenuItemClick(TObject *Sender)
{
   int                                 ItemIndex;           // index for list item accessing
   ImageCompare                        *Compare;            // compare image object

   ItemIndex = TestFileListBox->ItemIndex;                  // get current selecte image index
   if ( ItemIndex != -1 )
   {
      Compare = (ImageCompare*) TestFileListBox->Items->Objects[ItemIndex]; // get current image compare
      if ( Compare != 0 )
      {
         Compare->ManualAllignment(&ManualParam);           // ask manual alignment
         UpdateAllignment( Compare );                       // update alignment result
         memset( &ManualParam, -1, sizeof(ManualParam));    // remove manual parameters
      }
   }
}



/*---------------------------------------------------------------------------

   Load fit image list from test folder

---------------------------------------------------------------------------*/

void TForm1::LoadTestFiles( void )
{
   AnsiString                          Path;                // searching argument
   TSearchRec                          FindRec;             // record for file find operations
   int                                 Status;              // request status
   ImageCompare                        *Compare;            // compare image object
   int                                 Index;               // scan index
   char                                TestImageName[MAXPATH]; // test image name
   char                                RefImageName[MAXPATH]; // reference image name

   // remove all previous objects in the list
   for ( Index=0; Index<TestFileListBox->Items->Count; Index++)
   {
      Compare = (ImageCompare*) TestFileListBox->Items->Objects[Index]; // get object pointer from listbox
      if ( Compare != 0 )                                   // was object assgned ?
         delete Compare;                                    // remove object
   }

   TestFileListBox->Items->Clear();                         // remove all listbox content

   // loas new image list from folder
   Path = TestFolder + "\\*.fit";                           // create seaching path
   Status = FindFirst(Path, faAnyFile, FindRec);            // find the first element
   while ( Status == 0 )                                    // until files are present
   {
      strcpy( TestImageName, TestFolder.c_str() );          // build test image file name
      strcat( TestImageName, FindRec.Name.c_str() );
      strcpy( RefImageName, RefFolder.c_str() );            // build reference image file name
      strcat( RefImageName, FindRec.Name.c_str() );
      TestFileListBox->AddItem(FindRec.Name, 0 );           // add file name to listbox

      Status = FindNext( FindRec );                         // find another fit file
   }
   FindClose(FindRec);                                      // close search
}



/*---------------------------------------------------------------------------

   Update allignment information

---------------------------------------------------------------------------*/

void TForm1::UpdateAllignment( ImageCompare *Compare )
{
   ImageCompare::ALIGNEMENT_STATUS     AlignmentStatus;     // current alignment status
   char                                Temp1[20];

   AlignmentStatus = Compare->GetAllignementStatus();
   switch ( AlignmentStatus )
   {
      case ImageCompare::ALIGNEMENT_NOTDONE:
         ShiftX = 0;                                        // erase alignment paramaters
         ShiftY = 0;
         Rotation = 0;
         StatusBar->Panels->Items[0]->Text = "";            // remove alignment parameters
         StatusBar->Panels->Items[1]->Text = "";
         StatusBar->Panels->Items[2]->Text = "";
         StatusBar->Panels->Items[3]->Text = "Immagini non allineate";
         break;

      case ImageCompare::ALIGNEMENT_DONE:
         Compare->GetAllignment( &ShiftX, &ShiftY, &Rotation );
         gcvt(ShiftX, 4, Temp1);
         StatusBar->Panels->Items[0]->Text = AnsiString("Spostamento orizzontale: ") + AnsiString(Temp1);
         gcvt(ShiftY, 4, Temp1);
         StatusBar->Panels->Items[1]->Text = AnsiString("Spostamento verticale: ") + AnsiString(Temp1);
         gcvt(Rotation, 4, Temp1);
         StatusBar->Panels->Items[2]->Text = AnsiString("Rotazione: ") + AnsiString(Temp1) + "";
         StatusBar->Panels->Items[3]->Text = "Immagini allineate";
         break;

      case ImageCompare::ALIGNEMENT_NOTFOUND:
         ShiftX = 0;                                        // erase alignment paramaters
         ShiftY = 0;
         Rotation = 0;
         StatusBar->Panels->Items[0]->Text = "";            // remove alignment parameters
         StatusBar->Panels->Items[1]->Text = "";
         StatusBar->Panels->Items[2]->Text = "";
         StatusBar->Panels->Items[3]->Text = "Allineamento non riuscito";
         break;
   }
}



/*---------------------------------------------------------------------------

   get image info from current plate

---------------------------------------------------------------------------*/

void TForm1::GetInfoFromFits( IMAGE_INFO *Info )
{
   int                                 TotalSize;           // total image size
   unsigned short                      *pImage;             // pointer for image
   unsigned short                      *pImageScan;         // pointer for image scan
   int                                 Value;               // pixel value
   int                                 Histogram[VALUE_RANGE]; // Histogram of value distribuition
   int                                 IndexHisto;          // current histogram element
   int                                 IndexPixel;          // current pixel
   int                                 MaxHistoValue;       // maximum value in light histogram
   int                                 MaxHistoIndex;       // maximum index in light histogram
   int                                 Diff;                // difference for sigma computing
   double                              Sum;                 // Sum for sigma computing
   double                              Deno;                // denominator for sigma computing
   double                              Sigma;               // background sigma

   FitsImage->GetImageSize( &Info->SizeY, &Info->SizeX );   // get image size
   TotalSize = Info->SizeY * Info->SizeX * 2;               // get memory image size
   pImage = (unsigned short*) malloc( TotalSize );          // allocate image buffer
   FitsImage->GetImage( pImage );                           // get image

   // erase histogram array
   memset( Histogram, 0, sizeof(Histogram) );
   // scan image in order to build histogram
   pImageScan = pImage;
   for ( IndexPixel=0; IndexPixel<(Info->SizeX * Info->SizeY); IndexPixel++ ) // scan all pixel in the image
   {
      Value = *pImageScan++;                                // get pixel value
      Histogram[Value] ++;                                  // increment pixel number having this value
   }
   Histogram[0] = 0;                                        // pixel at value 0 not involved

   // findout the maximum value in historam
   MaxHistoValue=0;                                         // reset values
   MaxHistoIndex=0;
   for ( IndexHisto=0; IndexHisto < VALUE_RANGE; IndexHisto++)// scan all elements in histogram
   {
      if ( Histogram[IndexHisto] > MaxHistoValue )          // new value greater than the until now maximum ?
      {
         MaxHistoValue = Histogram[IndexHisto];             // store new maximum
         MaxHistoIndex = IndexHisto;                        // store its index
      }
   }
   Info->BackgrondLevel = MaxHistoIndex;                    // background level is the index of maximum in histogram

   // compute background sigma
   Sum = 0;
   Deno = 0;
   for ( IndexHisto=0; IndexHisto<MaxHistoIndex; IndexHisto++)// scan elements in histogram from 0 to maximum value index
   {
      Diff = IndexHisto - MaxHistoIndex;                    // difference (error) between mean value and pixel value
      Diff *= Diff;                                         // square difference
      Diff *= Histogram[IndexHisto]*2;                      // weigh this point with histogram hight
      Sum += Diff;                                          // sum the
      Deno += Histogram[IndexHisto]*2;                      // number of point
   }
   if ( Deno == 0 )
      Sigma = 0;
   else
      Sigma = Sum / Deno;                                   // variance
   Sigma = pow( Sigma, 0.5 );                               // final sigma
   Info->BackgroundSigma = Sigma;

   free ( pImage );
   return;
}



/*---------------------------------------------------------------------------

   get image from fits file

---------------------------------------------------------------------------*/

unsigned short * TForm1::GetImageFromFits( IMAGE_INFO *Info )
{
   unsigned short                      *pImage;             // pointer for image
   int                                 TotalSize;           // total image size

   TotalSize = Info->SizeX * Info->SizeY * 2;               // compute total size
   pImage = (unsigned short*) malloc( TotalSize );          // memory allocation for image
   FitsImage->GetImage( pImage );                           // get image
   return pImage;
}



/*---------------------------------------------------------------------------

   Create bmp from image

---------------------------------------------------------------------------*/

void TForm1::GetBmp( Graphics::TBitmap ** Bmp, unsigned short*Image, IMAGE_INFO *Info, int Min, int Max )
{
   int                                 IndexRow;            // index for row scan
   int                                 IndexCol;            // index for column scan
   unsigned char                       *pPixel;             // pointer for bitmap scan
   unsigned short                      *pImageScan;         // pointer for image scan
   int                                 Value;               // pixel value

   InternalBmp->Width = Info->SizeX;                        // bitmap size as plate size
   InternalBmp->Height = Info->SizeY;

   // build bitmap from the image with input minimum and maximum
   pImageScan = Image;
   for ( IndexRow=0; IndexRow<Info->SizeY; IndexRow++ )     // scan all the line
   {
      pPixel = (unsigned char*)InternalBmp->ScanLine[IndexRow];   // get line pointer in the bimap
      for ( IndexCol=0; IndexCol<Info->SizeX; IndexCol++ )  // scan pixel in the column
      {
         Value = *pImageScan++;                             // get value from the image
         Value = ComputeGrayValue( Value, Min, Max);        // scale value
         *pPixel++ = (unsigned char)Value;
      }
   }

  	*Bmp = InternalBmp;                                      // yeld bitmap
  	return ;
}



/*---------------------------------------------------------------------------

   Create zoomed bmp from image

---------------------------------------------------------------------------*/

void TForm1::GetZoomBmp( Graphics::TBitmap ** Bmp, unsigned short*Image, IMAGE_INFO *Info, int Min, int Max, int CenterX, int CenterY )
{
   int                                 IndexRow;            // index for row scan
   int                                 IndexCol;            // index for column scan
   unsigned char                       *pPixel;             // pointer for bitmap scan
   unsigned short                      *pImage;             // pointer for image scan
   int                                 Value;               // pixel value
   int                                 InitX;               // init orizontal pixel
   int                                 InitY;               // init vertical pixel
   int                                 ImageX;              // x position in the normal image
   int                                 ImageY;              // y posiziont in the normal image

   InitX=CenterX-PIXEL_ZOOMED/2;                            // set init image from center zoom point
   InitY=CenterY-PIXEL_ZOOMED/2;                            // these can be negative (out border)

   // build bitmap from the image with current minimum and maximum
   for ( IndexRow=0; IndexRow<InternalZoomBmp->Height; IndexRow++ )// scan all the line of bitmap
   {
      pPixel = (unsigned char*)InternalZoomBmp->ScanLine[IndexRow];   // get line pointer in the bimap
      ImageY = InitY + IndexRow / ZOOM_FACTOR ;             // compute sorce line from line in zoom image
      pImage = Image+ ImageY * Info->SizeX;                 // compute source image line
      for ( IndexCol=0; IndexCol<InternalZoomBmp->Width; IndexCol++ ) // scan pixel in the column of bitmap
      {
         ImageX = InitX + IndexCol / ZOOM_FACTOR;           // compute source column from colum in zoom image
         if ( ImageX < 0 || ImageX >= Info->SizeX ||
              ImageY < 0 || ImageY >= Info->SizeY )         // check current line and column are out out image border
            Value = COLOR_OUTBORDER;                        // out border pixel showed with out border color
         else
         {
            Value = pImage[ImageX];                         // get value from the image
            Value = ComputeGrayValue( Value, Min, Max);     // scale gray value
         }
         *pPixel++ = (unsigned char)Value;                  // assign bitmap pixel color
      }
   }

   // draw red cross in the center
   InternalZoomBmp->Canvas->Pen->Color = clRed;
   ImageX = InternalZoomBmp->Width / 2;
   ImageY = InternalZoomBmp->Height / 2;
   InternalZoomBmp->Canvas->MoveTo( ImageX-1, ImageY );
   InternalZoomBmp->Canvas->LineTo( ImageX+2, ImageY );
   InternalZoomBmp->Canvas->MoveTo( ImageX, ImageY-1 );
   InternalZoomBmp->Canvas->LineTo( ImageX, ImageY+2 );

  	*Bmp = InternalZoomBmp;                                  // yeld bitmap
}



/*---------------------------------------------------------------------------

   compute gray scaled value

---------------------------------------------------------------------------*/

int TForm1::ComputeGrayValue( int Value, int Min, int Max)
{
   if ( Max > Min )                                         // if max greater the min => positive image
   {
      if ( Value <= Min )
         Value = 0;                                         // image value less then min => color is black
      else
      {
         if ( Value >= Max )
            Value = GRAY_LEVELS-1;                          // image value greater then max => colore white
         else
         {
            Value -= Min;                                   // in the middle proportional gray
            Value *= (GRAY_LEVELS-1);
            Value /= (Max-Min);
         }
      }
   }
   else                                                     // if min greater the max => negative image
   {
      if ( Value >= Min )
         Value = 0;                                         // image value greater then min => color is blck
      else
      {
         if ( Value <= Max )
            Value = GRAY_LEVELS-1;                          // image value less then max => colore white
         else
         {
            Value -= Min;                                   // in the middle proportional gray
            Value *= (GRAY_LEVELS-1);
            Value /= (Max-Min);
         }
      }
   }
   return Value;
}



/*---------------------------------------------------------------------------

   execute comparation between reference and test image

---------------------------------------------------------------------------*/

void TForm1::ExecCompare( void )
{
   AnsiString                          Line;                // message string
   int                                 ItemIndex;           // index for list item accessing
   ImageCompare                        *Compare;            // compare image object
   ImageCompare::ERR                   Error;               // comparing error code

   // initialization
   DifferenceList = 0;                                      // remove previous difference list pointer
   CompList->Lines->Clear();                                // remove previous difference list
   StatusBar->Panels->Items[4]->Text = "Comparazione in corso"; // show current activity
   StatusBar->Update();

   ItemIndex = TestFileListBox->ItemIndex;                  // get current index
   if ( ItemIndex != -1 )                                   // an image should be selected
   {
      Compare = (ImageCompare*) TestFileListBox->Items->Objects[ItemIndex];  // get current image compare
      if ( Compare != 0 )
      {
         Error = Compare->Compare( );                       // excute comparation
         if ( Error == 0 )
            ComparedImages++;                               // increase compared image counter
         DifferenceList = Compare->GetDifferenceObjects();  // get difference list pointer
      }
   }
   return ;
}



/*---------------------------------------------------------------------------

   shows difference objects with red circle

---------------------------------------------------------------------------*/

void TForm1::MarkDifferenceObjects( void )
{
   ImageCompare::OBJECT_CHAIN          *Current;            // current scanning object
   int                                 XPos;                // horizontal new object position
   int                                 YPos;                // vertical new object position
   TColor                              Color;

   Current = DifferenceList;                                // get first difference object
   while ( Current != 0 )                                   // scan until end list
   {
      Color = clRed;
      if ( Current->Actual.Status == ImageCompare::OBJECT_BRIGHTER )
         Color = clYellow;

      XPos = Current->Actual.TestX;                         // get new object position in test image
      YPos = Current->Actual.TestY;

      TestImage->Canvas->Pen->Color = Color;                // set color
      TestImage->Canvas->Brush->Style = bsClear;            // brush don't fill inside circle
      TestImage->Canvas->Ellipse( XPos+8, YPos+8, XPos-8, YPos-8);   // draw circle

      XPos = Current->Actual.RefX;                          // get new object position in reference image
      YPos = Current->Actual.RefY;

      ReferenceImage->Canvas->Pen->Color = Color;           // set color
      ReferenceImage->Canvas->Brush->Style = bsClear;       // brush don't fill inside circle
      ReferenceImage->Canvas->Ellipse( XPos+8, YPos+8, XPos-8, YPos-8);  // draw circle

      Current = Current->Next;                              // get next object
   }
}



/*---------------------------------------------------------------------------

   shows alignment objects with green circle

---------------------------------------------------------------------------*/

void TForm1::MarkAlignmentObjects( void )
{
   ImageCompare::OBJECT_CHAIN          *Current;            // current scanning object
   int                                 XPos;                // horizontal new object position
   int                                 YPos;                // vertical new object position

   Current = AlignmentList;                                 // get first alignement object
   while ( Current != 0 )                                   // scan until end list
   {
      XPos = Current->Actual.TestX;                         // get new object position in test image
      YPos = Current->Actual.TestY;

      TestImage->Canvas->Pen->Color = clLime;               // set color
      TestImage->Canvas->Brush->Style = bsClear;            // brush don't fill inside circle
      TestImage->Canvas->Ellipse( XPos+8, YPos+8, XPos-8, YPos-8);   // draw circle

      XPos = Current->Actual.RefX;                          // get new object position in reference image
      YPos = Current->Actual.RefY;

      ReferenceImage->Canvas->Pen->Color = clLime;          // set color
      ReferenceImage->Canvas->Brush->Style = bsClear;       // brush don't fill inside circle
      ReferenceImage->Canvas->Ellipse( XPos+8, YPos+8, XPos-8, YPos-8);  // draw circle

      Current = Current->Next;                              // get next object
   }
}



/*---------------------------------------------------------------------------

   Load retain data from registry

---------------------------------------------------------------------------*/

void TForm1::LoadRetainData( void )
{
   TRegistry                           *Reg;                // registry obejct
   AnsiString                          Str;                 // temporary string
   AnsiString                          Key;                 // registry key

   Solving.SigmaFaintMag = 2.0;
   Solving.SigmaMagIncrease = 3.0;
   Solving.MinMagIncrease = 0.1;
   Solving.CreateMatchFile = false;;
   Solving.SkipReferencMatch = false;
   Solving.MinDetectionPixel = 1;
   Solving.DetectionThreshold = 2.0;
   Solving.ClassThreshold = 2.0;
   Solving.SaturationLevel = 50000;

   // open registry SNF key
   Reg = new TRegistry();
   if ( Reg->OpenKey( "Software\\SNF", false ) == true )
   {
      Str = Reg->ReadString("RefFolder");
      if ( Str != "" )
         RefFolder = Str;

      Str = Reg->ReadString("TestFolder");
      if ( Str != "" )
         TestFolder = Str;

      Str = Reg->ReadString("DetectionThreshold");
      if ( Str != "" )
         Solving.DetectionThreshold = atof( Str.c_str());

      Str = Reg->ReadString("SigmaFaintMag");
      if ( Str != "" )
         Solving.SigmaFaintMag = atof( Str.c_str());

      Str = Reg->ReadString("SigmaMagIncrease");
      if ( Str != "" )
         Solving.SigmaMagIncrease = atof( Str.c_str());

      Str = Reg->ReadString("MinMagIncrease");
      if ( Str != "" )
         Solving.MinMagIncrease = atof( Str.c_str());

      Str = Reg->ReadString("SkipReferencMatch");
      if ( Str == "1" )
         Solving.SkipReferencMatch = true;

      Str = Reg->ReadString("CreateMatchFile");
      if ( Str == "1" )
         Solving.CreateMatchFile = true;

      Str = Reg->ReadString("SigmaMinus");
      if ( Str != "" )
         SigmaMinusEdit->Text = Str;

      Str = Reg->ReadString("SigmaPlus");
      if ( Str != "" )
         SigmaPlusEdit->Text = Str;
   }
   delete Reg;
}



/*---------------------------------------------------------------------------

   Save retain data to registry

---------------------------------------------------------------------------*/

void TForm1::SaveRetainData( void )
{
   TRegistry                           *Reg;                // registry obejct
   char                                Str[20];             // temporary string
   AnsiString                          Key;                 // registry key

   // open registry SNF key
   Reg = new TRegistry();
   if ( Reg->OpenKey( "Software\\SNF", true ) == true )
   {
      Reg->WriteString( "RefFolder", RefFolder );

      Reg->WriteString( "TestFolder", TestFolder );

      gcvt( Solving.DetectionThreshold, 2, Str);
      Reg->WriteString( "DetectionThreshold", Str );

      gcvt( Solving.SigmaFaintMag, 2, Str);
      Reg->WriteString( "SigmaFaintMag", Str );

      gcvt( Solving.SigmaMagIncrease, 2, Str);
      Reg->WriteString( "SigmaMagIncrease", Str );

      gcvt( Solving.MinMagIncrease, 2, Str);
      Reg->WriteString( "MinMagIncrease", Str );

      strcpy( Str, "0");
      if ( Solving.SkipReferencMatch == true )
         strcpy( Str, "1");
      Reg->WriteString( "SkipReferencMatch", Str );

      strcpy( Str, "0");
      if ( Solving.CreateMatchFile == true )
         strcpy( Str, "1");
      Reg->WriteString( "CreateMatchFile", Str );

      Reg->WriteString( "SigmaMinus", SigmaMinusEdit->Text );
      Reg->WriteString( "SigmaPlus", SigmaPlusEdit->Text );
   }
   delete Reg;
}



/*---------------------------------------------------------------------------

   list image with differences

---------------------------------------------------------------------------*/

void TForm1::ListImagesWhitDifferences( void )
{
   AnsiString                          Line;                // message string
   int                                 ItemIndex;           // index for list item accessing
   ImageCompare                        *Compare;            // compare image object
   ImageCompare::OBJECT_CHAIN          *Current;            // found object
   int                                 NewObjects;          // new object count
   char                                Message[100];        // message for found objects list

   ImageMemo->Lines->Clear();                               // remove previous new object image list
   if ( Solving.SkipReferencMatch == true )
      return;

   ItemIndex = 0;                                           // goto to the first image

   while (ItemIndex < TestFileListBox->Items->Count )       // scan all images
   {
      Compare = (ImageCompare*) TestFileListBox->Items->Objects[ItemIndex];   // get current image compare
      if ( Compare != 0 )
      {
         Current = Compare->GetDifferenceObjects();         // get found objects
         if ( Current != 0 )                                // check for found objects
         {
            NewObjects = 0;                                 // erase object count
            while ( Current != 0 )                          // until object are the chain
            {
                NewObjects++;                               // increase counter
                Current = Current->Next;                    // goto next object
            }
            // create string message
            wsprintf( Message, "%-20s   %3d", TestFileListBox->Items->Strings[ItemIndex].c_str(), NewObjects );
            ImageMemo->Lines->Add(Message);                 // add message to list
         }
      }
      ItemIndex++;                                          // goto next image
   }
}



/*---------------------------------------------------------------------------

   show differences in current image

---------------------------------------------------------------------------*/

void TForm1::ListCurrentDifferences( void )
{
   AnsiString                          Line;                // message string
   int                                 ItemIndex;           // index for list item accessing
   ImageCompare                        *Compare;            // compare image object
   ImageCompare::OBJECT_CHAIN          *Current;            // found object
   char                                Message[100];        // message for found objects list
   char                                Reason[20];          // difference reasun
   char                                Temp1[20];           // temprary strings
   char                                Temp2[20];
   char                                Temp3[20];
   char                                Temp4[20];
   char                                Temp5[20];
   char                                Temp6[20];

   CompList->Lines->Clear();                                // remove previous new object image list

   ItemIndex = TestFileListBox->ItemIndex;                  // get current index
   if ( ItemIndex == -1 )                                   // is should be set
      return;

   Compare = (ImageCompare*) TestFileListBox->Items->Objects[ItemIndex];   // get current image compare
   if ( Compare == 0 )
      return;

   Current = Compare->GetDifferenceObjects();               // get found objects
   while ( Current != 0 )                                   // check for found objects
   {
      *Reason = 0;                                          // preventive reason clean
      // select reason description
      if ( Current->Actual.Status == ImageCompare::OBJECT_BRIGHTER )
         strcpy( Reason, "Pi luminoso" );
      if ( Current->Actual.Status == ImageCompare::OBJECT_NOTFOUND )
         strcpy( Reason, "Nuovo" );
      if ( Solving.SkipReferencMatch == true )
         strcpy( Reason, "" );

      // convert double in string
      gcvt(Current->Actual.TestX, 4, Temp1);
      gcvt(Current->Actual.TestY, 4, Temp2);
      gcvt(Current->Actual.TestMag, 4, Temp3);
      gcvt(Current->Actual.RefMag, 4, Temp4);
      gcvt(Current->Actual.AspectedMag, 4, Temp5);
      gcvt(Current->Actual.AspectedMag - Current->Actual.RefMag, 4, Temp6);

      // create string message
      sprintf( Message, "%-15s %5s %5s  %-7s   %-7s   %-7s  %s",
                Reason, Temp1, Temp2, Temp3, Temp4, Temp5, Temp6 );
      CompList->Lines->Add(Message);                     // add message to list
      Current = Current->Next;                           // goto next object
   }
}




