/******************************************************************************

	SNFMF.h | Header SN finder main form

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : November 2011

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   SNF - Supernovae finder
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus and SNF
   Polypus ans SNF are free software: you can redistribute it and/or modify
   them under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus ans SNF are distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it

******************************************************************************/



// include antirepeat
#ifndef SNFMFH
#define SNFMFH



// system include
#include <Dialogs.hpp>
#include <ComCtrls.hpp>
#include <Graphics.hpp>
#include <ExtCtrls.hpp>
#include <StdCtrls.hpp>
#include <Controls.hpp>
#include <Classes.hpp>


// project include
#include "ImageCompare.h"
#include "Fits.h"
#include <Menus.hpp>


// definitions
#define                                GRAY_PALETTE_ENTRY   256      // number of element in gray palette
#define                                GRAY_LEVELS          250      // number of gry level
#define                                COLOR_CIRCLE         251      // color for circle arround stars
#define                                COLOR_CIRCLE2        252      // color for circle arround stars
#define                                COLOR_CIRCLE3        253      // color for circle arround stars
#define                                COLOR_OUTBORDER      250      // color for out border shot
#define                                VALUE_RANGE          65536    // maximum pixel value
#define                                PIXEL_ZOOMED         29       // number of zoomed pixel
#define                                ZOOM_FACTOR          5        // zoom factor



class TForm1 : public TForm
{
   __published:	// IDE-managed Components
      TImage                           *ReferenceImage;
      TImage                           *TestImage;
      TButton                          *AutomaticButton;
      TListBox                         *TestFileListBox;
      TEdit                            *SigmaPlusEdit;
      TEdit                            *SigmaMinusEdit;
      TLabel                           *Label18;
      TPageControl                     *PageControl;
      TTabSheet                        *Generale;
      TTabSheet                        *Risultati;
      TStatusBar                       *StatusBar;
      TMemo                            *CompList;
      TLabel                           *Label3;
      TEdit                            *XRefEdit;
      TEdit                            *YRefEdit;
      TLabel                           *Label5;
      TLabel                           *Label6;
      TEdit                            *XTestEdit;
      TEdit                            *YTestEdit;
      TLabel                           *Label7;
      TLabel                           *Label8;
      TImage                           *RefZoomImage;
      TEdit                            *ValueRefEdit;
      TImage                           *TestZoomImage;
      TEdit                            *ValueTestEdit;
      TLabel                           *Label1;
      TOpenDialog                      *DirDialog;
      TButton                          *StopButton;
      TUpDown                          *UpDown1;
      TUpDown                          *UpDown2;
      TButton                          *CopyButton;
      TGroupBox                        *CommandGroupBox;
      TLabel                           *Label15;
      TEdit                            *ComparedImageEdit;
      TLabel                           *Label16;
      TEdit                            *ComparedTimeEdit;
      TMainMenu                        *MainMenu;
      TMenuItem                        *File1;
      TMenuItem                        *LicenceItem;
      TMenuItem                        *ExitItem;
      TPanel                           *CommandPanel;
      TPanel                           *RefPanel;
      TPanel                           *TestPanel;
      TSplitter                        *Splitter1;
      TLabel                           *Label19;
      TLabel                           *Label20;
      TLabel                           *Label21;
      TMenuItem                        *SetupItem;
      TCheckBox                        *StopCheckBox;
      TButton                          *UpButton;
      TButton                          *DownButton;
      TLabel                           *Label22;
      TMemo                            *ImageMemo;
      TLabel                           *Label23;
      TLabel                           *Label24;
      TPopupMenu                       *AllignPopupMenu;
      TMenuItem                        *SetObject1;
      TMenuItem                        *SetObject2;
      TMenuItem                        *AllignMenuItem;
   TEdit *Edit1;
   TUpDown *UpDown3;

      void __fastcall                  FormShow(TObject *Sender);
      void __fastcall                  FormDestroy(TObject *Sender);
      void __fastcall                  ShowImages(TObject *Sender);
      void __fastcall                  PageControlResize(TObject *Sender);
      void __fastcall                  SetupItemClick(TObject *Sender);
      void __fastcall                  LicenceItemClick(TObject *Sender);
      void __fastcall                  ExitItemClick(TObject *Sender);
      void __fastcall                  BuildGrayImages(TObject *Sender);
      void __fastcall                  TestImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y);
      void __fastcall                  TestImageMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
      void __fastcall                  ReferenceImageMouseMove(TObject *Sender, TShiftState Shift, int X, int Y);
      void __fastcall                  ReferenceImageMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
      void __fastcall                  AutomaticButtonClick(TObject *Sender);
      void __fastcall                  StopButtonClick(TObject *Sender);
      void __fastcall                  CopyButtonClick(TObject *Sender);
      void __fastcall                  DownButtonClick(TObject *Sender);
      void __fastcall                  UpButtonClick(TObject *Sender);
      void __fastcall                  SetObject1Click(TObject *Sender);
      void __fastcall                  SetObject2Click(TObject *Sender);
      void __fastcall                  AllignMenuItemClick(TObject *Sender);

   public:
      // gray palette structure
      typedef struct
      {
        TLogPalette                    PalHeader;           // windows palette header
        TPaletteEntry                  dummy[GRAY_PALETTE_ENTRY-1];
      } GRAY_PALETTE;

      // image info
      typedef struct
      {
         int                           SizeX;               // horizontal image size
         int                           SizeY;               // vertical image size
         int                           BackgrondLevel;      // background image level
         int                           BackgroundSigma;     // background image sigma
      } IMAGE_INFO;

      // variables involved in the setup
      ImageCompare::SOLVING            Solving;             // solving parameters
      AnsiString                       RefFolder;           // reference folder
      AnsiString                       TestFolder;          // test folder

      // contructor
      __fastcall                       TForm1(TComponent* Owner);

   private:
      // Load fit file list from test folder
      void                             LoadTestFiles( void );
      // Update allignment information
      void                             UpdateAllignment( ImageCompare *Compare );
      // get image info from current fits
      void                             GetInfoFromFits( IMAGE_INFO *Info );
      // get image info from current fits
      unsigned short *                 GetImageFromFits( IMAGE_INFO *Info );
      // Create bmp from image
      void                             GetBmp( Graphics::TBitmap ** Bmp, unsigned short*Image, IMAGE_INFO *Info, int Min, int Max );
      // Create zoomed bmp from image
      void                             GetZoomBmp( Graphics::TBitmap ** Bmp, unsigned short*Image, IMAGE_INFO *Info, int Min, int Max, int CenterX, int CenterY );
      // compute gray scaled value
      int                              ComputeGrayValue( int Value, int Min, int Max);
      // execute comparation between reference and test image
      void                             ExecCompare();
      // shows difference objects with red circle
      void                             MarkDifferenceObjects();
      // shows alignment objects with green circle
      void                             MarkAlignmentObjects();
      // Load retain data from registry
      void                             LoadRetainData( void );
      // Save retain data to registry
      void                             SaveRetainData( void );
      // list image with differences
      void                             ListImagesWhitDifferences( void );
      // show differences in current image
      void                             ListCurrentDifferences( void );

      char                             CurrentPath[MAXPATH];// program path
      Fits                             *FitsImage;          // fits loading object
      bool                             ReferenceLock;       // reference zoom image lock
      bool                             TestLock;            // test zoom image lock
      int                              ReferenceMinimumValue;// minimum value for reference image scaling
      int                              ReferenceMaximumValue;// maximum value for reference image scaling
      int                              TestMinimumValue;    // minimum value for test image scaling
      int                              TestMaximumValue;    // maximum value for test image scaling
      AnsiString                       TestImageName;       // test image name
      AnsiString                       RefImageName;        // reference image name
      GRAY_PALETTE                     GrayPalette;         // gray palette definition for image
      HPALETTE                         HPalette;            // palette handle
      GRAY_PALETTE                     GrayZoomPalette;     // gray palette definition for zoom image
      HPALETTE                         HPaletteZoom;        // palette handle for zoom image
      Graphics::TBitmap                *InternalBmp;        // image as a bitmap
      Graphics::TBitmap                *InternalZoomBmp;    // zoom image as a bitmap
      ImageCompare::OBJECT_CHAIN       *DifferenceList;     // image compare output list
      ImageCompare::OBJECT_CHAIN       *AlignmentList;      // image compare output list
      bool                             StopRequest;         // stop automatic comparing request
      IMAGE_INFO                       TestInfo;            // test image info
      unsigned short                   *TestImageBuffer;    // test image
      IMAGE_INFO                       ReferenceInfo;       // reference image info
      unsigned short                   *ReferenceImageBuffer; // reference image
      unsigned short                   *RotReferenceImage;  // rotated reference image buffer
      double                           ShiftX;              // shift X test image vs reference image
      double                           ShiftY;              // shift y test image vs reference image
      double                           Rotation;            // rotation test image vs reference image
      int                              ComparedImages;      // compared image counter
      DWORD                            Start;               // init comparing time
      DWORD                            Stop;                // end comparing time
      int                              ImageClick;          // image where is mouse right click ( 0=test, 1=reference)
      int                              XClick;              // x position where is clicked
      int                              YClick;              // y position where is clicked
      ImageCompare::TWO_STARS_ALLIGNMENT   ManualParam;     // parameter for manual allignment
};
//---------------------------------------------------------------------------
extern PACKAGE TForm1 *Form1;
//---------------------------------------------------------------------------
#endif
