/*---------------------------------------------------------------------------

	ScriptEditor.cpp  script editor for observatory management

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// system include
#include <string.h>



// project include
#include "ScriptEditor.h"
#include "KeyWords.h"



/*---------------------------------------------------------------------------

	object constructor

---------------------------------------------------------------------------*/

ScriptEditor::ScriptEditor()
{
   EnableTextChange=true;
}



/*---------------------------------------------------------------------------

	object destroyer

---------------------------------------------------------------------------*/

ScriptEditor::~ScriptEditor(  )
{
}



/*---------------------------------------------------------------------------

	set editor graphic interface

---------------------------------------------------------------------------*/

void ScriptEditor::SetEditor( TWinControl* AOwner, int Top, int Left, int Width, int Height )
{
   HANDLE                              hEditor;             // handle edit component
   DWORD                               Tab=12;

   // create panel cointainig all other components
   Panel = new TPanel(AOwner );
   Panel->Parent = AOwner;
   Panel->Top = Top;
   Panel->Left = Left;
   Panel->Width = Width;
   Panel->Height = Height;

   // create Richedit component
   Editor = new TRichEdit( Panel );
   Editor->Parent = Panel;
   Editor->Top = 3;
   Editor->Left = 20;
   Editor->Width = Width-40;
   Editor->Height = Height-26;
   Editor->Font->Name = "Courier New";
   Editor->Font->Size = 10;
   Editor->HideSelection = false;
   Editor->WantTabs = true;
   Editor->WordWrap = false;
   hEditor = Editor->Handle;                                // get handle
   SendMessage( hEditor, EM_SETTABSTOPS, 1, (LPARAM)&Tab ); // get line start in the text

   // create scroll bar
   ScrollBar = new TScrollBar( Panel );
   ScrollBar->Parent = Panel;
   ScrollBar->Kind = sbVertical;
   ScrollBar->Top = 4;
   ScrollBar->Left = Width-20;
   ScrollBar->Width = 16;
   ScrollBar->Height = Height-28;
   ScrollBar->Enabled = false;
   ScrollBar->OnChange = ScrollBarChange;

   // create label
   Label = new TLabel( Panel );
   Label->Parent = Panel;
   Label->Top = Height-20;
   Label->Left = 20;
   Label->Caption = "pippo";

   // compute lines spaces in the editor
   EditorLine = Height/16;

   // assign function managing text changing
   Editor->OnChange = TextChange;
   Editor->OnKeyDown = KeyDown;
   Editor->OnKeyUp = KeyUp;
   Editor->OnMouseDown = MouseDown;
   Editor->OnMouseMove = MouseMove;
}



/*---------------------------------------------------------------------------

	set text in the editor

---------------------------------------------------------------------------*/

void ScriptEditor::SetText( char *Text )
{
   EnableTextChange = false;                                // disable text change detect
   Editor->SetTextBuf(Text);                                // assign text to component
   PaintAll(Text);                                          // paint all token
   Editor->Modified = false;                                // text in the component is alligned with client
   Editor->SelStart = 0;                                    // cursor in the start position
   Editor->SelLength = 0;                                   // no selection
   EnableTextChange = true;                                 // enable text change detect
   Editor->Update();

   ArrangeScrollBar();                                      // arrange scroll bar with editor status
   UpdatePosition();                                        // update caret position
}



/*---------------------------------------------------------------------------

	get text lenght in the buffer

---------------------------------------------------------------------------*/

int ScriptEditor::GetTextLenght(void)
{
   return Editor->GetTextLen()+1;
}



/*---------------------------------------------------------------------------

	Get text from the editor

---------------------------------------------------------------------------*/

void ScriptEditor::GetText( char *Text, int Size )
{
   Editor->GetTextBuf(Text, Size);                          // get text from buffer
   Editor->Modified = false;                                // text in the component is alligned with client
}



/*---------------------------------------------------------------------------

	select line

---------------------------------------------------------------------------*/

void ScriptEditor::SelectLine( int Line )
{
   HANDLE                              hEditor;             // handle edit component
   int                                 LineStart;           // line start in the text
   int                                 LineStop;            // line stop in the text

   hEditor = Editor->Handle;                                // get handle
   LineStart = SendMessage( hEditor, EM_LINEINDEX, Line-1, 0 );  // get line start in the text
   if ( LineStart == -1 )
      return;
   LineStop = SendMessage( hEditor, EM_LINEINDEX, Line, 0 ); // get next line start in the text
   if ( LineStop == -1 )
      LineStop = Editor->GetTextLen();

   if ( LineStart > LineStop )
      LineStart = LineStop;

   Editor->SelStart = LineStart;                            // assign selection
   Editor->SelLength = LineStop-LineStart;
   SendMessage( hEditor, EM_SCROLLCARET, 0, 0 );            // make line visible
}



/*---------------------------------------------------------------------------

	select text

---------------------------------------------------------------------------*/

void ScriptEditor::SelectText( int Line, int Column, int Lenght )
{
   HANDLE                              hEditor;             // handle edit component
   int                                 LineStart;           // line start in the text
   int                                 LineStop;            // line stop in the text
   int                                 SelStart;            // selection start
   int                                 SelStop;             // selection stop

   hEditor = Editor->Handle;                                // get handle
   LineStart = SendMessage( hEditor, EM_LINEINDEX, Line-1, 0 );  // get line start in the text
   if ( LineStart == -1 )
      return;
   LineStop = SendMessage( hEditor, EM_LINEINDEX, Line, 0 ); // get next line start in the text
   if ( LineStop == -1 )
      LineStop = Editor->GetTextLen();

   SelStart = LineStart + Column;                           // compute selection start position
   if ( SelStart > LineStop )                               // clamp selection start position
      SelStart = LineStop;
   SelStop = LineStart + Column +Lenght;                    // compute selection stop position
   if ( SelStop > LineStop )                                // clamp selection stop position
      SelStop = LineStop;

   Editor->SelStart = SelStart;                             // assign selection
   Editor->SelLength = SelStop-SelStart;
}



/*---------------------------------------------------------------------------

	detect text changing

---------------------------------------------------------------------------*/

void __fastcall ScriptEditor::TextChange(TObject *Sender)
{
   AnsiString                          Str;                 // ansistring with current line
   char                                Str2[80];            // C string with current line
   int                                 Line;                // current line number
   int                                 CaretPos;            // caret position in whole text
   int                                 StartLinePos;        // start line position in whole test
   AnsiString                          Message;             // line/coln message

   // only if the function in enabled
   if ( EnableTextChange == false )
      return;

   // get current line
   Line = Editor->CaretPos.y;

   // manage line into the text
   if ( Line < Editor->Lines->Count )
   {
      Str = Editor->Lines->Strings[Line];                   // get line text
      strcpy( Str2, Str.c_str() );                          // converts text in C
      CaretPos = Editor->SelStart;                          // get caret position in the line (column)
      StartLinePos = CaretPos - Editor->CaretPos.x;         // compute start line position in the whole text
      PaintOnCaret( Str2, StartLinePos, Editor->CaretPos.x);// paint line (only token before and after caret)
      Editor->SelStart = CaretPos;                          // set caret in the corret position (painting destroy position)
   }
   else
   // manage new line at the end of the text
   {
      Editor->SelAttributes->Color = clBlack;               // normal color
      Editor->SelAttributes->Style = TFontStyles();         // normal style
   }
   UpdatePosition();
}



/*---------------------------------------------------------------------------

	detect key down

---------------------------------------------------------------------------*/

void __fastcall ScriptEditor::KeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
   ArrangeScrollBar();                                      // arrange scroll bar with editor status
}



/*---------------------------------------------------------------------------

	detect key up

---------------------------------------------------------------------------*/

void __fastcall ScriptEditor::KeyUp(TObject *Sender, WORD &Key, TShiftState Shift)
{
   UpdatePosition();                                        // update caret position
}



/*---------------------------------------------------------------------------

	detect mouse down

---------------------------------------------------------------------------*/

void __fastcall ScriptEditor::MouseDown(TObject* Sender, TMouseButton Button, TShiftState Shift, int X, int Y)
{
   UpdatePosition();                                        // update caret position
}



/*---------------------------------------------------------------------------

	detect mouse move

---------------------------------------------------------------------------*/

void __fastcall ScriptEditor::MouseMove(TObject* Sender, TShiftState Shift, int X, int Y)
{
   if ( Shift.Contains( ssLeft	 ) )
      UpdatePosition();                                      // update caret position
}



/*---------------------------------------------------------------------------

	detect scrollbar change

---------------------------------------------------------------------------*/

void __fastcall ScriptEditor::ScrollBarChange(TObject *Sender)
{
   HANDLE                              hEditor;             // handle edit component
   int                                 FirstVisibleLine;    // line start in the text
   int                                 Step;


   // compute diference between current first visible line and position required from scrollbar
   hEditor = Editor->Handle;                                // get handle
   FirstVisibleLine = SendMessage( hEditor, EM_GETFIRSTVISIBLELINE, 0, 0 );
   Step = ScrollBar->Position - FirstVisibleLine;

   // scroll line by line
   if ( Step > 0 )
   {
      while (Step > 0 )
      {
         SendMessage( hEditor, EM_SCROLL, SB_LINEDOWN, 0 );
         Step--;
      }
   }
   else
   {
      while (Step < 0 )
      {
         SendMessage( hEditor, EM_SCROLL, SB_LINEUP, 0 );
         Step++;
      }
   }
}



/*---------------------------------------------------------------------------

	paint all tokens in the text

---------------------------------------------------------------------------*/

void ScriptEditor::PaintAll( char *Buffer )
{
   int                                 Scan;                // scanning position
   int                                 TokenPos;            // token position in the line
   int                                 TokenLenght;         // token lenght
   int                                 TokenEnd;            // token end in the line (first position out of the token)
   bool                                IsToken;             // found a token the line
   char                                Token[50];           // token text

   Scan=0;                                                  // init scanning position
   do                                                       // scann all text
   {
      IsToken = GetToken( Buffer, Scan, &TokenPos, Token ); // searching for a token
      if ( IsToken == false )                               // when token are not longer present stop scanning
         break;

      // compute token position
      TokenLenght = strlen(Token);
      TokenEnd = TokenPos+TokenLenght;

      PaintToken( Token, TokenPos, TokenLenght );           // paint token

      Scan = TokenEnd;                                      // update scanning position for next token
   } while ( IsToken == true );                             // scan until token are present
}



/*---------------------------------------------------------------------------

	paint token near caret position

---------------------------------------------------------------------------*/

void ScriptEditor::PaintOnCaret( char *Line, int Start, int Caret )
{
   int                                 LineLen;             // lenght of the input line
   int                                 Scan;                // scanning position
   int                                 TokenPos;            // token position in the line
   int                                 TokenLenght;         // token lenght
   int                                 TokenEnd;            // token end in the line (first position out of the token)
   bool                                IsToken;             // found a token the line
   char                                Token[50];           // token test

   // if the line is empty not action
   LineLen = strlen( Line );
   if ( LineLen == 0 )
      return;

   Scan=0;                                                  // scanning starts at the init of the line
   do
   {
      IsToken = GetToken( Line, Scan, &TokenPos, Token );   // searching for a nre token
      if ( IsToken == false )                               // when token are not longer present stop loop
         break;

      // compute token position
      TokenLenght = strlen(Token);
      TokenEnd = TokenPos+TokenLenght;

      // is it the last token before caret?
      if ( TokenPos < Caret && TokenEnd >= (Caret-1) )
         PaintToken( Token, Start+TokenPos, TokenLenght );  // repaint token

      // is it the fisrt token after caret?
      if ( TokenPos == Caret )
         PaintToken( Token, Start+TokenPos, TokenLenght );  // repaint token

      Scan = TokenEnd;                                      // update scanning position for next token
   } while ( IsToken == true );                             // scan until token are present
}



/*---------------------------------------------------------------------------

	get token from a string

---------------------------------------------------------------------------*/

bool ScriptEditor::GetToken( char *Str, int Start, int *TokenStart, char *Token )
{
   char                                *Scan;               // scanning string pointer
   char                                *End;                // end string pointer

   Scan = Str+Start;                                        // compute scanning start position
   End = Str+strlen(Str);                                   // compute scanning end position
   *TokenStart = Start;                                     // iniztaliza start token position

   // bypass all chars spaces and less than space
   while ( *Scan <= ' ' && Scan != End )
   {
      Scan++;                                               // advance scan position
      (*TokenStart)++;                                      // advance token position
   }

   // when scan reach end tokening is finished
   if ( Scan == End )
      return false;

   // copy string in the token
   while ( *Scan > ' ' && Scan != End )
      *Token++ = *Scan++;

   *Token = 0;                                              // token terminator
   return true;
}



/*---------------------------------------------------------------------------

	paint token with correct color

---------------------------------------------------------------------------*/

void ScriptEditor::PaintToken( char *Token, int TokenStart, int TokenLen )
{
   // select token in the text on richedit
   Editor->SelStart = TokenStart;
   Editor->SelLength = TokenLen;

   // paint for command
   if ( IsCommand(Token) == true )
   {
      Editor->SelAttributes->Color = TColor(0xC00000);
      Editor->SelAttributes->Style = TFontStyles()<< fsBold;
      return;
   }

   // paint for parameter
   if ( IsParameter(Token) == true )
   {
      Editor->SelAttributes->Color = clTeal	;
      Editor->SelAttributes->Style = TFontStyles()<< fsBold;
      return;
   }

   // paint for number
   if ( IsNumber(Token) == true )
   {
      Editor->SelAttributes->Color = clBlack	;
      Editor->SelAttributes->Style = TFontStyles()<< fsBold;
      return;
   }

   // paint for coordinate
   if ( IsCoo(Token) == true )
   {
      Editor->SelAttributes->Color = clBlack;
      Editor->SelAttributes->Style = TFontStyles()<< fsBold;
      return;
   }

   // paint for time
   if ( IsTime(Token) == true )
   {
      Editor->SelAttributes->Color = clBlack;
      Editor->SelAttributes->Style = TFontStyles()<< fsBold;
      return;
   }

   // paint for normal text
   Editor->SelAttributes->Color = clGray;
   Editor->SelAttributes->Style = TFontStyles();
}



/*---------------------------------------------------------------------------

	check string to verify command

---------------------------------------------------------------------------*/

bool ScriptEditor::IsCommand( char *Str )
{
   char                                *Scan;               // scan string
   char                                **TabKeyWords;       // pointer to keyword table
   char                                KeyWord[40];         // current keyword

   // trasnsform string in uppercase
   Scan = Str;                                              // first string character
   while ( *Scan != 0 )                                     // until character are present
      *Scan++ = (char)toupper(*Scan);                       // transform in upper case

   // scan command table
   TabKeyWords = ScriptEditor_Commands;                     // pointer on the first string
   while ( **TabKeyWords != 0 )                             // until a string is present in the table
   {
      strcpy( KeyWord, *TabKeyWords) ;                      // copy string (I don't want change case in the table)
      Scan = KeyWord;                                       // first string character
      while ( *Scan != 0 )                                  // until character are present
         *Scan++ = (char)toupper(*Scan);                    // transform in upper case
      if ( strcmp(Str, KeyWord) == 0 )                      // is this keyword?
         return true;                                       // found
      TabKeyWords++;                                        // goto on next keyword in the table
   }

   return false;                                            // not found
}



/*---------------------------------------------------------------------------

	check string to verify parameter

---------------------------------------------------------------------------*/

bool ScriptEditor::IsParameter( char *Str )
{
   char                                *Scan;               // scan string
   char                                **TabKeyWords;       // pointer to keyword table
   char                                KeyWord[40];         // current keyword

   // trasnsform string in uppercase
   Scan = Str;                                              // first string character
   while ( *Scan != 0 )                                     // until character are present
      *Scan++ = (char)toupper(*Scan);                       // transform in upper case

   // scan command table
   TabKeyWords = ScriptEditor_Parameters;                   // pointer on the first string
   while ( **TabKeyWords != 0 )                             // until a string is present in the table
   {
      strcpy( KeyWord, *TabKeyWords) ;                      // copy string (I don't want change case in the table)
      Scan = KeyWord;                                       // first string character
      while ( *Scan != 0 )                                  // until character are present
         *Scan++ = (char)toupper(*Scan);                    // transform in upper case
      if ( strcmp(Str, KeyWord) == 0 )                      // is this keyword?
         return true;                                       // found
      TabKeyWords++;                                        // goto on next keyword in the table
   }

   return false;                                            // not found
}



/*---------------------------------------------------------------------------

	check string to verify number

---------------------------------------------------------------------------*/

bool ScriptEditor::IsNumber( char *Str )
{
   while ( *Str != 0 )                                      // until character are present
   {
      if ( *Str < '0' || *Str > '9' )                       // check if current character is a number
         return false;
      Str++;                                                // goto next character
   }

   return true;                                             // all number
}



/*---------------------------------------------------------------------------

	check string to verify coordinate

---------------------------------------------------------------------------*/

bool ScriptEditor::IsCoo( char *Str )
{
   char                                Str2[5];             // string of single coordinate element
   int                                 Index;               // copying index
   int                                 Value;               // value

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 359 || Index == 0 )            // test for degrees
      return false;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for minutes
      return false;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for seconds
      return false;

   if ( *Str != '+' && *Str != '-' )                        // test for separator
      return false;

   Str++;
   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 89 || Index == 0 )             // test for degree
      return false;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for minutes
      return false;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for seconds
      return false;

   if ( *Str++ != 0 )                                       // test for terminator
      return false;

   return true;                                             // correct equatorial coordinate
}



/*---------------------------------------------------------------------------

	check string to verify time

---------------------------------------------------------------------------*/

bool ScriptEditor::IsTime( char *Str )
{
   char                                Str2[5];             // string of single coordinate element
   int                                 Index;               // copying index
   int                                 Value;               // value

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 23 || Index == 0 )             // test for hours
      return false;

   if ( *Str++ != '.' )                                     // test for separator
      return false;

   Index=0;
   while ( *Str >= '0' && *Str <= '9' && Index < 3)         // copy until number
      Str2[Index++] = *Str++;
   Str2[Index] = 0;
   Value = atoi( Str2 );                                    // convert in number
   if ( Value < 0 || Value > 59 || Index == 0 )             // test for minutes
      return false;

   if ( *Str++ != 0 )                                       // test for terminator
      return false;

   return true;                                             // correct equatorial coordinate
}



/*---------------------------------------------------------------------------

	check text modified

---------------------------------------------------------------------------*/

bool ScriptEditor::IsModified( void )
{
   return Editor->Modified;
}



/*---------------------------------------------------------------------------

	check text modified

---------------------------------------------------------------------------*/

void ScriptEditor::ArrangeScrollBar( void )
{
   int                                 TotalLen;            // text lenght
   HANDLE                              hEditor;             // handle edit component
   int                                 LastLine;            // line start in the text
   int                                 FirstVisibleLine;    // first line visible in the editor

   TotalLen = Editor->GetTextLen();                         // get text lennht

   hEditor = Editor->Handle;                                // get handle
   LastLine = SendMessage( hEditor, EM_LINEFROMCHAR, TotalLen, 0 );  // get line start in the text
   FirstVisibleLine = SendMessage( hEditor, EM_GETFIRSTVISIBLELINE, 0, 0 );

   // scroll bar enabled when lines more than editor
   if ( LastLine > EditorLine )
   {
      ScrollBar->Enabled = true;                            // enable
      ScrollBar->Max = LastLine - EditorLine + 1;           // scroll the difference
      ScrollBar->Position = FirstVisibleLine;               // scoll position is fisrt visibile line
   }
   else
      ScrollBar->Enabled = false;
}



/*---------------------------------------------------------------------------

	update caret position

---------------------------------------------------------------------------*/

void ScriptEditor::UpdatePosition( void )
{
   int                                 Line;                // current line number
   int                                 Column;              // current column number
   AnsiString                          Message;             // line/coln message

   // get current line and column number
   Line = Editor->CaretPos.y + 1;
   Column = Editor->CaretPos.x + 1;

   // update line/column message
   Message = "Riga:" + AnsiString(Line) + " Colonna:" + AnsiString(Column);
   Label->Caption = Message;
}



/*---------------------------------------------------------------------------

	set status active and readonly

---------------------------------------------------------------------------*/

void ScriptEditor::SetStatus( bool Active, bool ReadOnly )
{
   if ( Active == true )
      Panel->Color = clLtGray;
   else
      Panel->Color = clBtnFace;
   Editor->ReadOnly = ReadOnly;
}



/*---------------------------------------------------------------------------

	Commands list

---------------------------------------------------------------------------*/

char  *ScriptEditor_Commands[]=
{
   KEYWORD_GOTO,
   KEYWORD_TRACKING,
   KEYWORD_DOME,
   KEYWORD_SSHUTTER,
   KEYWORD_BSHUTTER,
   KEYWORD_DECONDENCE,
   KWYWORD_POWER,
   KEYWORD_SHOT,
   KEYWORD_FLAT,
   KEYWORD_DARK,
   KEYWORD_SKYCHECK,
   KEYWORD_WAIT,
   KEYWORD_REPEAT,
   KEYWORD_ENDREPEAT,
   KEYWORD_VELOCITY,
   KEYWORD_ALLIGN,
   KEYWORD_SEARHSN,
   ""
};



/*---------------------------------------------------------------------------

	Patameters list

---------------------------------------------------------------------------*/

char  *ScriptEditor_Parameters[]=
{
   PARAMETER_PLANET,
   PARAMETER_GRB,
   PARAMETER_EQUATORIAL,
   PARAMETER_EQU2000,
   PARAMETER_ALTAZIMUT,
   PARAMETER_MPLANET,
   PARAMETER_NEBULA,
   PARAMETER_CLUSTER,
   PARAMETER_GALAXY,
   PARAMETER_STAR,
   PARAMETER_HOME,
   PARAMETER_ON,
   PARAMETER_OFF,
   PARAMETER_AUTO,
   PARAMETER_ZERO,
   PARAMETER_OPEN,
   PARAMETER_CLOSE,
   PARAMETER_TIMES,
   PARAMETER_UPTO,
   ""
};




