/******************************************************************************

	ScriptEditor.h | Header of script editor for observatory management

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : October 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef ScriptEditorH
#define ScriptEditorH



// system include
#include <ComCtrls.hpp>



extern char                            *ScriptEditor_Commands[];
extern char                            *ScriptEditor_Parameters[];




/******************************************************************************

	Script editor class

******************************************************************************/

class ScriptEditor
{
   public:
      // object constructor
                                       ScriptEditor( );
      // object destroyer
      virtual                          ~ScriptEditor();
      // set editor graphic interface
      void                             SetEditor( TWinControl* AOwner, int Top, int Left, int Width, int Height );
      // set text in the editor
      void                             SetText( char *Text );
      // get text lenght in the buffer
      int                              GetTextLenght(void);
      // fet text from the editor
      void                             GetText( char *Text, int Size );
      // select line
      void                             SelectLine( int Line );
      // select text
      void                             SelectText( int Line, int Column, int Lenght );
      // check text modified
      bool                             IsModified( void );
      // set status active and readonly
      void                             SetStatus( bool Active, bool Readonly );

   protected:
      // detect text changing
      void __fastcall                  TextChange(TObject *Sender);
      // detect key down
      void __fastcall                  KeyDown(TObject *Sender, WORD &Key, TShiftState Shift);
      // detect key up
      void __fastcall                  KeyUp(TObject *Sender, WORD &Key, TShiftState Shift);
      // detect mouse down
      void __fastcall                  MouseDown(TObject* Sender, TMouseButton Button, TShiftState Shift, int X, int Y);
      // detect mouse move
      void __fastcall                  MouseMove(TObject* Sender, TShiftState Shift, int X, int Y);

      // detect scrollbar change
      void __fastcall                  ScrollBarChange(TObject *Sender);
      // paint all tokens
      void                             PaintAll( char *Buffer);
      // paint token near caret position
      void                             PaintOnCaret( char *Line, int Start, int Caret );
      // get token from a string
      bool                             GetToken( char *Str, int Start, int *TokenStart, char *Token );
      // paint token with correct color
      void                             PaintToken( char *Token, int TokenStart, int TokenLen );
      // check string to verify commands
      bool                             IsCommand( char *Str );
      // check string to verify parameters
      bool                             IsParameter( char *Str );
      // check string to verify number
      bool                             IsNumber( char *Str );
      // check string to verify coordinate
      bool                             IsCoo( char *Str );
      // check string to verify coordinate
      bool                             IsTime( char *Str );
      // arrange scrollbar
      void                             ArrangeScrollBar( void );
      // update caret position
      void                             UpdatePosition();

      bool                             EnableTextChange;    // enable text change detect
      TPanel                           *Panel;              // panel component parent of all others
      TRichEdit                        *Editor;             // text editor component
      TScrollBar                       *ScrollBar;          // scrollbar for editor scrolling
      TLabel                           *Label;              // label for line column view
      int                              EditorLine;          // visible lines in the editor
};
#endif


