/*---------------------------------------------------------------------------

	ScriptErrorList.h  Header for list of script error

   Author : Luca Strabla (Bassano Bresciano Observatory)

  	Date : October 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

---------------------------------------------------------------------------*/



// antirepeat
#ifndef ScriptErrorListH
#define ScriptErrorListH


// system include
#include <math.h>



class ScriptErrorList
{
   public:

      typedef struct LIST_ELEMENT_Tag
      {
         int                           Script;              // script code
         int                           Error;               // error code
         int                           Line;                // line where is error
         int                           Column;              // column where is error
         LIST_ELEMENT_Tag              *Next;               // pointer to next element
      } LIST_ELEMENT;

      // object constructor
                                       ScriptErrorList();
      // object destroyer
      virtual                          ~ScriptErrorList();
      // Add an object to list
      void                             Add( int Script, int Error, int Line, int Column);
      // Set list pointer to the first element
      void                             First(void) ;
      // get next object in the list
      LIST_ELEMENT *                   Next( void );
      // get number of objects in the list
      int                              GetCount( void );

   protected:
      int                              ObjectCount;         // number of error in the list
      LIST_ELEMENT                     *FirstElement;       // first error in the list
      LIST_ELEMENT                     *LastElement;        // last error in the list
      LIST_ELEMENT                     *ScanElement;        // scanning element
      LIST_ELEMENT                     *CurrentElement;     // last element yeld
      LIST_ELEMENT                     *PrevElement;        // previous yeld element
};

#endif
