/******************************************************************************

	ShotSequence.h | Header of shot sequence management

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : June 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef ShotSequenceH
#define ShotSequenceH



// system include
//#include <Classes.hpp>


// project include
#include "Observer.h"
#include "TentaclesMgr.h"




class ShotSequence : public TThread, public Observer
{
   public:

      // state of sequence
      typedef enum
      {
         SEQ_STATE_IDLE,                                    // idle, wait to start
         SEQ_STATE_START,                                   // start state
         SEQ_STATE_CHECK_VENT_ON,                           // check for ventilation om
         SEQ_STATE_CHECK_VENT_OFF,                          // check for ventilation off
         SEQ_STATE_CHECK_VENT_PAUSE,                        // check for ventilation pause
         SEQ_STATE_SHOT,                                    // shot a photo
         SEQ_STATE_WAIT,                                    // waiting for a photo
         SEQ_STATE_END,                                     // sequence ended
      } SEQ_STATE;

      // Object constructor
      __fastcall                       ShotSequence(TMemo *ServiceMemo);
      // Object destroyer
      virtual _fastcall                ~ShotSequence();
      // set subject
      int                              SetSubject( TentaclesMgr *ParPuntSubject );
      // start sequence
      void                             StartSequence(HANDLE ParFormHandle, int ParExposure, int ParNumShot, char *ParObject, int VentilazInterval, int VentilazTime, int VentilazPause, bool PECheck);
      // get sequence status
      void                             GetStatus( bool *End, int *ParExposure, int *ParProgShot, bool *ParCloudy );

   private:
      // notify change in subject
      virtual void                     Update( int ANotifyCode );

   protected:
      TMemo                            *ServiceMemo;        // memo for service output information
      TentaclesMgr                     *PuntSubject;        // pointer to subject
      HANDLE                           FormHandle;          // handle of the parent form
      int                              Exposure;            // programmed exposure time (millisec)
      int                              Exposing;            // current exposing time
      int                              NumShot;             // programmed number of shot
      int                              VentilazInterval;    // interval between every ventilation (minutes)
      int                              VentilazTime;        // ventilation time (seconds)
      int                              VentilazPause;       // ventilazion pause before start new shot (seconds)
      int                              ProgShot;            // current number of shot
      char                             Object[81];          // object shoted
      SEQ_STATE                        State;               // actual sequence state
      bool                             Control;             // control taken
      bool                             Cloudy;              // cloudy sky
      DWORD                            StartInterval;       // start interval for ventilation
      DWORD                            PauseTime;           // start pause time for ventilation
      bool                             PECheck;             // Periodic error check

      // Threaded function
      void __fastcall                  Execute(void);
};
//---------------------------------------------------------------------------
#endif
