/******************************************************************************

	Subject.cpp Class implementing a generic subject of Observer pattern

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : April 2009

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 
   

*****************************************************************************/



// project include
#include "Subject.h"



/*---------------------------------------------------------------------------

   constructor

---------------------------------------------------------------------------*/

Subject::Subject( )
{
}



/*---------------------------------------------------------------------------

   Destroyer

---------------------------------------------------------------------------*/

Subject::~Subject( )
{
   Notify( 0, -1 );
   while ( ListaObj.size() != 0 )
      ListaObj.pop_back();
}



/*---------------------------------------------------------------------------

   Add an observer to the list

---------------------------------------------------------------------------*/

int Subject::Attach(
   Observer                            *Obj                 // observer to add
)
{
   vector<Observer*>::iterator         i;

   // is it already present
   i = find( ListaObj.begin(), ListaObj.end(), Obj );
   if ( i != ListaObj.end() )
      return -1;

   ListaObj.push_back( Obj );
   return 0;
}



/******************************************************************************

   remove an observer from the list

******************************************************************************/

int Subject::Detach(
   Observer                            *Obj                 // observer to remove
)
{
   vector<Observer*>::iterator         i;

   // it should be present
   i = find( ListaObj.begin(), ListaObj.end(), Obj );
   if ( i == ListaObj.end() )
      return -1;

   ListaObj.erase(i);
   return 0;
}



/******************************************************************************

   event notify to all observer gente della notifica

******************************************************************************/

int Subject::Notify(
   Observer                            *Source,             // souce observer
   int                                 NotifyCode           // Notify code to give all other observer
)
{
   int                                 i,n;
   Observer                            *p;

   // browse the list
   n = ListaObj.size();
   for ( i=0; i<n; i++ )
   {
      p = ListaObj[i];
      p->Update( NotifyCode );            // notify object function
   }
   return 0;
}

