/******************************************************************************

         Tokenizer.cpp

			Autore :		Luca
			Data :		Aprile 2002

			Classe di tokenizzazione da buffer

******************************************************************************/



// include miei
#include "Tokenizer.h"



/******************************************************************************

	costruttore dell'oggetto tokenizzatore

******************************************************************************/

Tokenizer::Tokenizer(
   )
{
   // azzero la tabella handle per separatori
  	TabSep = NULL;

   // inizializzo puntatore buffer
   PuntBuffer 	= NULL;
}



/******************************************************************************

   distruttore dell'oggetto

******************************************************************************/

Tokenizer::~Tokenizer( )
{
   // libero la memoria per le tabelle di seperatori allocate
  	if ( TabSep != NULL )
   {
   	GlobalFree( TabSep );
      TabSep == NULL;
   }
}



/******************************************************************************

   Imposta una tebella di separatori

   Chiede all'oggetto di memorizzare all'interno una nuova tabella di separatori.
   Se l'oggetto  disponibile alloca lo spazio necessario ed effettua la copia e
   restituisce l'handle della tabella. In questo modo  possibile che lo stesso
   oggetto disponga in momenti diversi di separatori diversi con i quali lavorare.
   L'indicazione di quale tabella utilizzare deve essere fatta utilizzando la
   funzione SelectSeparators.
   Se la funzione SetSeparators non  in grado di eseguire l'operazione di
   immagazzinamento della tabella restituisce TKN_INVALID_HANDLE

   La tabella dei separatori viene indicata come una unica stringa che contiene
   una dopo l'altra i separatori, il termine della tabella deve essere indicato
   con una stringa nulla. Nel caso un separatore sia parte di secondo separatore
   (ad esempio > e >= ) il separatore di lunghezza inferiore deve essere posto
   necessariamente dopo nella tabella

******************************************************************************/

int  Tokenizer::SetSeparators(
	const char*			BufSep		            // Buffer con la tabella separatori
   )
{
   int		l;												// lunghezza buffer da allocare
   int		l2;											// lunghezza singola stringa
   const char 	*p;											// puntatore di comodo per seguire buffer

   // calcolo lunghezza buffer
   l = 0;
   p = BufSep;
   do
   {
      l2 = strlen( p );
      l += ( l2 + 1 );
      p += ( l2 + 1 );
   } while ( l2 > 0 );

   // alloco area e copio
   TabSep = (char *) GlobalAlloc( GMEM_FIXED, l );
   memcpy( TabSep, BufSep, l );
   return ERR_NONE;
}



/******************************************************************************

	Imposto buffer su oggetto tokenizzatore

******************************************************************************/

void Tokenizer::SetBuffer(
	const char*				               Buf,				      // puntatore al buffer conenente il testo da tokenizzare
   int 				                     LenBuffer				// dimensione del buffer da tokenizzare
)
{
   // inizializzo puntatori
   PuntBuffer 	= PuntScan = Buf;
	DimBuffer  	= LenBuffer;
   PuntEnd 		= (char*)(PuntScan)+LenBuffer;

   Start();
}



/******************************************************************************

   Imposta scanerizzazione da inizio buffer

******************************************************************************/

int Tokenizer::Start(
)
{
   if ( PuntBuffer == NULL )
      return ERR_DATA;

   // inizializzo dati di scansione
   Riga = 1;
   Colonna = 1;
   PuntScan = PuntBuffer;
   return ERR_NONE;
}



/******************************************************************************

   Legge il prossimo token in formato carattere

******************************************************************************/

int Tokenizer::Read(
   char        *Token,
   int         *Line,
   int         *Column
)
{
   int         n;                               // contatore di comodo
	char*			pIni;								      // puntatore di inizio scansione corrente su buffer
   char*			pCur;								      // puntatore di scansione corrente su buffer
   char*			pSep;								      // puntatore per scansione tabella separatori
   int			LenToken;						      // lunghezza del token
   int         RigaToken;                       // riga alla quale  iniziato il token
   int         ColonnaToken;                    // colonna alla quale  iniziato il token
   int         DimResto;                        // dimensione resto del buffer

	// preparo puntatori per scansione in avanti
	pIni = (char*)PuntScan;                      // la scansione inizia dall'ultimo puntatore memorizzato
   pCur = pIni;                                 // per ora la posizione di scansione  uguale all'inizio
   LenToken = 0;                                // e la lunghezza del token trovato e nulla
   RigaToken = -1;

   // gestisco richiesta dopo esaurimento buffer
   if ( pCur >= PuntEnd )
   {
      *Token = 0;
      if ( Line != NULL )
         *Line = Riga;
      if ( Column != NULL )
         *Column = Colonna;
      return ERR_END;
   }

   do
   {
      // gestione del line feed
      if ( *pCur == '\n' )
      {
         Colonna = 1;                           // torno a capo
         Riga++;                                // cambio riga
      }

      // gestione degli spazi bianchi
      if ( *pCur <= ' ' )
      {
         if ( LenToken > 0 )                    // se ho gi trovato dei caratteri
            break;                              // esco perche il token sono i caratteri sin qui trovati
         else
         {                                      // mi trovo nel caso di spazi iniziali
            if ( *pCur != '\n' )
               Colonna++;                       // sono su una colonna successiva
            pIni++;                             // incremento puntatore di inizio analisi
            pCur++;                             // incremento puntatore corrente
            continue;
         }
      }

      // controllo se dalla posizione corrente trovo un separatore registrato in tabella
      if ( TabSep != NULL )
      {
         bool        Trovato=false;

         pSep = TabSep;             // punto al primo separatore in tabella
         while ( *pSep != 0 )                   // ed esiste un separatore in tabella ...
         {
            n = strlen( pSep );                 // lunghezza del separatore che sto per confrontare
            DimResto = (char*)PuntEnd - pCur;
            if ( n <= DimResto &&         // il separatore  questo ?
                 strncmp( pSep, pCur, n ) == 0 )
            {
               if ( LenToken == 0 )
               {
                  LenToken = n;
                  RigaToken = Riga;
                  ColonnaToken = Colonna;
                  Colonna += n;
               }
               Trovato = true;
               break;
            }
            else
               pSep += n+1;                     // NO!: non era il separatore giusto, passo al successivo
         }
         if ( Trovato == true )
            break;
      }

      // passo ad analizzare il carattere successivo
      if ( RigaToken == -1 )
      {
         RigaToken = Riga;
         ColonnaToken = Colonna;
      }
      pCur++;
      LenToken++;
      Colonna++;

   } while ( pCur != PuntEnd );                 // il buffer non  teminato

   // se puntatore  sulla fine non ho pi niente da dire
   if ( LenToken == 0 )
   {
	   *Token = 0;
      if ( Line != NULL )
         *Line = Riga;
      if ( Column != NULL )
         *Column = Colonna;
      return ERR_END;
   }

   // aggiorno il puntatore per prossima volta
   PuntScan = pIni + LenToken;

   // il token va da inizio scansione per n caratteri
   if ( LenToken > MAXLN )
      LenToken = MAXLN;
   while ( LenToken != 0 )
   {
      if ( *pIni >= ' ' )
         *Token++ = *pIni;
      pIni++;
      LenToken--;
   }
   *Token = 0;

   if ( Line != NULL )
      *Line = RigaToken;
   if ( Column != NULL )
      *Column = ColonnaToken;

   return ERR_NONE;
}



/******************************************************************************

   Get current scan pointer

******************************************************************************/

char * Tokenizer::GetScanPointer( void )
{
   return (char*)PuntScan;
}

