/******************************************************************************

	XYGraph.cpp | XYGraphics manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : December 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// project include
#include "XYGraph.h"
#pragma package(smart_init)



/****************************************************************************

   Object constructor

*****************************************************************************/

XYGraph::XYGraph()
{
   // reset internal param
   FirstElement = 0;
}



/****************************************************************************

   Object destroyer

*****************************************************************************/

XYGraph::~XYGraph()
{
}



/****************************************************************************

   Set X axel

*****************************************************************************/

void XYGraph::SetX( int Start, int Stop, int Grid )
{
   // store parameter
   StartX = Start;
   StopX = Stop;
   GridX = Grid;
}



/****************************************************************************

   Set Y axel

*****************************************************************************/

void XYGraph::SetY( int Start, int Stop, int Grid )
{
   // store parameter
   StartY = Start;
   StopY = Stop;
   GridY = Grid;
}



/****************************************************************************

   set data color

*****************************************************************************/

void XYGraph::SetColor( TColor AColor )
{
   // store parameter
   AxisColor = AColor;
}



/****************************************************************************

   clear graph

*****************************************************************************/

void XYGraph::Clear( void )
{
   XYGRAPH_LISTELE                     *ScanElement;        // scanning element
   XYGRAPH_LISTELE                     *NextElement;        // next element

   // start from the list begining
   ScanElement = FirstElement;

   // free all elements
   while ( ScanElement != 0 )                               // until element are present
   {
      NextElement = ScanElement->Next;                      // save next element
      free ( ScanElement );                                 // free actual element
      ScanElement = NextElement;                            // focus next element
   }

   // list empty
   FirstElement = 0;
}



/****************************************************************************

   add a point to graph

*****************************************************************************/

void XYGraph::Add( XYGRAPH_POINT *Point )
{
   XYGRAPH_LISTELE                     *NewElement;         // new added element
   XYGRAPH_LISTELE                     *PrevElement;        // preious element
   XYGRAPH_LISTELE                     *ScanElement;        // scanning element

   // create and fill new element
   NewElement = (XYGRAPH_LISTELE*)malloc(sizeof(XYGRAPH_LISTELE));
   NewElement->Point = *Point;
   NewElement->Next = 0;

   // when list is empty this is the first and the last
   if ( FirstElement == 0 )
      FirstElement = NewElement;
   else
   {
      // search the insertion position
      PrevElement = 0;                                      // no previous element
      ScanElement = FirstElement;                           // start from the first
      while ( ScanElement != 0 && ScanElement->Point.X < Point->X ) // bypass all element less than actual
      {
         PrevElement = ScanElement;                         // store actal as previous
         ScanElement = ScanElement->Next;                   // go to the next element
      }

      if (PrevElement == 0 )                                // if no previous is not present ..
         FirstElement = NewElement;                         // new element is the first
      else
         PrevElement->Next = NewElement;                    // new element follows the previous

      NewElement->Next = ScanElement;                       // after new element, scanning element
   }
}



/****************************************************************************

   get graph bitmap

*****************************************************************************/

void XYGraph::GetBmp( Graphics::TBitmap **PBmp )
{
   Graphics::TBitmap                   *Bmp;                // pointer to input bitmap
   TRect                               Rect;                // rectangle for graphic drawing
   int                                 InitX;               // initial posizion on x axel
   int                                 EndX;                // ending posizion on x axel
   int                                 InitY;               // initial posizion on y axel
   int                                 EndY;                // ending posizion on y axel
   double                              ScaleX;              // scale pixel for unit on X axel
   double                              ScaleY;              // scale pixel for unit on Y axel
   XYGRAPH_LISTELE                     *ScanElement;        // scanning element
   double                              Xd;                  // line x coordiante for computing
   double                              Yd;                  // line y coordiante for computing
   int                                 X;                   // line x coordiante
   int                                 Y;                   // line y coordiante
   int                                 PosGridX;            // position for X grid
   int                                 PosGridY;            // position for Y grid

   // save chart parameters
   Bmp = *PBmp;                                             // get pointer to input bitmap

   // initialize limit
   InitX = 0;
   InitY = 0;
   EndX  = Bmp->Width;
   EndY  = Bmp->Height;

   // rectangle inizialization as whole bitmap
   Rect.Top = 0;
   Rect.Left = 0;
   Rect.Right = Bmp->Width;
   Rect.Bottom = Bmp->Height;

   // draw background color
   Bmp->Canvas->Brush->Color = TColor(0x506050);
   Bmp->Canvas->FillRect(Rect);

   // draw border color
   Bmp->Canvas->Pen->Color = clBlack;
   Bmp->Canvas->Rectangle(Rect);
   InitX++;
   InitY++;
   EndX--;
   EndY--;

   // compute  scale
   ScaleX = double( EndX - InitX) / double( StopX - StartX );
   ScaleY = double( EndY - InitY - 1) / double( StopY - StartY );

   // draw grid
   Bmp->Canvas->Pen->Color = TColor(0x00a000);
   for ( PosGridX = StartX+GridX; PosGridX<StopX; PosGridX += GridX )
   {
      Xd = (PosGridX - (double)StartX) * ScaleX + 0.5;
      X = InitX + Xd;
      Bmp->Canvas->MoveTo(X, InitY);
      Bmp->Canvas->LineTo(X, EndY);
   }
   for ( PosGridY = StartY+GridY; PosGridY<StopY; PosGridY += GridY )
   {
      Yd = (PosGridY - (double)StartY) * ScaleY + 0.5;
      Y = EndY - Yd;
      Bmp->Canvas->MoveTo(InitX, Y);
      Bmp->Canvas->LineTo(EndX, Y);
   }

   ScanElement = FirstElement;
   if ( ScanElement == 0 )
      return;

   Bmp->Canvas->Pen->Color = AxisColor	;
   Xd = (ScanElement->Point.X - (double)StartX) * ScaleX + 0.5;
   Yd = (ScanElement->Point.Y - (double)StartY) * ScaleY + 0.5;
   X = InitX + Xd;
   Y = EndY - Yd;
   Bmp->Canvas->MoveTo(X, Y);

   while ( ScanElement->Next != 0 )
   {
      ScanElement = ScanElement->Next;
      Xd = (ScanElement->Point.X - (double)StartX) * ScaleX + 0.5;
      Yd = (ScanElement->Point.Y - (double)StartY) * ScaleY + 0.5;
      X = InitX + Xd;
      Y = EndY - Yd;
      Bmp->Canvas->LineTo(X, Y);
   }

}

