/******************************************************************************

	XYGraph.h | Header of XYGraphics manager

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : December 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef XYGRAPHH
#define XYGRAPHH



// system include
#include <vcl.h>
#pragma hdrstop



/******************************************************************************

	XYGraph manager class

******************************************************************************/

class XYGraph
{
   public:
      // point definition
      typedef struct
      {
         double                        X;                   // x value
         double                        Y;                   // y value
      }XYGRAPH_POINT;

      typedef struct XYGRAPH_LIST_Tag
      {
         XYGRAPH_POINT                 Point;               // point in the list
         XYGRAPH_LIST_Tag              *Next;               // pointer to the next element
      } XYGRAPH_LISTELE;

      // object constructor
                                       XYGraph( );
      // object destroyer
      virtual                          ~XYGraph();
      // set X axel
      void                             SetX( int Start, int Stop, int Grid );
      // set Y Axel
      void                             SetY( int Start, int Stop, int Grid );
      // set data color
      void                             SetColor( TColor AColor );
      // clear graph
      void                             Clear( void );
      // add a point to graph
      void                             Add( XYGRAPH_POINT *Point );
      // get graph bitmap
      void                             GetBmp( Graphics::TBitmap ** );

  protected:
      int                              StartX;              // minimum value in X axel
      int                              StopX;               // maximum value in X axel
      int                              GridX;               // grid size in X axel
      int                              StartY;              // minimum value in Y axel
      int                              StopY;               // maximum value in Y axel
      int                              GridY;               // grid size in Y axel
      XYGRAPH_LISTELE                  *FirstElement;       // first element in the list
      TColor                           AxisColor;           // axis color

};

#endif
