/******************************************************************************

	dpCCDServer.cpp | virtual dual port ram file mapped for CCD server interface

   Author : Luca Strabla (Bassano Bresciano Observatory)

   Date : May 2010

   Polypus - Astromonical observatory manager able to take care of
   unmanned observation
   Copyright (C) 2009,2010,2011  Luca Pietro Strabla
   This file is part of Polipus.
   Polypus is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License.

   Polypus is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   mail   contact: Luca Pietro Strabla via Paolo Vi 53 Palazzolo Sull'Oglio BS Italy
   e-mail contact: lucapietro.strabla@fastwebnet.it 

******************************************************************************/



// antirepeat
#ifndef dpCCDServerH
#define dpCCDServerH



// file name definition
#define        FILENAME_CCD_SERVER           "DP_CCD_SERVER"

// command definition
enum
{
	DP_COM_NULL,												// command none
   DP_COM_OPEN,											   // ccd open
   DP_COM_CLOSE,												// ccd close
   DP_COM_SHOT,												// start ccd shot
   DP_COM_ABORT,												// abort short
   DP_COM_EXIT,                                    // exit program
};



// error definition
enum
{
	DP_ERR_NONE,												// no error
   DP_ERR_CCD,													// unknown ccd
   DP_ERR_SHOT,                                    // it is not possibile shot a picture
};



// status definition
enum
{
	DP_STATUS_IDLE,											// ccd in idle
   DP_STATUS_EXPOSING,	   								// ccd exposing
};



typedef struct
{
	int					Command;							   // command code
	int					Error;								// error code
   bool              Opened;                       // ccd opened
   char              CCDName[20];                  // name of ccd
   float             ExposureTime;                 // shot exposure
   float             ExposingTime;                 // current exposing
   int               ExposingStatus;               // exposing status
   int               Width;                        // orizontal ccd size
   int               Height;                       // vertical ccd size  
} DP_CCD_SERVER;


#endif
